@@ -1,6 +1,6 @@
 // Player.cpp:  Top level SWF player, for gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -28,13 +28,17 @@
 #include "gui.h"
 #include "NullGui.h"
 
-#include "gnash.h" // still needed ?
+#include "MovieFactory.h"
 #include "movie_definition.h"
 #include "sound_handler.h" // for set_sound_handler and create_sound_handler_*
-#include "sprite_instance.h" // for setting FlashVars
+#include "MovieClip.h" // for setting FlashVars
 #include "movie_root.h" 
 #include "Player.h"
+#include "StreamProvider.h"
 
+#include "swf/TagLoadersTable.h"
+#include "swf/DefaultTagLoaders.h"
+#include "NamingPolicy.h"
 #include "StringPredicates.h"
 #include "URL.h"
 #include "rc.h"
@@ -49,12 +53,11 @@
 # include "MediaHandlerGst.h"
 #endif
 
-
+#include "GnashSystemIOHeaders.h" // for write() 
 #include "log.h"
 #include <iostream>
 #include <sstream>
 #include <iomanip>
-#include <unistd.h> // for write() on BSD
 
 using namespace gnash;
 
@@ -62,6 +65,8 @@ namespace {
 gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 }
 
+/// @todo Shouldn't Player be in 'gnash' namespace ?
+
 /*static private*/
 void
 Player::setFlashVars(const std::string& varstr)
@@ -96,7 +101,8 @@ Player::Player()
     _fpsDebugTime(0.0),
 #endif
     _hostfd(-1),
-    _startFullscreen(false)
+    _startFullscreen(false),
+    _hideMenu(false)
 {
     init();
 }
@@ -157,19 +163,39 @@ Player::init_logfile()
 
 }
 
-bool
-Player::silentStream(void* /*udata*/, boost::uint8_t* stream, int len)
+unsigned int
+Player::silentStream(void* /*udata*/, boost::int16_t* stream, unsigned int len, bool& atEOF)
 {
-    memset((void*)stream, 0, len);
-    return true;
+    std::fill(stream, stream+len, 0);
+    atEOF=false;
+    return len;
 }
 
 void
 Player::init_sound()
 {
+
     if (_doSound) {
 #ifdef SOUND_SDL
-        _soundHandler.reset( gnash::media::create_sound_handler_sdl(_audioDump) );
+        try {
+            _soundHandler.reset(sound::create_sound_handler_sdl(_audioDump));
+        } catch (SoundException& ex) {
+            log_error(_("Could not create sound handler: %s."
+                " Will continue w/out sound."), ex.what());
+        }
+        if (! _audioDump.empty()) {
+            // add a silent stream to the audio pool so that our output file
+            // is homogenous;  we actually want silent wave data when no sounds
+            // are playing on the stage
+            _soundHandler->attach_aux_streamer(silentStream, (void*) this);
+        }
+#elif defined(SOUND_AHI)
+        try {
+            _soundHandler.reset(sound::create_sound_handler_aos4(_audioDump));
+        } catch (SoundException& ex) {
+            log_error(_("Could not create sound handler: %s."
+                " Will continue w/out sound."), ex.what());
+        }
         if (! _audioDump.empty()) {
             // add a silent stream to the audio pool so that our output file
             // is homogenous;  we actually want silent wave data when no sounds
@@ -177,13 +203,11 @@ Player::init_sound()
             _soundHandler->attach_aux_streamer(silentStream, (void*) this);
         }
 #elif defined(SOUND_GST)
-        _soundHandler.reset( gnash::media::create_sound_handler_gst() );
+        _soundHandler.reset(media::create_sound_handler_gst());
 #else
         log_error(_("Sound requested but no sound support compiled in"));
         return;
 #endif
-        
-        gnash::set_sound_handler(_soundHandler.get());
     }
 }
 
@@ -191,9 +215,9 @@ void
 Player::init_media()
 {
 #ifdef USE_FFMPEG
-        _mediaHandler.reset( new gnash::media::MediaHandlerFfmpeg() );
+        _mediaHandler.reset( new gnash::media::ffmpeg::MediaHandlerFfmpeg() );
 #elif defined(USE_GST)
-        _mediaHandler.reset( new gnash::media::MediaHandlerGst() );
+        _mediaHandler.reset( new gnash::media::gst::MediaHandlerGst() );
 #else
         log_error(_("No media support compiled in"));
         return;
@@ -209,17 +233,10 @@ Player::init_gui()
     if ( _doRender )
     {
         _gui = getGui(); 
-
-        RcInitFile& rcfile = RcInitFile::getDefaultInstance();
-        if ( rcfile.startStopped() )
-        {
-            _gui->stop();
-        }
-
     }
     else
     {
-        _gui.reset(new NullGui(_doLoop));
+        _gui.reset(new NullGui(_doLoop, *_runResources));
     }
 
     _gui->setMaxAdvances(_maxAdvances);
@@ -236,6 +253,10 @@ Player::init_gui()
 boost::intrusive_ptr<movie_definition>
 Player::load_movie()
 {
+    /// The RunResources must be initialized by this point to provide resources
+    /// for parsing.
+    assert(_runResources.get());
+
     boost::intrusive_ptr<gnash::movie_definition> md;
 
     RcInitFile& rcfile = RcInitFile::getDefaultInstance();
@@ -253,8 +274,9 @@ Player::load_movie()
     try {
         if ( _infile == "-" )
         {
-            std::auto_ptr<IOChannel> in ( noseek_fd_adapter::make_stream(fileno(stdin)) );
-            md = gnash::create_movie(in, _url, false);
+            std::auto_ptr<IOChannel> in (
+                    noseek_fd_adapter::make_stream(fileno(stdin)));
+            md = MovieFactory::makeMovie(in, _url, *_runResources, false);
         }
         else
         {
@@ -265,7 +287,7 @@ Player::load_movie()
                 // We'll need to allow load of the file, no matter virtual url
                 // specified...
                 // This is kind of hackish, cleaner would be adding an argument
-                // to create_library_movie to skip the security checking phase.
+                // to createMovie to skip the security checking phase.
                 // NOTE that if we fail to allow this load, the konqueror plugin
                 // would not be able to load anything
                 //
@@ -274,7 +296,8 @@ Player::load_movie()
             }
 
             // _url should be always set at this point...
-            md = gnash::create_library_movie(url, _url.c_str(), false);
+            md = MovieFactory::makeMovie(url, *_runResources, _url.c_str(),
+                    false);
         }
     } catch (const GnashException& er) {
         std::cerr << er.what() << std::endl;
@@ -292,7 +315,8 @@ Player::load_movie()
 
 /* \brief Run, used to open a new flash file. Using previous initialization */
 int
-Player::run(int argc, char* argv[], const std::string& infile, const std::string& url)
+Player::run(int argc, char* argv[], const std::string& infile,
+        const std::string& url)
 {
     
     // Call this at run() time, so the caller has
@@ -301,14 +325,13 @@ Player::run(int argc, char* argv[], const std::string& infile, const std::string
     init_logfile();
     init_media();
     init_sound();
-    init_gui();
    
     // gnash.cpp should check that a filename is supplied.
     assert (!infile.empty());
 
     _infile = infile;
 
-    // Set base url
+    // Work out base url
     if ( _baseurl.empty() )
     {
         if (! url.empty() ) _baseurl = url;
@@ -323,6 +346,29 @@ Player::run(int argc, char* argv[], const std::string& infile, const std::string
         _url = infile;
     }
 
+    // These flags are here so we can construct
+    // the correct URL for base url later.
+    // If the URL class was not immutable we could do something smarter...
+    bool hasOverriddenBaseUrl=false;
+    std::string overriddenBaseUrl;
+
+    URL baseURL = hasOverriddenBaseUrl ? URL(overriddenBaseUrl, URL(_baseurl))
+                                       : URL(_baseurl);
+
+    /// The RunResources should be populated before parsing.
+    _runResources.reset(new RunResources(baseURL.str()));
+    _runResources->setSoundHandler(_soundHandler);
+
+    boost::shared_ptr<SWF::TagLoadersTable> loaders(new SWF::TagLoadersTable());
+    addDefaultLoaders(*loaders);
+    _runResources->setTagLoaders(loaders);
+
+    std::auto_ptr<NamingPolicy> np(new IncrementalRename(_baseurl));
+    boost::shared_ptr<StreamProvider> sp(new StreamProvider(np));
+
+    _runResources->setStreamProvider(sp);
+    
+    init_gui();
 
     // Initialize gui (we need argc/argv for this)
     // note that this will also initialize the renderer
@@ -333,15 +379,10 @@ Player::run(int argc, char* argv[], const std::string& infile, const std::string
         return EXIT_FAILURE;
     }
 
-    // Parse querystring (before FlashVars, see testsuite/misc-ming.all/FlashVarsTest*)
+    // Parse querystring (before FlashVars, see
+    // testsuite/misc-ming.all/FlashVarsTest*)
     setFlashVars(URL(_url).querystring());
-
-    // These flags are here so we can construct
-    // the correct URL for base url later.
-    // If the URL class was not immutable we could do something smarter...
-    bool hasOverriddenBaseUrl=false;
-    std::string overriddenBaseUrl;
-
+    
     // Parse parameters
     StringNoCaseEqual noCaseCompare;
     for ( std::map<std::string,std::string>::const_iterator it=params.begin(),
@@ -361,9 +402,6 @@ Player::run(int argc, char* argv[], const std::string& infile, const std::string
         }
     }
 
-    // Set base url for this movie (needed before parsing)
-    if ( hasOverriddenBaseUrl ) gnash::set_base_url(URL(overriddenBaseUrl, URL(_baseurl)));
-    else gnash::set_base_url(URL(_baseurl));
 
     // Load the actual movie.
     _movieDef = load_movie();
@@ -393,13 +431,15 @@ Player::run(int argc, char* argv[], const std::string& infile, const std::string
         if ( ! _height ) _height = 1;
     }
 
+    // Register movie definition before creating the window
+    _gui->setMovieDefinition(_movieDef.get());
+
     // Now that we know about movie size, create gui window.
     _gui->createWindow(_url.c_str(), _width, _height);
 
-    SystemClock clock; // use system clock here...
-    movie_root& root = VM::init(*_movieDef, clock).getRoot();
+    movie_root root(*_movieDef, _gui->getClock(), *_runResources);
 
-    _callbacksHandler.reset(new CallbacksHandler(_gui.get())); 
+    _callbacksHandler.reset(new CallbacksHandler(*_gui, *this)); 
     
     // Register Player to receive events from the core (Mouse, Stage,
     // System etc)
@@ -413,6 +453,18 @@ Player::run(int argc, char* argv[], const std::string& infile, const std::string
 
     _gui->setStage(&root);
 
+    // When startStopped is true, stop here after the stage has been 
+    // registered, but before the movie has started. Initial loading
+    // and VM initialization have been done by this stage, but not
+    // the complete parsing of the SWF. This is important because
+    // the Gui accesses movie_root to get the sound_handler, but also
+    // because the gui window should be properly set up by this point.
+    RcInitFile& rcfile = RcInitFile::getDefaultInstance();
+    if ( rcfile.startStopped() )
+    {
+        _gui->stop();
+    }
+
     // Start loader thread
     // NOTE: the loader thread might (in IMPORT tag parsing)
     //       create new movies and register them to the MovieLibrary.
@@ -423,8 +475,6 @@ Player::run(int argc, char* argv[], const std::string& infile, const std::string
     //       intrusive_ptr...
     _movieDef->completeLoad();
 
-    _gui->setMovieDefinition(_movieDef.get());
-
     if (! _delay) {
       //float movie_fps = _movieDef->get_frame_rate();
       //_delay = static_cast<unsigned int>(1000 / movie_fps) ; // milliseconds per frame
@@ -439,9 +489,14 @@ Player::run(int argc, char* argv[], const std::string& infile, const std::string
     if (!_windowID && _startFullscreen) {
         _gui->setFullscreen();
     }
+
+    if (!_windowID && _hideMenu) {
+        _gui->hideMenu();
+    }
+    
     _gui->run();
 
-    std::cerr << "Main loop ended, cleaning up" << std::endl;
+    log_debug("Main loop ended, cleaning up");
 
     // Clean up as much as possible, so valgrind will help find actual leaks.
     gnash::clear();
@@ -449,43 +504,64 @@ Player::run(int argc, char* argv[], const std::string& infile, const std::string
     return EXIT_SUCCESS;
 }
 
+
+void
+Player::CallbacksHandler::error(const std::string& msg)
+{
+    _gui.error(msg);
+}
+
+bool
+Player::CallbacksHandler::yesNo(const std::string& query)
+{
+    return _gui.yesno(query);
+}
+
 std::string
 Player::CallbacksHandler::call(const std::string& event, const std::string& arg)
 {
-    if (event == "Mouse.hide")
-    {
-        return _gui->showMouse(false) ? "true" : "false";
+    StringNoCaseEqual noCaseCompare;
+	
+    if (event == "Mouse.hide") {
+        return _gui.showMouse(false) ? "true" : "false";
     }
 
     if (event == "Mouse.show")
     {
-        return _gui->showMouse(true) ? "true" : "false";
+        return _gui.showMouse(true) ? "true" : "false";
     }
     
     if (event == "Stage.displayState")
     {
-        if (arg == "fullScreen") _gui->setFullscreen();
-        else if (arg == "normal") _gui->unsetFullscreen();
+        if (arg == "fullScreen") _gui.setFullscreen();
+        else if (arg == "normal") _gui.unsetFullscreen();
         return "";
     }
 
     if (event == "Stage.scaleMode" || event == "Stage.align" )
     {
-        _gui->updateStageMatrix();
+        _gui.updateStageMatrix();
         return "";
     }
-    
+
+    if (event == "Stage.showMenu")
+    {
+        if (noCaseCompare(arg, "true")) _gui.showMenu(true);
+        else if (noCaseCompare(arg, "false")) _gui.showMenu(false);
+        return "";
+    }
+
     if (event == "System.capabilities.screenResolutionX")
     {
         std::ostringstream ss;
-        ss << _gui->getScreenResX();
+        ss << _gui.getScreenResX();
         return ss.str();
     }
 
     if (event == "System.capabilities.screenResolutionY")
     {
         std::ostringstream ss;
-        ss << _gui->getScreenResY();
+        ss << _gui.getScreenResY();
         return ss.str();
     }
 
@@ -494,25 +570,25 @@ Player::CallbacksHandler::call(const std::string& event, const std::string& arg)
         std::ostringstream ss;
         // Whether the pp actively limits the precision or simply
         // gets a slightly different result isn't clear.
-        ss << std::setprecision(7) << _gui->getPixelAspectRatio();
+        ss << std::setprecision(7) << _gui.getPixelAspectRatio();
         return ss.str();
     }
 
     if (event == "System.capabilities.screenDPI")
     {
         std::ostringstream ss;
-        ss << _gui->getScreenDPI();
+        ss << _gui.getScreenDPI();
         return ss.str();
     }
 
     if (event == "System.capabilities.screenColor")
     {
-        return _gui->getScreenColor();
+        return _gui.getScreenColor();
     }
 
     if (event == "System.capabilities.playerType")
     {
-        return _gui->isPlugin() ? "PlugIn" : "StandAlone";
+        return _gui.isPlugin() ? "PlugIn" : "StandAlone";
     }
 
     log_error(_("Unhandled callback %s with arguments %s"), event, arg);
@@ -520,15 +596,16 @@ Player::CallbacksHandler::call(const std::string& event, const std::string& arg)
 }
 
 void
-Player::CallbacksHandler::notify(const std::string& command, const std::string& args)
+Player::CallbacksHandler::notify(const std::string& command,
+        const std::string& args)
 {
     //log_debug(_("fs_callback(%p): %s %s"), (void*)movie, command, args);
 
     gnash::RcInitFile& rcfile = gnash::RcInitFile::getDefaultInstance();
 
     // it's _hostfd, but we're a static method...
-    int hostfd = VM::get().getRoot().getHostFD();
-    if ( hostfd != -1 )
+    const int hostfd = _player.getHostFD();
+    if (hostfd != -1)
     {
         //log_debug("user-provided host requests fd is %d", hostfd);
         std::stringstream request;
@@ -561,7 +638,7 @@ Player::CallbacksHandler::notify(const std::string& command, const std::string&
 
     /// Fscommands can be ignored using an rcfile setting. As a 
     /// plugin they are always ignored.
-    if (_gui->isPlugin())
+    if (_gui.isPlugin())
     {
         // We log the request to the fd above
         log_debug(_("Running as plugin: skipping internal "
@@ -583,23 +660,23 @@ Player::CallbacksHandler::notify(const std::string& command, const std::string&
     // FSCommand quit
     if (noCaseCompare(command, "quit"))
     {
-        _gui->quit();
+        _gui.quit();
         return;
     }
 
     // FSCommand fullscreen
     if (noCaseCompare(command, "fullscreen"))
     {
-        if (noCaseCompare(args, "true")) _gui->setFullscreen();
-        else if (noCaseCompare(args, "false")) _gui->unsetFullscreen();
+        if (noCaseCompare(args, "true")) _gui.setFullscreen();
+        else if (noCaseCompare(args, "false")) _gui.unsetFullscreen();
         return;
     }
        
     // FSCommand showmenu
     if (noCaseCompare(command, "showmenu"))
     {
-        if (noCaseCompare(args, "true")) _gui->showMenu(true);
-        else if (noCaseCompare(args, "false")) _gui->showMenu(false);
+        if (noCaseCompare(args, "true")) _gui.showMenu(true);
+        else if (noCaseCompare(args, "false")) _gui.showMenu(false);
         return;
     }
 
@@ -617,11 +694,11 @@ Player::CallbacksHandler::notify(const std::string& command, const std::string&
     if (noCaseCompare(command, "allowscale"))
     {
         //log_debug("allowscale: %s", args);
-        if (noCaseCompare(args, "true")) _gui->allowScale(true);
+        if (noCaseCompare(args, "true")) _gui.allowScale(true);
         else
         {
-                if (strtol(args.c_str(), NULL, 0)) _gui->allowScale(true);
-                else _gui->allowScale(false);
+                if (strtol(args.c_str(), NULL, 0)) _gui.allowScale(true);
+                else _gui.allowScale(false);
         }
         return;
     }
@@ -646,44 +723,53 @@ std::auto_ptr<Gui>
 Player::getGui()
 {
 #ifdef GUI_GTK
-    return createGTKGui(_windowID, _scale, _doLoop, _bitDepth);
+    return createGTKGui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
-#ifdef GUI_KDE
-    return createKDEGui(_windowID, _scale, _doLoop, _bitDepth);
+#ifdef GUI_KDE3
+    return createKDEGui(_windowID, _scale, _doLoop, *_runResources);
+#endif
+
+#ifdef GUI_KDE4
+    return createKDE4Gui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
 #ifdef GUI_SDL
-    return createSDLGui(_windowID, _scale, _doLoop, _bitDepth);
+    return createSDLGui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
 #ifdef GUI_AQUA
-    return createAQUAGui(_windowID, _scale, _doLoop, _bitDepth);
+    return createAQUAGui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
 #ifdef GUI_RISCOS
-    return createRISCOSGui(_windowID, _scale, _doLoop, _bitDepth);
+    return createRISCOSGui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
 #ifdef GUI_FLTK
-    return createFLTKGui(_windowID, _scale, _doLoop, _bitDepth);
+    return createFLTKGui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
 #ifdef GUI_FB
-    return createFBGui(_windowID, _scale, _doLoop, _bitDepth);
+    return createFBGui(_windowID, _scale, _doLoop, *_runResources);
+#endif
+
+#ifdef GUI_AOS4
+    return createAOS4Gui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
 #ifdef GUI_DUMP
-    return createDumpGui(_windowID, _scale, _doLoop, _bitDepth);
+    return createDumpGui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
-    return std::auto_ptr<Gui>(new NullGui(_doLoop));
+    return std::auto_ptr<Gui>(new NullGui(_doLoop, *_runResources));
 }
 
 Player::~Player()
 {
     if (_movieDef.get())
     {
-            log_debug("~Player - _movieDef refcount: %d (1 will be dropped now)", _movieDef->get_ref_count());
+        log_debug("~Player - _movieDef refcount: %d (1 will be dropped "
+                "now)", _movieDef->get_ref_count());
     }
 }
