@@ -87,9 +87,10 @@
 #include "fbsup.h"
 #include "log.h"
 #include "movie_root.h"
+#include "RunResources.h"
 
-#include "render_handler.h"
-#include "render_handler_agg.h"
+#include "Renderer.h"
+#include "Renderer_agg.h"
 #include "GnashSleep.h" // for gnashSleep
 
 #include <linux/input.h>    // for /dev/input/event*
@@ -164,12 +165,8 @@ void terminate_signal(int /*signo*/) {
 
 //---------------
 
-FBGui::FBGui() : Gui()
-{
-}
-
-FBGui::FBGui(unsigned long xid, float scale, bool loop, unsigned int depth)
-  : Gui(xid, scale, loop, depth)
+FBGui::FBGui(unsigned long xid, float scale, bool loop, RunResources& r)
+  : Gui(xid, scale, loop, r)
 {
   fd      = -1;
   fbmem   = NULL;
@@ -299,7 +296,7 @@ bool FBGui::initialize_renderer() {
   int _bpp = var_screeninfo.bits_per_pixel;
   int _size = fix_screeninfo.smem_len;   // TODO: should recalculate!  
   unsigned char *_mem;
-  render_handler_agg_base *agg_handler;
+  Renderer_agg_base *agg_handler;
   
   m_stage_width = _width;
   m_stage_height = _height;
@@ -336,16 +333,16 @@ bool FBGui::initialize_renderer() {
   );
 
   if (pixelformat) {    
-    agg_handler = create_render_handler_agg(pixelformat);      
+    agg_handler = create_Renderer_agg(pixelformat);      
   } else {
     fatal_error("The pixel format of your framebuffer could not be detected.");
     return false;
   }
 
   assert(agg_handler!=NULL);
-  _renderer = agg_handler;
+  _renderer.reset(agg_handler);
 
-  set_render_handler(agg_handler);
+  _runResources.setRenderer(boost::shared_ptr<Renderer>(_renderer));
   
   m_rowsize = var_screeninfo.xres_virtual*((_bpp+7)/8);
   
@@ -360,7 +357,6 @@ bool FBGui::run()
 {
   struct timeval tv;
 
-  double timer = 0.0;
   double start_timer;
   
   if (!gettimeofday(&tv, NULL))
@@ -374,22 +370,21 @@ bool FBGui::run()
 
   while (!terminate_request) {
   
-    double prevtimer = timer; 
-    
-    while ((timer-prevtimer)*1000 < _interval) {
-    
-      gnashSleep(1); // task switch
-      
-      check_mouse(); // TODO: Exit delay loop on mouse events! 
-      check_keyboard(); // TODO: Exit delay loop on keyboard events!
-            
-      if (!gettimeofday(&tv, NULL))
-        timer = (double)tv.tv_sec + (double)tv.tv_usec / 1000000.0;
-    }
+    // wait the "heartbeat" inteval
+    gnashSleep(_interval * 1000);    
+    // TODO: Do we need to check the real time slept or is it OK when we woke
+    // up early because of some Linux signal sent to our process (and thus
+    // "advance" faster than the "heartbeat" interval)? - Udo
   
-  // advance movie  
-  Gui::advance_movie(this);
+    // check input devices
+    check_mouse();
+    check_keyboard();
+  
+    // advance movie  
+    Gui::advance_movie(this);
+    
   }
+  
   return true;
 }
 
@@ -842,9 +837,9 @@ bool FBGui::init_mouse()
 #endif
 
 #ifdef USE_MOUSE_PS2    
-void FBGui::check_mouse() 
+bool FBGui::check_mouse() 
 {
-  if (input_fd<0) return;   // no mouse available
+  if (input_fd<0) return false;   // no mouse available
   
   int i;
   int xmove, ymove, btn, btn_changed;
@@ -858,7 +853,7 @@ void FBGui::check_mouse()
     pos = i;
     break;    
   }
-  if (pos<0) return; // no sync or no data
+  if (pos<0) return false; // no sync or no data
   
   if (pos>0) {
     // remove garbage:
@@ -906,6 +901,7 @@ void FBGui::check_mouse()
     memmove(mouse_buf, mouse_buf + pos, mouse_buf_size - pos);
     mouse_buf_size -= pos;  
     
+    return true;
   
   }
   
@@ -943,9 +939,11 @@ bool FBGui::init_mouse()
 #endif
 
 #ifdef USE_MOUSE_ETT    
-void FBGui::check_mouse() 
+bool FBGui::check_mouse() 
 {
-  if (input_fd<0) return;   // no mouse available
+  bool activity = false;
+  
+  if (input_fd<0) return false;   // no mouse available
   
   read_mouse_data();
   
@@ -957,7 +955,7 @@ void FBGui::check_mouse()
     pos = i;
     break;    
   }
-  if (pos<0) return; // no sync or no data
+  if (pos<0) return false; // no sync or no data
   
   if (pos>0) {
     //printf("touchscreen: removing %d bytes garbage!\n", pos);  
@@ -996,11 +994,13 @@ void FBGui::check_mouse()
       mouse_x = new_x;
       mouse_y = new_y;
       notify_mouse_moved(mouse_x, mouse_y);
+      activity = true;
     }
     
     if (new_btn != mouse_btn) {
       mouse_btn = new_btn;      
       notify_mouse_clicked(mouse_btn, 1);  // mask=?
+      activity = true;
     }
     
     // remove from buffer
@@ -1009,6 +1009,8 @@ void FBGui::check_mouse()
     mouse_buf_size -= pos;    
   }
   
+  return activity;
+  
 }
 #endif
 
@@ -1134,10 +1136,11 @@ void FBGui::apply_ts_calibration(float* cx, float* cy, int rawx, int rawy) {
   *cy = (rawy-cal1y) / (cal2y-cal1y) * (ref2y-ref1y) + ref1y;
 }
 
-void FBGui::check_mouse()
+bool FBGui::check_mouse()
 {
-
-  if (input_fd < 0) return;
+  bool activity = false;
+  
+  if (input_fd < 0) return false;
 
   struct input_event ev;  // time,type,code,value
   
@@ -1188,11 +1191,13 @@ void FBGui::check_mouse()
       
         if (move_pending) {
           notify_mouse_moved(notify_x, notify_y);
+          activity = true;
           move_pending = false;
         }
       
         mouse_btn = new_mouse_btn;
         notify_mouse_clicked(mouse_btn, 1);  // mark=??
+        activity = true;
       }
 
       if (coordinatedebug)
@@ -1225,8 +1230,12 @@ void FBGui::check_mouse()
   
   } 
   
-  if (move_pending) 
+  if (move_pending) {
     notify_mouse_moved(notify_x, notify_y);
+    activity = true;
+  }
+  
+  return activity;
  
 } //check_mouse
 #endif
@@ -1335,13 +1344,11 @@ gnash::key::code FBGui::scancode_to_gnash_key(int code, bool shift) {
     case KEY_KP8    : return gnash::key::KP_8; 
     case KEY_KP9    : return gnash::key::KP_9;
 
-    /*    
-    case KEY_KPMINUS       : return gnash::key::;
-    case KEY_KPPLUS        : return gnash::key::;
-    case KEY_KPDOT         : return gnash::key::;
-    case KEY_KPASTERISK    : return gnash::key::;
-    case KEY_KPENTER       : return gnash::key::;
-    */
+    case KEY_KPMINUS       : return gnash::key::KP_SUBTRACT;
+    case KEY_KPPLUS        : return gnash::key::KP_ADD;
+    case KEY_KPDOT         : return gnash::key::KP_DECIMAL;
+    case KEY_KPASTERISK    : return gnash::key::KP_MULITPLY;  // typo in GnashKey.h
+    case KEY_KPENTER       : return gnash::key::KP_ENTER;
     
     case KEY_ESC           : return gnash::key::ESCAPE;
     case KEY_MINUS         : return gnash::key::MINUS;
@@ -1358,13 +1365,12 @@ gnash::key::code FBGui::scancode_to_gnash_key(int code, bool shift) {
     case KEY_LEFTSHIFT     : return gnash::key::SHIFT;
     case KEY_BACKSLASH     : return gnash::key::BACKSLASH;
     case KEY_COMMA         : return gnash::key::COMMA;
-    //case KEY_DOT           : return gnash::key::DOT;
     case KEY_SLASH         : return gnash::key::SLASH;
     case KEY_RIGHTSHIFT    : return gnash::key::SHIFT;
     case KEY_LEFTALT       : return gnash::key::ALT;
     case KEY_SPACE         : return gnash::key::SPACE;
     case KEY_CAPSLOCK      : return gnash::key::CAPSLOCK;
-    //case KEY_NUMLOCK       : return gnash::key::NUMLOCK;
+    case KEY_NUMLOCK       : return gnash::key::NUM_LOCK;
     //case KEY_SCROLLLOCK    : return gnash::key::SCROLLLOCK;
     
     case KEY_UP            : return gnash::key::UP;
@@ -1383,10 +1389,11 @@ gnash::key::code FBGui::scancode_to_gnash_key(int code, bool shift) {
   return gnash::key::INVALID;  
 }
 
-void FBGui::check_keyboard()
+bool FBGui::check_keyboard()
 {
-
-  if (keyb_fd < 0) return;
+  bool activity = false;
+  
+  if (keyb_fd < 0) return false;
 
   struct input_event ev;  // time,type,code,value
 
@@ -1438,14 +1445,18 @@ void FBGui::check_keyboard()
           
           
         // send event
-        if (c != gnash::key::INVALID) 
-            Gui::notify_key_event(c, modifier, ev.value);
+        if (c != gnash::key::INVALID) {
+          Gui::notify_key_event(c, modifier, ev.value);
+          activity=true;
+        }
               
       } //if normal key
 
     } //if EV_KEY      
   
   } //while
+  
+  return activity;
 
 }
 
