@@ -28,9 +28,11 @@
 #include "rc.h"
 #include "gtksup.h"
 #include "sound_handler.h"
-#include "render_handler.h"
+#include "Renderer.h"
+#include "RunResources.h"
 #include "VM.h"
 #include "lirc.h"
+#include "gnash.h" // Quality
 
 #include <iostream>
 #ifdef HAVE_X11
@@ -42,6 +44,10 @@
 #include <gdk/gdkkeysyms.h>
 #include <string>
 
+#ifdef BUILD_CANVAS
+#include "gtk_canvas.h"
+#endif
+
 #ifdef HAVE_FFMPEG_AVCODEC_H
 extern "C" {
 # include "ffmpeg/avcodec.h" // Only for the version number
@@ -58,21 +64,6 @@ extern "C" {
 # include "gst/gstversion.h" // Only for the version number
 #endif
 
-#ifdef RENDERER_OPENGL
-#include "gtk_glue_gtkglext.h"
-#endif
-
-#ifdef RENDERER_CAIRO
-#include "gtk_glue_cairo.h"
-#endif
-
-#ifdef RENDERER_AGG
-#include "gtk_glue_agg.h"
-#ifdef HAVE_XV
-#include "gtk_glue_agg_xv.h"
-#endif // HAVE_XV
-#endif
-
 #ifdef GUI_HILDON
 # include <hildon/hildon.h>
 #endif
@@ -98,12 +89,14 @@ namespace {
     void menuMovieInfo(GtkMenuItem *menuitem, gpointer instance);
     void menuRefreshView(GtkMenuItem *menuitem, gpointer instance);
     void menuShowUpdatedRegions(GtkMenuItem *menuitem, gpointer instance); 
+    void menuQualityLow(GtkMenuItem *menuitem, gpointer instance); 
+    void menuQualityMedium(GtkMenuItem *menuitem, gpointer instance); 
+    void menuQualityHigh(GtkMenuItem *menuitem, gpointer instance); 
+    void menuQualityBest(GtkMenuItem *menuitem, gpointer instance); 
 
     // Event handlers
     gboolean realizeEvent(GtkWidget *widget, GdkEvent *event, gpointer data);
     gboolean deleteEvent(GtkWidget *widget, GdkEvent *event, gpointer data);
-    gboolean exposeEvent(GtkWidget *widget, GdkEventExpose *event,
-                                 gpointer data);
     gboolean configureEvent(GtkWidget *widget, GdkEventConfigure *event,
                                     gpointer data);
     gboolean keyPressEvent(GtkWidget *widget, GdkEventKey *event,
@@ -118,6 +111,8 @@ namespace {
                                         gpointer data);
     gint popupHandler(GtkWidget *widget, GdkEvent *event);    
 
+    gint popupHandlerAlt(GtkWidget *widget, GdkEvent *event);    
+
     void openFile(GtkWidget *widget, gpointer data);
 
     void addPixmapDirectory(const gchar *directory);
@@ -132,6 +127,9 @@ namespace {
 
     int gdk_to_gnash_modifier(int key);
 
+	//for use in popupHandler
+	bool _showMenuState;
+
 }
 
 // This is global so it can be accessed by the evnt handler, which
@@ -143,20 +141,20 @@ GtkGui::~GtkGui()
 {
 }
 
-GtkGui::GtkGui(unsigned long xid, float scale, bool loop, unsigned int depth)
+GtkGui::GtkGui(unsigned long xid, float scale, bool loop, RunResources& r)
 	:
-	Gui(xid, scale, loop, depth)
+	Gui(xid, scale, loop, r)
 #ifdef GUI_HILDON
 	,_hildon_program(0)
 #endif
 	,_window(0)
 	,_resumeButton(0)
 	,_overlay(0)
-	,_drawingArea(0)
+	,_canvas(0)
 	,_popup_menu(0)
+    ,_popup_menu_alt(0)
 	,_menubar(0)
 	,_vbox(0)
-	,_glue()
 	,_advanceSourceTimer(0)
 {
 }
@@ -170,34 +168,6 @@ GtkGui::init(int argc, char **argv[])
 #ifdef GUI_HILDON
     _hildon_program = hildon_program_get_instance();
 #endif
-    
-    // TODO: don't rely on a macro to select renderer
-#ifdef RENDERER_CAIRO
-    _glue.reset(new GtkCairoGlue);
-#elif defined(RENDERER_OPENGL)
-    _glue.reset(new GtkGlExtGlue);
-#elif defined(RENDERER_AGG) && !defined(HAVE_XV)
-    _glue.reset(new GtkAggGlue);
-#elif defined(RENDERER_AGG) && defined(HAVE_XV)
-    RcInitFile& rcfile = RcInitFile::getDefaultInstance();
-
-    if (rcfile.useXv()) {
-        _glue.reset(new GtkAggXvGlue);
-        if (!_glue->init (argc, argv)) {
-            _glue.reset(new GtkAggGlue);
-            _glue->init(argc, argv);
-        }
-    } else {
-        _glue.reset(new GtkAggGlue);
-        _glue->init(argc, argv);
-    }
-#endif
-
-#if ! (defined(HAVE_XV) && defined(RENDERER_AGG))
-    if (!_glue->init (argc, argv)) {
-        return false;
-    }
-#endif
 
     addPixmapDirectory (PKGDATADIR);
 
@@ -216,11 +186,18 @@ GtkGui::init(int argc, char **argv[])
     
     addGnashIcon(GTK_WINDOW(_window));
 
+#ifdef BUILD_CANVAS
+    _canvas = gnash_canvas_new();
+    gnash_canvas_setup(GNASH_CANVAS(_canvas), argc, argv);
+    // Increase reference count to prevent its destruction (which could happen
+    // later if we remove it from its container).
+    g_object_ref(G_OBJECT(_canvas));
+#else
     _drawingArea = gtk_drawing_area_new();
-
     // Increase reference count to prevent its destruction (which could happen
     // later if we remove it from its container).
     g_object_ref(G_OBJECT(_drawingArea));
+#endif
 
     _resumeButton = gtk_button_new();
     gtk_container_add(GTK_CONTAINER(_resumeButton),
@@ -233,18 +210,8 @@ GtkGui::init(int argc, char **argv[])
     // This callback indirectly results in playHook() being called.
     g_signal_connect(_resumeButton, "clicked", G_CALLBACK(menuPlay), this);
 
-    // If we don't set this flag we won't be able to grab focus
-    // ( grabFocus() would be a no-op )
-    GTK_WIDGET_SET_FLAGS (GTK_WIDGET(_drawingArea), GTK_CAN_FOCUS);
-
     createMenu();
-
-#ifdef RENDERER_OPENGL
-    // OpenGL _glue needs to prepare the drawing area for OpenGL rendering before
-    // widgets are realized and before the configure event is fired.
-    // TODO: find a way to make '_glue' use independent from actual renderer in use
-    _glue->prepDrawingArea(_drawingArea);
-#endif
+    createMenuAlt();
 
     // A vertical box is used to allow display of the menu bar and paused widget
     _vbox = gtk_vbox_new(FALSE, 0);
@@ -257,19 +224,21 @@ GtkGui::init(int argc, char **argv[])
     }
 #endif
 
+#ifdef BUILD_CANVAS
+    gtk_box_pack_start(GTK_BOX(_vbox), _canvas, TRUE, TRUE, 0);
+#else
     gtk_box_pack_start(GTK_BOX(_vbox), _drawingArea, TRUE, TRUE, 0);
+#endif
 
     setupEvents();
 
     gtk_widget_realize(_window);
+#ifdef BUILD_CANVAS
+    gtk_widget_show(_canvas);
+#else
     gtk_widget_show(_drawingArea);
-    gtk_widget_show(_window);
-
-#if defined(RENDERER_CAIRO) || defined(RENDERER_AGG)
-    // cairo needs the _drawingArea.window to prepare it ..
-    // TODO: find a way to make '_glue' use independent from actual renderer in use
-    _glue->prepDrawingArea(_drawingArea);
 #endif
+    gtk_widget_show(_window);
     
 #ifdef USE_LIRC
     lirc = new Lirc();
@@ -280,10 +249,14 @@ GtkGui::init(int argc, char **argv[])
         log_debug(_("LIRC daemon not running"));
     }
 #endif
-    
-    _renderer = _glue->createRenderHandler();
-    if ( ! _renderer ) return false;
-    set_render_handler(_renderer);
+
+#ifdef BUILD_CANVAS
+    _renderer = gnash_canvas_get_renderer(GNASH_CANVAS(_canvas));
+#else
+    _renderer.reset(_glue->createRenderHandler());
+    if (!_renderer.get()) return false;
+#endif
+    _runResources.setRenderer(_renderer);
 
     // The first time stop() was called, stopHook() might not have had a chance
     // to do anything, because GTK+ wasn't garanteed to be initialised.
@@ -360,7 +333,7 @@ GtkGui::error(const std::string& msg)
     gtk_label_set_line_wrap(GTK_LABEL(label), true);
     gtk_box_pack_start(GTK_BOX(content), label, false, false, 0);
     gtk_widget_show_all(popup);
-}
+} 
 
 void
 GtkGui::setFullscreen()
@@ -396,7 +369,7 @@ GtkGui::setFullscreen()
         // It could be a good hack if it were done earlier.
         // There really doesn't seem to be a proper way of setting the
         // starting size of a widget but allowing it to be shrunk.
-        gtk_widget_set_size_request(_drawingArea, -1, -1);
+        gtk_widget_set_size_request(_canvas, -1, -1);
     	gtk_window_fullscreen(GTK_WINDOW(_window));
 
         showMenu(false);
@@ -467,7 +440,7 @@ GtkGui::setCursor(gnash_cursor_type newcursor)
     }
 
     // The parent of _drawingArea is different for the plugin in fullscreen
-    gdk_window_set_cursor(_drawingArea->window, gdkcursor);
+    gdk_window_set_cursor(_canvas->window, gdkcursor);
   
     if (gdkcursor) {
         gdk_cursor_unref(gdkcursor);
@@ -493,7 +466,7 @@ GtkGui::showMouse(bool show)
         GdkCursor* cursor = gdk_cursor_new_from_pixmap(pixmap, pixmap,
                                                     color, color, 0, 0);
 
-        gdk_window_set_cursor (_drawingArea->window, cursor);
+        gdk_window_set_cursor (_canvas->window, cursor);
 
         g_free(color);
         g_object_unref(pixmap);	
@@ -510,10 +483,25 @@ GtkGui::showMouse(bool show)
 void
 GtkGui::showMenu(bool show)
 {
+    RcInitFile& rcfile = RcInitFile::getDefaultInstance();
+
+    // If we allow the swf author to set Stage.showMenu
+    if( !rcfile.ignoreShowMenu() ) {
+		_showMenuState = show;
+	}
+
 #ifdef USE_MENUS
-    if (!_menubar) return;
-    if (show) gtk_widget_show(_menubar);
+	if (!_menubar) {
+		return;
+	}
+
+    if (show) {
+		gtk_widget_show(_menubar);
+	} else {
+		gtk_widget_hide(_menubar);
+	}
 #endif
+	
 }
 
 double
@@ -557,11 +545,11 @@ GtkGui::getScreenDPI()
 void
 GtkGui::setupWindowEvents()
 {
-    g_signal_connect(gtk_widget_get_toplevel(_drawingArea),
+    g_signal_connect(gtk_widget_get_toplevel(_canvas),
             "delete_event", G_CALLBACK(deleteEvent), this);
-    g_signal_connect(gtk_widget_get_toplevel(_drawingArea),
+    g_signal_connect(gtk_widget_get_toplevel(_canvas),
             "key_press_event", G_CALLBACK(keyPressEvent), this);
-    g_signal_connect(gtk_widget_get_toplevel(_drawingArea),
+    g_signal_connect(gtk_widget_get_toplevel(_canvas),
             "key_release_event", G_CALLBACK(keyReleaseEvent), this);
 }
 
@@ -572,29 +560,35 @@ GtkGui::setupEvents()
 
     setupWindowEvents();
 
-    gtk_widget_add_events(_drawingArea, GDK_EXPOSURE_MASK
+    gtk_widget_add_events(_canvas, GDK_EXPOSURE_MASK
                         | GDK_BUTTON_PRESS_MASK
                         | GDK_BUTTON_RELEASE_MASK
                         | GDK_KEY_RELEASE_MASK
                         | GDK_KEY_PRESS_MASK        
                         | GDK_POINTER_MOTION_MASK);
   
-    g_signal_connect_swapped(_drawingArea, "button_press_event",
+	_showMenuState = true; //Default for showMenu
+
+    g_signal_connect_swapped(_canvas, "button_press_event",
                             G_CALLBACK(popupHandler), _popup_menu);
   
-    g_signal_connect(_drawingArea, "button_press_event",
+    g_signal_connect_swapped(_canvas, "button_press_event",
+                            G_CALLBACK(popupHandlerAlt), _popup_menu_alt);
+
+    g_signal_connect(_canvas, "button_press_event",
                    G_CALLBACK(buttonPressEvent), this);
-    g_signal_connect(_drawingArea, "button_release_event",
+    g_signal_connect(_canvas, "button_release_event",
                    G_CALLBACK(buttonReleaseEvent), this);
-    g_signal_connect(_drawingArea, "motion_notify_event",
+    g_signal_connect(_canvas, "motion_notify_event",
                    G_CALLBACK(motionNotifyEvent), this);
   
-    g_signal_connect_after(_drawingArea, "realize",
+    g_signal_connect_after(_canvas, "realize",
                          G_CALLBACK (realizeEvent), NULL);
-    g_signal_connect(_drawingArea, "configure_event",
+
+    // connect_after because we are going to cause a rendering and the canvas
+    // widget should have had a chance to update the size of the render area
+    g_signal_connect_after(_canvas, "configure_event",
                    G_CALLBACK (configureEvent), this);
-    g_signal_connect(_drawingArea, "expose_event",
-                    G_CALLBACK (exposeEvent), this);
 
     return true;
 }
@@ -602,7 +596,7 @@ GtkGui::setupEvents()
 void
 GtkGui::grabFocus()
 {
-    gtk_widget_grab_focus(GTK_WIDGET(_drawingArea));
+    gtk_widget_grab_focus(GTK_WIDGET(_canvas));
 }
 
 void
@@ -611,14 +605,11 @@ GtkGui::quit()
     gtk_main_quit();
 }
 
+/*private*/
 void
-GtkGui::setInterval(unsigned int interval)
+GtkGui::startAdvanceTimer()
 {
-    _interval = interval;
-    if (_advanceSourceTimer)
-    {
-        g_source_remove(_advanceSourceTimer);
-    }
+    stopAdvanceTimer();
     
     _advanceSourceTimer = g_timeout_add_full(G_PRIORITY_LOW, _interval,
             (GSourceFunc)advance_movie, this, NULL);
@@ -627,6 +618,25 @@ GtkGui::setInterval(unsigned int interval)
             _interval, 1000/_interval);
 }
 
+/*private*/
+void
+GtkGui::stopAdvanceTimer()
+{
+    if (_advanceSourceTimer)
+    {
+        g_source_remove(_advanceSourceTimer);
+        _advanceSourceTimer = 0;
+    }
+}
+
+void
+GtkGui::setInterval(unsigned int interval)
+{
+    _interval = interval;
+
+    if ( ! isStopped() ) startAdvanceTimer();
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////
 ///                                                                         ///
@@ -675,9 +685,9 @@ GtkGui::createMenu()
 #endif
     createHelpMenu(GTK_WIDGET(_popup_menu));
 
-    GtkWidget *separator1 = gtk_separator_menu_item_new ();
-    gtk_widget_show (separator1);
-    gtk_container_add (GTK_CONTAINER (_popup_menu), separator1);
+    GtkWidget *separator1 = gtk_separator_menu_item_new();
+    gtk_widget_show(separator1);
+    gtk_container_add (GTK_CONTAINER(_popup_menu), separator1);
 
     /// The sound handler is initialized after the Gui is created, and
     /// may be disabled or enabled dynamically.
@@ -686,15 +696,16 @@ GtkGui::createMenu()
     gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menusound), TRUE);
     gtk_menu_append(_popup_menu, GTK_WIDGET(menusound));
     gtk_widget_show(GTK_WIDGET(menusound));
-    g_signal_connect(menusound, "activate",
-                     G_CALLBACK(menusound), this);
+    g_signal_connect(menusound, "activate", G_CALLBACK(menuSound), this);
 
-    GtkMenuItem *menuquit =
- 	GTK_MENU_ITEM(gtk_menu_item_new_with_label(_("Quit Gnash")));
-    gtk_menu_append(_popup_menu, GTK_WIDGET(menuquit));
-    gtk_widget_show(GTK_WIDGET(menuquit));
-    g_signal_connect(menuquit, "activate",
-                     G_CALLBACK(menuquit), this);
+    GtkWidget *separator2 = gtk_separator_menu_item_new();
+    gtk_widget_show(separator2);
+    gtk_container_add (GTK_CONTAINER(_popup_menu), separator2);
+
+    GtkWidget *quit = gtk_image_menu_item_new_from_stock("gtk-quit", 0);
+    gtk_widget_show(quit);
+    gtk_container_add(GTK_CONTAINER(_popup_menu), quit);
+    g_signal_connect(quit, "activate", G_CALLBACK(menuQuit), this);
 
 #ifdef GUI_HILDON
      hildon_window_set_menu(HILDON_WINDOW(_window),
@@ -705,6 +716,38 @@ GtkGui::createMenu()
     return true;
 }
 
+bool
+GtkGui::createMenuAlt()
+{
+    //An alternative short version of the popup menu
+
+    _popup_menu_alt = GTK_MENU(gtk_menu_new());
+
+#ifdef USE_MENUS
+    // If menus are disabled, these are not added to the popup menu
+    // either.
+    createEditMenu(GTK_WIDGET(_popup_menu_alt));
+#endif
+    createHelpMenu(GTK_WIDGET(_popup_menu_alt));
+
+    GtkWidget *separator1 = gtk_separator_menu_item_new();
+    gtk_widget_show(separator1);
+    gtk_container_add (GTK_CONTAINER(_popup_menu_alt), separator1);
+
+    GtkWidget *quit = gtk_image_menu_item_new_from_stock("gtk-quit", 0);
+    gtk_widget_show(quit);
+    gtk_container_add(GTK_CONTAINER(_popup_menu_alt), quit);
+    g_signal_connect(quit, "activate", G_CALLBACK(menuQuit), this);
+
+#ifdef GUI_HILDON
+     hildon_window_set_menu(HILDON_WINDOW(_window),
+                               GTK_MENU(_popup_menu_alt));
+     gtk_widget_show_all(GTK_WIDGET(_popup_menu_alt));   
+#endif
+
+    return true;
+}
+
 bool
 GtkGui::createWindow(const char *title, int width, int height)
 {
@@ -720,7 +763,7 @@ GtkGui::createWindow(const char *title, int width, int height)
         // Advantage: The window is sized correctly, no matter what other
         // widgets are visible
         // Disadvantage: The window cannot be shrinked, which is bad.   
-        gtk_widget_set_size_request(_drawingArea, width, height);
+        gtk_widget_set_size_request(_canvas, width, height);
 
     }
     return ret;
@@ -817,7 +860,6 @@ GtkGui::createWindow(int width, int height)
     _height = height;
     
     _validbounds.setTo(0, 0, _width, _height);
-    _glue->setRenderHandlerSize(_width, _height);
     
     return true;
 }
@@ -825,34 +867,13 @@ GtkGui::createWindow(int width, int height)
 void
 GtkGui::beforeRendering()
 {
-    _glue->beforeRendering();
+    gnash_canvas_before_rendering(GNASH_CANVAS(_canvas));
 }
 
 void
 GtkGui::renderBuffer()
 {
-    gdk_window_process_updates(_drawingArea->window, false);
-}
-
-void
-GtkGui::expose(const GdkRegion *region) 
-{
-    gint num_rects;
-    GdkRectangle* rects;
-
-    // In some versions of GTK this can't be const...
-    GdkRegion* nonconst_region = const_cast<GdkRegion*>(region);
-
-    gdk_region_get_rectangles (nonconst_region, &rects, &num_rects);
-    assert(num_rects);
-
-    for (int i=0; i<num_rects; ++i) {
-      const GdkRectangle& cur_rect = rects[i];
-      _glue->render(cur_rect.x, cur_rect.y, cur_rect.x + cur_rect.width,
-                    cur_rect.y + cur_rect.height);
-    }
-
-    g_free(rects);
+    gdk_window_process_updates(_canvas->window, false);
 }
 
 void
@@ -895,7 +916,7 @@ GtkGui::setInvalidatedRegions(const InvalidatedRanges& ranges)
         // We add the rectangle to the part of the window to be redrawn
         // (also known as the "clipping" or "damaged" area). in renderBuffer(),
         // we force a redraw.
-        gdk_window_invalidate_rect(_drawingArea->window, &rect, false);
+        gdk_window_invalidate_rect(_canvas->window, &rect, false);
     }
 
 }
@@ -1767,6 +1788,11 @@ GtkGui::showAboutDialog()
 	"Benjamin Wolsey",
 	"Russ Nelson",
 	"Dossy Shiobara",
+	"Jonathan Crider",
+	"Ben Limmer",
+	"Bob Naugle",
+	"Si Liu",
+	"Sharad Desai",
 	NULL
     };
 
@@ -1926,7 +1952,6 @@ GtkGui::createFileMenu(GtkWidget *obj)
     GtkWidget *quit = gtk_image_menu_item_new_from_stock("gtk-quit", 0);
     gtk_widget_show(quit);
     gtk_container_add(GTK_CONTAINER(menu), quit);
-
     g_signal_connect(quit, "activate", G_CALLBACK(menuQuit), this);
 }
 
@@ -2018,6 +2043,51 @@ GtkGui::createViewMenu(GtkWidget *obj)
                      G_CALLBACK(menuShowUpdatedRegions), this);
 #endif
 
+    createQualityMenu(menu);
+
+}
+
+// Create a Quality menu that can be used from the View menu
+void
+GtkGui::createQualityMenu(GtkWidget *obj)
+{
+    GNASH_REPORT_FUNCTION;
+
+    GtkWidget *menuitem = gtk_menu_item_new_with_mnemonic (_("_Quality"));
+    gtk_widget_show (menuitem);
+    gtk_container_add (GTK_CONTAINER (obj), menuitem);
+    
+    GtkWidget *menu = gtk_menu_new ();
+    gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menu);
+
+    Quality curQuality = getQuality();
+
+    // TODO: use to also show current quality state
+
+    // Low
+    GtkWidget* item = gtk_menu_item_new_with_label(_("Low"));
+    gtk_menu_append(menu, item);
+    gtk_widget_show(item);
+    g_signal_connect(item, "activate", G_CALLBACK(menuQualityLow), this);
+
+    // Medium
+    item = gtk_menu_item_new_with_label(_("Medium"));
+    gtk_menu_append(menu, item);
+    gtk_widget_show(item);
+    g_signal_connect(item, "activate", G_CALLBACK(menuQualityMedium), this);
+
+    // High
+    item = gtk_menu_item_new_with_label(_("High"));
+    gtk_menu_append(menu, item);
+    gtk_widget_show(item);
+    g_signal_connect(item, "activate", G_CALLBACK(menuQualityHigh), this);
+
+    // Best
+    item = gtk_menu_item_new_with_label(_("Best"));
+    gtk_menu_append(menu, item);
+    gtk_widget_show(item);
+    g_signal_connect(item, "activate", G_CALLBACK(menuQualityBest), this);
+
 }
 
 // Create a Control menu that can be used from the menu bar or the popup.
@@ -2103,6 +2173,8 @@ GtkGui::stopHook()
     if (_resumeButton) {
         gtk_box_pack_start(GTK_BOX(_vbox), _resumeButton, FALSE, FALSE, 0);
     }
+
+    stopAdvanceTimer();
 }
 
 void
@@ -2112,6 +2184,8 @@ GtkGui::playHook()
     if (_resumeButton) {
         gtk_container_remove(GTK_CONTAINER(_vbox), _resumeButton);
     }
+
+    startAdvanceTimer();
 }
 
 
@@ -2290,31 +2364,14 @@ findPixmapFile(const gchar* filename)
 ///////////////////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////
 
-
 gboolean
-exposeEvent(GtkWidget *const /*widget*/, GdkEventExpose *const event,
-        const gpointer data)
-{
-
-    GtkGui* gui = static_cast<GtkGui*>(data);
-
-    gui->expose(event->region);
-
-    return TRUE;
-}
-
-gboolean
-configureEvent(GtkWidget *const widget, GdkEventConfigure *const event,
+configureEvent(GtkWidget *const /*widget*/, GdkEventConfigure *const event,
         const gpointer data)
 {
     GtkGui* obj = static_cast<GtkGui*>(data);
-
-    GtkGlue& glue = obj->rendererGlue();
-
-    glue.configure(widget, event);
     obj->resize_view(event->width, event->height);
 
-    return TRUE;
+    return FALSE;
 }
 
 gboolean
@@ -2513,20 +2570,46 @@ menuMovieInfo(GtkMenuItem* /*menuitem*/, gpointer data)
     gui->showPropertiesDialog();
 }
 
-// This pops up the menu when the right mouse button is clicked
+/// \brief This pops up the menu when the right mouse button is clicked
 gint
 popupHandler(GtkWidget *widget, GdkEvent *event)
 {
     GtkMenu *menu = GTK_MENU(widget);
     
-    if (event->type == GDK_BUTTON_PRESS) {
-        GdkEventButton* event_button = reinterpret_cast<GdkEventButton*>(event);
-        if (event_button->button == 3) {
-            gtk_menu_popup(menu, NULL, NULL, NULL, NULL,
-                           event_button->button, event_button->time);
-            return TRUE;
+    if( _showMenuState ) {
+        if (event->type == GDK_BUTTON_PRESS) {
+            GdkEventButton* event_button =
+                            reinterpret_cast<GdkEventButton*>(event);
+            if (event_button->button == 3) {
+                gtk_menu_popup(menu, NULL, NULL, NULL, NULL,
+                               event_button->button, event_button->time);
+                return TRUE;
+            } 
         }
     }
+
+    return FALSE;
+}
+
+/// \brief This handles the alternative popup for showMenu 
+gint
+popupHandlerAlt(GtkWidget *widget, GdkEvent *event)
+{
+    GtkMenu *menu = GTK_MENU(widget);
+    
+    
+    if( !_showMenuState ) {
+        if (event->type == GDK_BUTTON_PRESS) {
+            GdkEventButton* event_button =
+                            reinterpret_cast<GdkEventButton*>(event);
+            if (event_button->button == 3) {
+                gtk_menu_popup(menu, NULL, NULL, NULL, NULL,
+                               event_button->button, event_button->time);
+                return TRUE;
+            } 
+        }
+    }
+
     return FALSE;
 }
 
@@ -2605,6 +2688,38 @@ menuShowUpdatedRegions(GtkMenuItem* /*menuitem*/, gpointer data)
     if (!gui->showUpdatedRegions()) gui->refreshView();
 }
 
+/// \brief Set quality to LOW level
+void
+menuQualityLow(GtkMenuItem* /*menuitem*/, gpointer data)
+{
+    Gui* gui = static_cast<Gui*>(data);
+    gui->setQuality(QUALITY_LOW);
+}
+
+/// \brief Set quality to MEDIUM level
+void
+menuQualityMedium(GtkMenuItem* /*menuitem*/, gpointer data)
+{
+    Gui* gui = static_cast<Gui*>(data);
+    gui->setQuality(QUALITY_MEDIUM);
+}
+
+/// \brief Set quality to HIGH level
+void
+menuQualityHigh(GtkMenuItem* /*menuitem*/, gpointer data)
+{
+    Gui* gui = static_cast<Gui*>(data);
+    gui->setQuality(QUALITY_HIGH);
+}
+
+/// \brief Set quality to BEST level
+void
+menuQualityBest(GtkMenuItem* /*menuitem*/, gpointer data)
+{
+    Gui* gui = static_cast<Gui*>(data);
+    gui->setQuality(QUALITY_BEST);
+}
+
 } // anonymous namespace
 
 } // end of namespace gnash
