@@ -1,5 +1,5 @@
 //
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,21 +15,16 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-#include "gtk_glue.h"
+#ifndef GTK_GLU_GTKGLEXT_H
+#define GTK_GLU_GTKGLEXT_H
 
-#include <gtk/gtk.h>
-#ifndef _WIN32
-#include <gdk/gdkx.h>
-#else
-#include <gdk/gdk.h>
-#endif
+#include "Renderer_ogl.h"
+#include "gtk_glue.h"
 
 #include <gtk/gtkgl.h>
 
 #include "tu_opengl_includes.h"
 
-using namespace std;
-
 
 namespace gnash
 {
@@ -42,7 +37,7 @@ class GtkGlExtGlue : public GtkGlue
 
     bool init(int argc, char **argv[]);
     void prepDrawingArea(GtkWidget *drawing_area);
-    render_handler* createRenderHandler();
+    Renderer* createRenderHandler();
     void render();
     void render(int /*minx*/, int /*miny*/, int /*maxx*/, int /*maxy*/) { render(); };
     void configure(GtkWidget *const widget, GdkEventConfigure *const event);
@@ -54,3 +49,6 @@ class GtkGlExtGlue : public GtkGlue
 };
 
 }
+
+// end of GNASH_GTK_GLEXT_H
+#endif
