@@ -43,7 +43,8 @@
 #include "gui.h"
 #include "kdesup.h"
 #include "klash3.moc"
-#include "utility.h" // for PIXELS_TO_TWIPS 
+#include "GnashNumeric.h" // for pixelsToTwips 
+#include "RunResources.h" // for pixelsToTwips 
 
 using namespace std;
 
@@ -56,8 +57,8 @@ KdeGui::~KdeGui()
 }
 
 
-KdeGui::KdeGui(unsigned long xid, float scale, bool loop, unsigned int depth)
- : Gui(xid, scale, loop, depth)
+KdeGui::KdeGui(unsigned long xid, float scale, bool loop, RunResources& r)
+ : Gui(xid, scale, loop, r)
 {
 }
 
@@ -90,8 +91,8 @@ KdeGui::createWindow(const char* windowtitle, int width, int height)
     _qwidget->show();
 
     _glue.prepDrawingArea(_qwidget.get());
-    _renderer = _glue.createRenderHandler();
-    if ( ! _renderer )
+    _renderer.reset(_glue.createRenderHandler());
+    if (!_renderer.get())
     {
         // something went wrong
         return false;
@@ -100,7 +101,7 @@ KdeGui::createWindow(const char* windowtitle, int width, int height)
     
     _width = width;
     _height = height;
-    set_render_handler(_renderer);
+    _runResources.setRenderer(_renderer);
     
     return true;
 }
@@ -473,10 +474,10 @@ qwidget::paintEvent(QPaintEvent *event)
     const QRegion& region = event->region();
     QRect rect = region.boundingRect();
 
-    int xmin = static_cast<int> (PIXELS_TO_TWIPS(rect.x()-1)),
-        ymin = static_cast<int> (PIXELS_TO_TWIPS(rect.y()-1)),
-        xmax = static_cast<int> (PIXELS_TO_TWIPS(rect.right()+1)),
-        ymax = static_cast<int> (PIXELS_TO_TWIPS(rect.bottom()+1));
+    int xmin = static_cast<int> (pixelsToTwips(rect.x()-1)),
+        ymin = static_cast<int> (pixelsToTwips(rect.y()-1)),
+        xmax = static_cast<int> (pixelsToTwips(rect.right()+1)),
+        ymax = static_cast<int> (pixelsToTwips(rect.bottom()+1));
 
     geometry::Range2d<int> range(xmin, ymin, xmax, ymax);
     InvalidatedRanges ranges;
