@@ -1,9 +1,9 @@
 //
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
@@ -15,10 +15,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-//
-//
-
-/* $Id: riscos_glue_agg.h,v 1.4 2007/05/28 15:41:00 ann Exp $ */
 
 #include "riscos_glue.h"
 
@@ -33,7 +29,7 @@ class RiscosAggGlue : public RiscosGlue
 
     bool init(int argc, char **argv[]);
     void prepFramebuffer(void *framebuffer, int width, int height);
-    render_handler* createRenderHandler();
+    Renderer* createRenderHandler();
     void setRenderHandlerSize(int width, int height);
     void render(int x, int y);
     void render(int x, int y, int minx, int miny, int maxx, int maxy);
@@ -42,7 +38,7 @@ class RiscosAggGlue : public RiscosGlue
   private:
     unsigned char *_offscreenbuf;
     int _offscreenbuf_size;
-    render_handler *_agg_renderer;
+    Renderer *_agg_renderer;
     int _width, _height, _bpp;
 };
 
