@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,7 +15,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: sdl.cpp,v 1.63 2008/02/19 19:20:49 bwy Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "gnashconfig.h"
@@ -27,7 +26,6 @@
 # include "getopt_win32.h"
 #else
 extern "C"{
-# include <unistd.h>
 # ifdef HAVE_GETOPT_H
 #  include <getopt.h>
 # endif
@@ -40,14 +38,17 @@ extern "C"{
 #include "gnash.h"
 #include "log.h"
 #include "sdlsup.h"
+#include "Range2d.h" // for Intersection of inv bounds
+#include "Renderer.h" // for setInvalidatedRegions
+#include "RunResources.h"
 
 using namespace std;
 
 namespace gnash 
 {
 
-SDLGui::SDLGui(unsigned long xid, float scale, bool loop, unsigned int depth)
- : Gui(xid, scale, loop, depth),
+SDLGui::SDLGui(unsigned long xid, float scale, bool loop, RunResources& r)
+ : Gui(xid, scale, loop, r),
    _timeout(0),
    _core_trap(true)
 {
@@ -85,7 +86,7 @@ SDLGui::run()
                 if (event.motion.x == x_old && event.motion.y == y_old) { break; }
                 x_old = event.motion.x;
                 y_old = event.motion.y;
-                notify_mouse_moved((int) (x_old / _xscale), (int) (y_old / _yscale));
+                notify_mouse_moved(x_old, y_old);
                 break;
 
             case SDL_MOUSEBUTTONDOWN:
@@ -167,7 +168,7 @@ SDLGui::init(int argc, char **argv[])
 
     if (_xid) {
         char SDL_windowhack[32];
-        sprintf (SDL_windowhack,"SDL_WINDOWID=%ld", _xid);
+        snprintf (SDL_windowhack, sizeof(SDL_windowhack), "SDL_WINDOWID=%ld", _xid);
         putenv (SDL_windowhack);
     }
 
@@ -185,7 +186,7 @@ SDLGui::init(int argc, char **argv[])
 
     _glue.init(argc, argv);
 
-    _renderer = _glue.createRenderHandler(_depth);
+    _renderer.reset(_glue.createRenderHandler(32));
     if ( ! _renderer ) return false;
 
     return true;
@@ -211,7 +212,7 @@ SDLGui::createWindow(const char *title, int width, int height)
 
     _glue.prepDrawingArea(_width, _height, sdl_flags);
 
-    set_render_handler(_renderer);
+    _runResources.setRenderer(boost::shared_ptr<Renderer>(_renderer));
 
     SDL_WM_SetCaption( title, title);
 
@@ -224,6 +225,13 @@ SDLGui::disableCoreTrap()
     _core_trap = false;
 }
 
+/* public, overridden */
+void
+SDLGui::setInvalidatedRegions(const InvalidatedRanges& ranges)
+{
+    _glue.setInvalidatedRegions(ranges);
+}
+
 void
 SDLGui::renderBuffer()
 {
@@ -295,16 +303,16 @@ SDLGui::sdl_to_gnash_key(SDL_KeyboardEvent * key)
 int 
 SDLGui::sdl_to_gnash_modifier(int state)
 {
-  int modifier = gnash::key::MOD_NONE;
+  int modifier = gnash::key::GNASH_MOD_NONE;
   
   if (state & KMOD_SHIFT) {
-      modifier = modifier | gnash::key::MOD_SHIFT;
+      modifier = modifier | gnash::key::GNASH_MOD_SHIFT;
     }
     if (state & KMOD_CTRL) {
-      modifier = modifier | gnash::key::MOD_CONTROL;
+      modifier = modifier | gnash::key::GNASH_MOD_CONTROL;
     }
     if (state & KMOD_ALT) {
-      modifier = modifier | gnash::key::MOD_ALT;
+      modifier = modifier | gnash::key::GNASH_MOD_ALT;
     }
 
     return modifier;
@@ -315,10 +323,11 @@ void SDLGui::key_event(SDL_KeyboardEvent* key, bool down)
 {
     gnash::key::code c = sdl_to_gnash_key(key);  
 
-    if (c != gnash::key::INVALID) {
-        // 0 should be any modifier instead..
-        // see Gui::notify_key_event in gui.h
-        notify_key_event(c, 0, down);
+    if (c != gnash::key::INVALID)
+    {
+        int mod = sdl_to_gnash_modifier(key->keysym.mod);
+
+        notify_key_event(c, mod, down);
     }
 }
 
@@ -335,6 +344,5 @@ SDLGui::expose_event()
     renderBuffer();
 }
 
-
 } // namespace gnash
 
