@@ -19,18 +19,8 @@
 #include "gnashconfig.h"
 #endif
 
-#include <boost/cstdint.hpp>
-#include <boost/shared_ptr.hpp>
-#include <cerrno>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <string>
-#include <vector>
-#include <iostream>
-#include <fstream>
-#include <cassert>
-
-//#include "network.h"
+#include "GnashSystemNetHeaders.h"
+#include "GnashFileUtilities.h"
 #include "element.h"
 #include "amf.h"
 #include "buffer.h"
@@ -38,17 +28,16 @@
 #include "log.h"
 #include "GnashException.h"
 
-#if defined(HAVE_WINSOCK_H) && !defined(__OS2__)
-# include <winsock2.h>
-# include <windows.h>
-# include <sys/stat.h>
-# include <io.h>
-# include <ws2tcpip.h>
-#else
-#include <arpa/inet.h>
-#endif
-
 #include <boost/scoped_array.hpp>
+#include <boost/cstdint.hpp>
+#include <boost/shared_ptr.hpp>
+#include <cerrno>
+#include <string>
+#include <vector>
+#include <iostream>
+#include <fstream>
+#include <cassert>
+
 
 using namespace std;
 using namespace amf;
@@ -472,6 +461,28 @@ SOL::readFile(const std::string &filespec)
     
 }
 
+bool 
+SOL::updateSO(boost::shared_ptr<amf::Element> &newel)
+{
+//    GNASH_REPORT_FUNCTION;
+    vector<boost::shared_ptr<amf::Element> >::iterator ita; 
+    for (ita = _amfobjs.begin(); ita != _amfobjs.end(); ita++) {
+        boost::shared_ptr<amf::Element> oldel = (*(ita));
+	if (oldel == newel) {
+	    oldel = newel;
+	}
+   }
+   return true;
+}
+
+bool
+SOL::updateSO(int index, boost::shared_ptr<amf::Element> &el)
+{
+//    GNASH_REPORT_FUNCTION;
+    _amfobjs[index] = el;    
+    return true;
+}
+
 ///  \brief Dump the internal data of this class in a human readable form.
 ///
 /// @remarks This should only be used for debugging purposes.
