@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-#ifndef __GNASH_URL_H__
-#define __GNASH_URL_H__
+#ifndef GNASH_URL_H
+#define GNASH_URL_H
 
 #include "dsodefs.h"
 
@@ -63,7 +63,16 @@ public:
 	///
 	const std::string& hostname() const { return _host; }
 
+	/// Return the 'port' member of this URL, as a string
+	//
+	/// NOTE: return the empty string if the port isn't specified,
+        /// as this is an optional field.
+	///
+	const std::string& port() const { return _port; }
+
 	/// Return the 'path' member of this URL, as a string
+    //
+    /// The returned path starts with '/'
 	const std::string& path() const { return _path; }
 
 	/// Return the 'anchor' member of this URL, as a string
@@ -156,6 +165,9 @@ private:
 	/// Extract anchor from path
 	void split_anchor_from_path();
 
+	/// Extract tcp/ip port from path
+	void split_port_from_host();
+
 	/// Extract and parse query string from path
 	void split_querystring_from_path();
 
@@ -168,13 +180,10 @@ private:
 	void normalize_path(std::string& path);
 
 	std::string _proto;
-
 	std::string _host;
-
+	std::string _port;
 	std::string _path;
-
 	std::string _anchor;
-
 	std::string _querystring;
 };
 
