@@ -36,16 +36,26 @@
 #if defined(WIN32) || defined(_WIN32)
 #define LIBLTDL_DLL_IMPORT 1
 #endif
-#include <ltdl.h>
 #include <cstring>
 #include <iostream>
 #include <sys/types.h>
 
 #include "log.h"
+#include "ltdl.h"
 #include "sharedlib.h"
 #include "extension.h"
 #include "as_object.h"
 
+#if defined(WIN32) || defined(_WIN32)
+int        lt_dlsetsearchpath   (const char *search_path);
+int        lt_dlinit           (void);
+void *     lt_dlsym            (lt_dlhandle handle, const char *name);
+const char *lt_dlerror         (void);
+int        lt_dlclose          (lt_dlhandle handle);
+int        lt_dlmakeresident   (lt_dlhandle handle);
+lt_dlhandle lt_dlopenext       (const char *filename);
+#endif
+
 #if HAVE_DIRENT_H || WIN32==1    // win32 hack
 # include <dirent.h>
 # define NAMLEN(dirent) std::strlen((dirent)->d_name)
@@ -123,7 +133,7 @@ Extension::scanAndLoad(as_object& where)
     std::vector<std::string>::iterator it;
     for (it = _modules.begin(); it != _modules.end(); it++) {
         const std::string& mod = *it;
-        log_security(_("Loading module: %s"), mod);
+        log_security(_("Loading module: %s from %s"), mod, _pluginsdir);
         initModule(mod, where);
     }
     return true;
@@ -132,18 +142,18 @@ Extension::scanAndLoad(as_object& where)
 bool
 Extension::initModule(const std::string& module, as_object &where)
 {
+    GNASH_REPORT_FUNCTION;
 
     SharedLib *sl;
     std::string symbol(module);
 
-    log_security(_("Initializing module: \"%s\""), symbol);
+    log_security(_("Initializing module: \"%s\" from %"), symbol, _pluginsdir);
     
     if (_plugins[module] == 0) {
-        sl = new SharedLib(module);
+        sl = new SharedLib(module, "GNASH_PLUGINS");
         sl->openLib();
         _plugins[module] = sl;
-    }
-    else {
+    } else {
         sl = _plugins[module];
     }
     
@@ -153,8 +163,7 @@ Extension::initModule(const std::string& module, as_object &where)
 
     if (symptr) {    
         symptr(where);
-    }
-    else {
+    } else {
         log_error(_("Couldn't get class_init symbol"));
     }
     
@@ -165,6 +174,8 @@ bool
 Extension::initModuleWithFunc(const std::string& module, const std::string& func,
     as_object &obj)
 {
+    GNASH_REPORT_FUNCTION;
+
     SharedLib *sl;
 
     log_security(_("Initializing module: \"%s\""), module);
@@ -173,8 +184,7 @@ Extension::initModuleWithFunc(const std::string& module, const std::string& func
         sl = new SharedLib(module);
         sl->openLib();
         _plugins[module] = sl;
-    }
-    else {
+    } else {
         sl = _plugins[module];
     }
 
@@ -182,8 +192,7 @@ Extension::initModuleWithFunc(const std::string& module, const std::string& func
 
     if (symptr) {
         symptr(obj);
-    }
-    else {
+    } else {
         log_error(_("Couldn't get class_init symbol: \"%s\""), func);
     }
 
@@ -201,7 +210,7 @@ Extension::scanDir()
 bool
 Extension::scanDir(const std::string& dirlist)
 {
-//    GNASH_REPORT_FUNCTION;
+    GNASH_REPORT_FUNCTION;
     
     Tok t(dirlist, Sep(":"));
     for (Tok::iterator i = t.begin(), e = t.end(); i != e; ++i) {
@@ -216,17 +225,11 @@ Extension::scanDir(const std::string& dirlist)
             return false;
         }   
         
-        struct dirent *entry = readdir(libdir);
+        struct dirent *entry;
 
-        for (int i = 0; entry > 0; ++i) {
+        while ((entry = readdir(libdir)) != NULL) {
             // We only want shared libraries that end with the suffix, otherwise
             // we get all the duplicates.
-            entry = readdir(libdir);
-
-            if (entry <= 0) { // All done
-                continue;
-            }
-
             std::string name(entry->d_name);
 
             // Hidden files.
