@@ -25,20 +25,21 @@
 #include "Button.h"
 #include "DefineButtonTag.h"
 #include "as_value.h"
-
+#include "Button.h"
 #include "ActionExec.h"
 #include "MovieClip.h"
 #include "movie_root.h"
 #include "VM.h"
 #include "builtin_function.h"
-#include "fn_call.h" // for shared ActionScript getter-setters
-#include "GnashException.h" // for shared ActionScript getter-setters
+#include "NativeFunction.h"
+#include "fn_call.h" 
 #include "ExecutableCode.h"
 #include "namedStrings.h"
 #include "Object.h" // for getObjectInterface
 #include "StringPredicates.h"
-#include "GnashKey.h" // key::code
-#include "SoundInfoRecord.h" // for use
+#include "GnashKey.h" 
+#include "SoundInfoRecord.h" 
+#include "Global_as.h" 
 
 #include <boost/bind.hpp>
 
@@ -192,7 +193,7 @@ private:
 
 class ButtonActionPusher {
 public:
-    ButtonActionPusher(movie_root& mr, character* this_ptr)
+    ButtonActionPusher(movie_root& mr, DisplayObject* this_ptr)
         :
         called(false),
         _mr(mr),
@@ -201,7 +202,7 @@ public:
 
     void operator() (const action_buffer& ab)
     {
-        _mr.pushAction(ab, boost::intrusive_ptr<character>(_tp));
+        _mr.pushAction(ab, boost::intrusive_ptr<DisplayObject>(_tp));
         called = true;
     }
 
@@ -209,7 +210,7 @@ public:
 
 private:
     movie_root& _mr;
-    character* _tp;
+    DisplayObject* _tp;
 };
 
 }
@@ -217,22 +218,38 @@ private:
 // Forward declarations
 static as_object* getButtonInterface();
 
+namespace {
+    void addInstanceProperty(Button& b, DisplayObject* d) {
+        if (!d) return;
+        const std::string& name = d->get_name();
+        if (name.empty()) return;
+        b.init_member(name, d, 0);
+    }
+
+    void removeInstanceProperty(Button& b, DisplayObject* d) {
+        if (!d) return;
+        const std::string& name = d->get_name();
+        if (name.empty()) return;
+        b.delProperty(getStringTable(b).find(name));
+    }
+}
+
 /// Predicates for standard algorithms.
 
-/// Depth comparator for characters.
-static bool charDepthLessThen(const character* ch1, const character* ch2) 
+/// Depth comparator for DisplayObjects.
+static bool charDepthLessThen(const DisplayObject* ch1, const DisplayObject* ch2) 
 {
     return ch1->get_depth() < ch2->get_depth();
 }
 
-/// Predicate for finding active characters.
+/// Predicate for finding active DisplayObjects.
 //
-/// Returns true if the character should be skipped:
+/// Returns true if the DisplayObject should be skipped:
 /// 1) if it is NULL, or 
-/// 2) if we don't want unloaded characters and the character is unloaded.
-static bool isCharacterNull(character* ch, bool includeUnloaded)
+/// 2) if we don't want unloaded DisplayObjects and the DisplayObject is unloaded.
+static bool isCharacterNull(DisplayObject* ch, bool includeUnloaded)
 {
-    return (!ch || (!includeUnloaded && ch->isUnloaded()));
+    return (!ch || (!includeUnloaded && ch->unloaded()));
 }
 
 static void
@@ -241,83 +258,96 @@ attachButtonInterface(as_object& o)
 
     as_c_function_ptr gettersetter;
 
-    o.init_property(NSV::PROP_uQUALITY, character::quality,
-            character::quality);
+    o.init_property(NSV::PROP_uQUALITY, DisplayObject::quality,
+            DisplayObject::quality);
     
-    o.init_property(NSV::PROP_uHIGHQUALITY, character::highquality,
-            character::highquality);
+    o.init_property(NSV::PROP_uHIGHQUALITY, DisplayObject::highquality,
+            DisplayObject::highquality);
 
-    gettersetter = &character::x_getset;
+    gettersetter = &DisplayObject::x_getset;
     o.init_property(NSV::PROP_uX, *gettersetter, *gettersetter);
 
-    gettersetter = &character::y_getset;
+    gettersetter = &DisplayObject::y_getset;
     o.init_property(NSV::PROP_uY, *gettersetter, *gettersetter);
 
-    gettersetter = &character::xscale_getset;
+    gettersetter = &DisplayObject::xscale_getset;
     o.init_property(NSV::PROP_uXSCALE, *gettersetter, *gettersetter);
 
-    gettersetter = &character::yscale_getset;
+    gettersetter = &DisplayObject::yscale_getset;
     o.init_property(NSV::PROP_uYSCALE, *gettersetter, *gettersetter);
 
-    gettersetter = &character::xmouse_get;
+    gettersetter = &DisplayObject::xmouse_get;
     o.init_readonly_property(NSV::PROP_uXMOUSE, *gettersetter);
 
-    gettersetter = &character::ymouse_get;
+    gettersetter = &DisplayObject::ymouse_get;
     o.init_readonly_property(NSV::PROP_uYMOUSE, *gettersetter);
 
-    gettersetter = &character::alpha_getset;
+    gettersetter = &DisplayObject::alpha_getset;
     o.init_property(NSV::PROP_uALPHA, *gettersetter, *gettersetter);
 
-    gettersetter = &character::visible_getset;
+    gettersetter = &DisplayObject::visible_getset;
     o.init_property(NSV::PROP_uVISIBLE, *gettersetter, *gettersetter);
 
-    gettersetter = &character::width_getset;
+    gettersetter = &DisplayObject::width_getset;
     o.init_property(NSV::PROP_uWIDTH, *gettersetter, *gettersetter);
 
-    gettersetter = &character::height_getset;
+    gettersetter = &DisplayObject::height_getset;
     o.init_property(NSV::PROP_uHEIGHT, *gettersetter, *gettersetter);
 
-    gettersetter = &character::rotation_getset;
+    gettersetter = &DisplayObject::rotation_getset;
     o.init_property(NSV::PROP_uROTATION, *gettersetter, *gettersetter);
 
-    gettersetter = &character::parent_getset;
+    gettersetter = &DisplayObject::parent_getset;
     o.init_property(NSV::PROP_uPARENT, *gettersetter, *gettersetter);
     
-    gettersetter = &character::target_getset;
+    gettersetter = &DisplayObject::target_getset;
     o.init_property(NSV::PROP_uTARGET, *gettersetter, *gettersetter);
 
-    gettersetter = character::name_getset;
+    gettersetter = DisplayObject::name_getset;
     o.init_property(NSV::PROP_uNAME, gettersetter, gettersetter);
     
     const int unprotected = 0;
     o.init_member(NSV::PROP_ENABLED, true, unprotected);
+    o.init_member("useHandCursor", true, unprotected);
 
 }
 
-Button::Button(SWF::DefineButtonTag& def, character* parent, int id)
+Button::Button(const SWF::DefineButtonTag* const def, DisplayObject* parent,
+        int id)
     :
-    character(parent, id),
-    m_last_mouse_flags(IDLE),
-    m_mouse_flags(IDLE),
-    m_mouse_state(UP),
+    InteractiveObject(parent, id),
+    _lastMouseFlags(FLAG_IDLE),
+    _mouseFlags(FLAG_IDLE),
+    _mouseState(MOUSESTATE_UP),
     _def(def)
 {
 
     set_prototype(getButtonInterface());
 
     // check up presence Key events
-    if ( _def.hasKeyPressHandler() )
-    {
-        _vm.getRoot().add_key_listener(this);
+    if (_def->hasKeyPressHandler()) {
+        getRoot(*this).add_key_listener(this);
     }
 
 }
 
 Button::~Button()
 {
-    _vm.getRoot().remove_key_listener(this);
+    getRoot(*this).remove_key_listener(this);
 }
 
+bool
+Button::trackAsMenu()
+{
+    // TODO: check whether the AS or the tag value takes precedence.
+    as_value track;
+    string_table& st = getStringTable(*this);
+    if (get_member(st.find("trackAsMenu"), &track)) {
+        return track.to_bool();
+    }
+    if (_def) return _def->trackAsMenu();
+    return false;
+}
 
 bool 
 Button::isEnabled()
@@ -329,12 +359,10 @@ Button::isEnabled()
 }
 
 
-// called from Key listener only
-// (the above line is wrong, it's also called with onConstruct, for instance)
 bool
 Button::on_event(const event_id& id)
 {
-    if ( isUnloaded() )
+    if (unloaded())
     {
         // We dont' respond to events while unloaded
         // See bug #22982
@@ -351,8 +379,8 @@ Button::on_event(const event_id& id)
     // We only respond to valid key code (should we assert here?)
     if ( id.keyCode() == key::INVALID ) return false;
 
-    ButtonActionPusher xec(getVM().getRoot(), this); 
-    _def.forEachTrigger(id, xec);
+    ButtonActionPusher xec(getRoot(*this), this); 
+    _def->forEachTrigger(id, xec);
 
     return xec.called;
 }
@@ -365,34 +393,30 @@ Button::handleFocus() {
 
 
 void
-Button::restart()
-{
-    log_error("Button::restart called, from whom??");
-}
-
-void
-Button::display()
+Button::display(Renderer& renderer)
 {
 
-    std::vector<character*> actChars;
+    DisplayObjects actChars;
     getActiveCharacters(actChars);
 
     // TODO: by keeping chars sorted by depth we'd avoid the sort on display
     std::sort(actChars.begin(), actChars.end(), charDepthLessThen);
 
-    std::for_each(actChars.begin(), actChars.end(),
-            std::mem_fun(&character::display)); 
+    for (DisplayObjects::iterator it = actChars.begin(), e = actChars.end();
+            it != e; ++it) {
+        (*it)->display(renderer);
+    }
 
     clear_invalidated();
 }
 
 
-character*
-Button::get_topmost_mouse_entity(boost::int32_t x, boost::int32_t y)
 // Return the topmost entity that the given point covers.  NULL if none.
 // I.e. check against ourself.
+InteractiveObject*
+Button::topmostMouseEntity(boost::int32_t x, boost::int32_t y)
 {
-    if (!isVisible() || !isEnabled())
+    if (!visible() || !isEnabled())
     {
         return 0;
     }
@@ -401,8 +425,7 @@ Button::get_topmost_mouse_entity(boost::int32_t x, boost::int32_t y)
     // Check our active and visible children first
     //-------------------------------------------------
 
-    typedef std::vector<character*> Chars;
-    Chars actChars;
+    DisplayObjects actChars;
     getActiveCharacters(actChars);
 
     if ( ! actChars.empty() )
@@ -413,12 +436,12 @@ Button::get_topmost_mouse_entity(boost::int32_t x, boost::int32_t y)
         point  p(x, y);
         m.invert().transform(p);
 
-        for (Chars::reverse_iterator it=actChars.rbegin(), itE=actChars.rend();
-                it!=itE; ++it)
+        for (DisplayObjects::reverse_iterator it = actChars.rbegin(),
+                itE=actChars.rend(); it!=itE; ++it)
         {
-            character* ch = *it;
-            if ( ! ch->isVisible() ) continue;
-            character *hit = ch->get_topmost_mouse_entity(p.x, p.y);
+            DisplayObject* ch = *it;
+            if ( ! ch->visible() ) continue;
+            InteractiveObject *hit = ch->topmostMouseEntity(p.x, p.y);
             if ( hit ) return hit;
         }
     }
@@ -427,13 +450,13 @@ Button::get_topmost_mouse_entity(boost::int32_t x, boost::int32_t y)
     // If that failed, check our hit area
     //-------------------------------------------------
 
-    // Find hit characters
+    // Find hit DisplayObjects
     if ( _hitCharacters.empty() ) return 0;
 
     // point is in parent's space,
     // we need to convert it in world space
     point  wp(x,y);
-    character* parent = get_parent();
+    DisplayObject* parent = get_parent();
     if ( parent )
     {
         parent->getWorldMatrix().transform(wp);
@@ -454,9 +477,9 @@ Button::get_topmost_mouse_entity(boost::int32_t x, boost::int32_t y)
 
 
 void
-Button::on_button_event(const event_id& event)
+Button::mouseEvent(const event_id& event)
 {
-    if ( isUnloaded() )
+    if ( unloaded() )
     {
         // We don't respond to events while unloaded. See bug #22982.
         log_debug("Button %s received %s button event while unloaded: ignored",
@@ -464,27 +487,27 @@ Button::on_button_event(const event_id& event)
         return;
     }
 
-    MouseState new_state = m_mouse_state;
+    MouseState new_state = _mouseState;
   
     // Set our mouse state (so we know how to render).
     switch (event.id())
     {
         case event_id::ROLL_OUT:
         case event_id::RELEASE_OUTSIDE:
-            new_state = UP;
+            new_state = MOUSESTATE_UP;
             break;
 
         case event_id::RELEASE:
         case event_id::ROLL_OVER:
         case event_id::DRAG_OUT:
         case event_id::MOUSE_UP:
-            new_state = OVER;
+            new_state = MOUSESTATE_OVER;
             break;
 
         case event_id::PRESS:
         case event_id::DRAG_OVER:
         case event_id::MOUSE_DOWN:
-            new_state = DOWN;
+            new_state = MOUSESTATE_DOWN;
             break;
 
         default:
@@ -498,10 +521,10 @@ Button::on_button_event(const event_id& event)
     // Button transition sounds.
     do {
 
-        if (!_def.hasSound()) break;
+        if (!_def->hasSound()) break;
 
         // Check if there is a sound handler
-        sound::sound_handler* s = _vm.getRoot().runInfo().soundHandler();
+        sound::sound_handler* s = getRunResources(*this).soundHandler();
         if (!s) break;
 
         int bi; // button sound array index [0..3]
@@ -529,9 +552,9 @@ Button::on_button_event(const event_id& event)
         if (bi < 0) break;
 
         const SWF::DefineButtonSoundTag::ButtonSound& bs = 
-            _def.buttonSound(bi);
+            _def->buttonSound(bi);
 
-        // character zero is considered as null character
+        // DisplayObject zero is considered as null DisplayObject
         if (!bs.soundID) break;
 
         // No actual sound ?
@@ -548,12 +571,12 @@ Button::on_button_event(const event_id& event)
             const sound::SoundEnvelopes* env = 
                 sinfo.envelopes.empty() ? 0 : &sinfo.envelopes;
 
-            s->playSound(bs.sample->m_sound_handler_id,
+            s->startSound(bs.sample->m_sound_handler_id,
                     bs.soundInfo.loopCount,
-                    0, // secs offset
-                    0, // byte offset
                     env, // envelopes
-                    !sinfo.noMultiple // allow multiple instances ?
+                    !sinfo.noMultiple, // allow multiple instances ?
+                    sinfo.inPoint,
+                    sinfo.outPoint
                     );
         }
 
@@ -569,10 +592,10 @@ Button::on_button_event(const event_id& event)
     // the action queue on mouse event.
     //
 
-    movie_root& mr = getVM().getRoot();
+    movie_root& mr = getRoot(*this);
 
     ButtonActionPusher xec(mr, this); 
-    _def.forEachTrigger(event, xec);
+    _def->forEachTrigger(event, xec);
 
     // check for built-in event handler.
     std::auto_ptr<ExecutableCode> code ( get_event_handler(event) );
@@ -585,21 +608,19 @@ Button::on_button_event(const event_id& event)
     // Call conventional attached method.
     boost::intrusive_ptr<as_function> method =
         getUserDefinedEventHandler(event.functionKey());
-    if ( method )
-    {
+    if (method) {
         mr.pushAction(method, this, movie_root::apDOACTION);
     }
 }
 
 
 void
-Button::getActiveCharacters(
-        std::vector<const character*>& list) const
+Button::getActiveCharacters(ConstDisplayObjects& list) const
 {
     list.clear();
 
-    // Copy all the characters to the new list, skipping NULL and unloaded
-    // characters.
+    // Copy all the DisplayObjects to the new list, skipping NULL and unloaded
+    // DisplayObjects.
     std::remove_copy_if(_stateCharacters.begin(), _stateCharacters.end(),
             std::back_inserter(list),
             boost::bind(&isCharacterNull, _1, false));
@@ -608,13 +629,12 @@ Button::getActiveCharacters(
 
 
 void 
-Button::getActiveCharacters(
-        std::vector<character*>& list, bool includeUnloaded)
+Button::getActiveCharacters(DisplayObjects& list, bool includeUnloaded)
 {
     list.clear();
 
-    // Copy all the characters to the new list, skipping NULL
-    // characters, optionally including unloaded characters.
+    // Copy all the DisplayObjects to the new list, skipping NULL
+    // DisplayObjects, optionally including unloaded DisplayObjects.
     std::remove_copy_if(_stateCharacters.begin(), _stateCharacters.end(),
             std::back_inserter(list),
             boost::bind(&isCharacterNull, _1, includeUnloaded));
@@ -627,37 +647,30 @@ Button::get_active_records(ActiveRecords& list, MouseState state)
     list.clear();
     
     using namespace SWF;
-    const DefineButtonTag::ButtonRecords& br = _def.buttonRecords();
+    const DefineButtonTag::ButtonRecords& br = _def->buttonRecords();
     size_t index = 0;
 
     for (DefineButtonTag::ButtonRecords::const_iterator i = br.begin(),
             e = br.end(); i != e; ++i, ++index)
     {
         const ButtonRecord& rec =*i;
-
-        if ((state == UP && rec.m_up)
-            || (state == DOWN && rec.m_down)
-            || (state == OVER && rec.m_over)
-            || (state == HIT && rec.m_hit_test))
-        {
-            list.insert(index);
-        }
+        if (rec.hasState(state)) list.insert(index);
     }
 }
 
 #ifdef GNASH_DEBUG_BUTTON_DISPLAYLIST
-static void dump(std::vector< character* >& chars, std::stringstream& ss)
+static void dump(Button::DisplayObjects& chars, std::stringstream& ss)
 {
     for (size_t i=0, e=chars.size(); i<e; ++i)
     {
         ss << "Record" << i << ": ";
-        character* ch = chars[i];
+        DisplayObject* ch = chars[i];
         if ( ! ch ) ss << "NULL.";
         else
         {
             ss << ch->getTarget() << " (depth:" << 
-                ch->get_depth()-character::staticDepthOffset-1
-                << " unloaded:" << ch->isUnloaded() <<
+                ch->get_depth()-DisplayObject::staticDepthOffset-1
+                << " unloaded:" << ch->unloaded() <<
                 " destroyed:" << ch->isDestroyed() << ")";
         }
         ss << std::endl;
@@ -668,7 +681,7 @@ static void dump(std::vector< character* >& chars, std::stringstream& ss)
 void
 Button::set_current_state(MouseState new_state)
 {
-    if (new_state == m_mouse_state)
+    if (new_state == _mouseState)
         return;
         
 #ifdef GNASH_DEBUG_BUTTON_DISPLAYLIST
@@ -685,14 +698,15 @@ Button::set_current_state(MouseState new_state)
     // For each possible record, check if it should still be there
     for (size_t i=0, e=_stateCharacters.size(); i<e; ++i)
     {
-        character* oldch = _stateCharacters[i];
+        DisplayObject* oldch = _stateCharacters[i];
         bool shouldBeThere = ( newChars.find(i) != newChars.end() );
 
         if ( ! shouldBeThere )
         {
+
             // is there, but is unloaded: destroy, clear slot and go on
-            if ( oldch && oldch->isUnloaded() )
-            {
+            if ( oldch && oldch->unloaded() ) {
+                removeInstanceProperty(*this, oldch);
                 if ( ! oldch->isDestroyed() ) oldch->destroy();
                 _stateCharacters[i] = NULL;
                 oldch = NULL;
@@ -705,14 +719,15 @@ Button::set_current_state(MouseState new_state)
                 if ( ! oldch->unload() )
                 {
                     // No onUnload handler: destroy and clear slot
-                    if ( ! oldch->isDestroyed() ) oldch->destroy();
+                    removeInstanceProperty(*this, oldch);
+                    if (!oldch->isDestroyed()) oldch->destroy();
                     _stateCharacters[i] = NULL;
                 }
                 else
                 {
                     // onUnload handler: shift depth and keep slot
                     int oldDepth = oldch->get_depth();
-                    int newDepth = character::removedDepthOffset - oldDepth;
+                    int newDepth = DisplayObject::removedDepthOffset - oldDepth;
 #ifdef GNASH_DEBUG_BUTTON_DISPLAYLIST
                     log_debug("Removed button record shifted from depth %d to depth %d", oldDepth, newDepth);
 #endif
@@ -723,45 +738,23 @@ Button::set_current_state(MouseState new_state)
         else // should be there
         {
             // Is there already, but is unloaded: destroy and consider as gone
-            if ( oldch && oldch->isUnloaded() )
+            if ( oldch && oldch->unloaded() )
             {
+                removeInstanceProperty(*this, oldch);
                 if ( ! oldch->isDestroyed() ) oldch->destroy();
                 _stateCharacters[i] = NULL;
                 oldch = NULL;
             }
 
-            if ( ! oldch )
-            {
+            if (!oldch) {
                 // Not there, instantiate
-                SWF::ButtonRecord& bdef = _def.buttonRecords()[i];
-
-                const SWFMatrix& mat = bdef.m_button_matrix;
-                const cxform& cx = bdef.m_button_cxform;
-                int ch_depth = bdef.m_button_layer + 
-                    character::staticDepthOffset + 1;
-                int ch_id = bdef.m_character_id;
-
-                character* ch = bdef.m_character_def->create_character_instance(
-                        this, ch_id);
-                ch->setMatrix(mat, true); // update caches
-                ch->set_cxform(cx); 
-                ch->set_depth(ch_depth); 
-                assert(ch->get_parent() == this);
-
-                // no way to specify a name for button chars anyway...
-                assert(ch->get_name().empty()); 
-
-                if ( ch->wantsInstanceName() )
-                {
-                    //std::string instance_name = getNextUnnamedInstanceName();
-                    ch->set_name(getNextUnnamedInstanceName());
-                }
-
+                const SWF::ButtonRecord& rec = _def->buttonRecords()[i];
+                DisplayObject* ch = rec.instantiate(this);
+                
                 set_invalidated();
-
                 _stateCharacters[i] = ch;
-                ch->stagePlacementCallback(); // give this character a life
-
+                addInstanceProperty(*this, ch);
+                ch->stagePlacementCallback(); 
             }
         }
     }
@@ -774,30 +767,23 @@ Button::set_current_state(MouseState new_state)
 #endif
 
     // Remember current state
-    m_mouse_state=new_state;
+    _mouseState=new_state;
      
 }
 
-//
-// ActionScript overrides
-//
-
-
-
 void 
-Button::add_invalidated_bounds(InvalidatedRanges& ranges, 
-    bool force)
+Button::add_invalidated_bounds(InvalidatedRanges& ranges, bool force)
 {
 
     // Not visible anyway
-    if (!isVisible()) return;
+    if (!visible()) return;
 
     ranges.add(m_old_invalidated_ranges);  
 
-    std::vector<character*> actChars;
+    DisplayObjects actChars;
     getActiveCharacters(actChars);
     std::for_each(actChars.begin(), actChars.end(),
-            boost::bind(&character::add_invalidated_bounds, _1,
+            boost::bind(&DisplayObject::add_invalidated_bounds, _1,
                 boost::ref(ranges), force||m_invalidated)
     );
 }
@@ -807,12 +793,12 @@ Button::getBounds() const
 {
     rect allBounds;
 
-    typedef std::vector<const character*> CharVect;
-    CharVect actChars;
+    typedef std::vector<const DisplayObject*> Chars;
+    Chars actChars;
     getActiveCharacters(actChars);
-    for(CharVect::const_iterator i=actChars.begin(),e=actChars.end(); i!=e; ++i)
+    for(Chars::const_iterator i=actChars.begin(),e=actChars.end(); i!=e; ++i)
     {
-        const character* ch = *i;
+        const DisplayObject* ch = *i;
         // Child bounds need be transformed in our coordinate space
         rect lclBounds = ch->getBounds();
         SWFMatrix m = ch->getMatrix();
@@ -825,13 +811,13 @@ Button::getBounds() const
 bool
 Button::pointInShape(boost::int32_t x, boost::int32_t y) const
 {
-    typedef std::vector<const character*> CharVect;
-    CharVect actChars;
+    typedef std::vector<const DisplayObject*> Chars;
+    Chars actChars;
     getActiveCharacters(actChars);
-    for(CharVect::const_iterator i=actChars.begin(),e=actChars.end(); i!=e; ++i)
+    for(Chars::const_iterator i=actChars.begin(),e=actChars.end(); i!=e; ++i)
     {
-        const character* ch = *i;
-        if ( ch->pointInShape(x,y) ) return true;
+        const DisplayObject* ch = *i;
+        if (ch->pointInShape(x,y)) return true;
     }
     return false; 
 }
@@ -839,17 +825,17 @@ Button::pointInShape(boost::int32_t x, boost::int32_t y) const
 as_object*
 Button::get_path_element(string_table::key key)
 {
-    as_object* ch = get_path_element_character(key);
+    as_object* ch = getPathElementSeparator(key);
     if ( ch ) return ch;
 
-    const std::string& name = _vm.getStringTable().value(key);
+    const std::string& name = getStringTable(*this).value(key);
     return getChildByName(name); // possibly NULL
 }
 
-character *
+DisplayObject *
 Button::getChildByName(const std::string& name)
 {
-    // Get all currently active characters, including unloaded
+    // Get all currently active DisplayObjects, including unloaded
     DisplayObjects actChars;
     getActiveCharacters(actChars, true);
 
@@ -859,10 +845,10 @@ Button::getChildByName(const std::string& name)
     for (DisplayObjects::iterator i=actChars.begin(), e=actChars.end(); i!=e; ++i)
     {
 
-        character* const child = *i;
+        DisplayObject* const child = *i;
         const std::string& childname = child->get_name();
  
-        if ( _vm.getSWFVersion() >= 7 )
+        if (getSWFVersion(*this) >= 7 )
         {
             if ( childname == name ) return child;
         }
@@ -889,70 +875,42 @@ Button::stagePlacementCallback(as_object* initObj)
 
     saveOriginalTarget(); // for soft refs
 
-    // Don't register this button instance as a live character.
+    // Don't register this button instance as a live DisplayObject.
 
-    // Instantiate the hit characters
+    // Instantiate the hit DisplayObjects
     ActiveRecords hitChars;
-    get_active_records(hitChars, HIT);
+    get_active_records(hitChars, MOUSESTATE_HIT);
     for (ActiveRecords::iterator i=hitChars.begin(),e=hitChars.end(); i!=e; ++i)
     {
-        SWF::ButtonRecord& bdef = _def.buttonRecords()[*i];
-
-        const SWFMatrix& mat = bdef.m_button_matrix;
-        const cxform& cx = bdef.m_button_cxform;
-        int ch_depth = bdef.m_button_layer+character::staticDepthOffset+1;
-        int ch_id = bdef.m_character_id;
-
-        character* ch =
-            bdef.m_character_def->create_character_instance(this, ch_id);
-        ch->setMatrix(mat, true);  // update caches
-    
-        // TODO: who cares about color, depth etc.
-        ch->set_cxform(cx); 
-        ch->set_depth(ch_depth);
-        assert(ch->get_parent() == this);
-        assert(ch->get_name().empty()); 
+        const SWF::ButtonRecord& rec = _def->buttonRecords()[*i];
 
+        // These should not be named!
+        DisplayObject* ch = rec.instantiate(this, false);
         _hitCharacters.push_back(ch);
     }
 
-    // Setup the state characters container
-    // It will have a slot for each character record.
+    // Setup the state DisplayObjects container
+    // It will have a slot for each DisplayObject record.
     // Some slots will probably be never used (consider HIT-only records)
     // but for now this direct corrispondence between record number
-    // and active character will be handy.
-    _stateCharacters.resize(_def.buttonRecords().size());
+    // and active DisplayObject will be handy.
+    _stateCharacters.resize(_def->buttonRecords().size());
 
-    // Instantiate the default state characters 
+    // Instantiate the default state DisplayObjects 
     ActiveRecords upChars;
-    get_active_records(upChars, UP);
+    get_active_records(upChars, MOUSESTATE_UP);
 
-    for (ActiveRecords::iterator i=upChars.begin(),e=upChars.end(); i!=e; ++i)
+    for (ActiveRecords::iterator i = upChars.begin(), e=upChars.end();
+            i != e; ++i)
     {
         int rno = *i;
-        SWF::ButtonRecord& bdef = _def.buttonRecords()[rno];
+        const SWF::ButtonRecord& rec = _def->buttonRecords()[rno];
 
-        const SWFMatrix& mat = bdef.m_button_matrix;
-        const cxform& cx = bdef.m_button_cxform;
-        int ch_depth = bdef.m_button_layer+character::staticDepthOffset+1;
-        int ch_id = bdef.m_character_id;
-
-        character* ch = bdef.m_character_def->create_character_instance(
-                this, ch_id);
-        ch->setMatrix(mat, true);  // update caches
-        ch->set_cxform(cx); 
-        ch->set_depth(ch_depth); 
-        assert(ch->get_parent() == this);
-        assert(ch->get_name().empty()); 
-
-        if ( ch->wantsInstanceName() )
-        {
-            //std::string instance_name = getNextUnnamedInstanceName();
-            ch->set_name(getNextUnnamedInstanceName());
-        }
+        DisplayObject* ch = rec.instantiate(this);
 
         _stateCharacters[rno] = ch;
-        ch->stagePlacementCallback(); // give this character a life
+        addInstanceProperty(*this, ch);
+        ch->stagePlacementCallback(); // give this DisplayObject a life
     }
 
     // There is no INITIALIZE/CONSTRUCT/LOAD/ENTERFRAME/UNLOAD event 
@@ -965,27 +923,27 @@ Button::markReachableResources() const
 {
     assert(isReachable());
 
-    _def.setReachable();
+    _def->setReachable();
 
-    // Mark state characters as reachable
+    // Mark state DisplayObjects as reachable
     for (DisplayObjects::const_iterator i=_stateCharacters.begin(), e=_stateCharacters.end();
             i!=e; ++i)
     {
-        character* ch = *i;
+        DisplayObject* ch = *i;
         if ( ch ) ch->setReachable();
     }
 
-    // Mark hit characters as reachable
+    // Mark hit DisplayObjects as reachable
     for (DisplayObjects::const_iterator i = _hitCharacters.begin(),
             e=_hitCharacters.end(); i != e; ++i)
     {
-        character* ch = *i;
+        DisplayObject* ch = *i;
         assert ( ch );
         ch->setReachable();
     }
 
-    // character class members
-    markCharacterReachable();
+    // DisplayObject class members
+    markDisplayObjectReachable();
 }
 #endif // GNASH_USE_GC
 
@@ -1000,9 +958,9 @@ Button::unload()
     for (DisplayObjects::iterator i = _stateCharacters.begin(),
             e = _stateCharacters.end(); i != e; ++i)
     {
-        character* ch = *i;
+        DisplayObject* ch = *i;
         if ( ! ch ) continue;
-        if ( ch->isUnloaded() ) continue;
+        if ( ch->unloaded() ) continue;
         if ( ch->unload() ) childsHaveUnload = true;
     }
 
@@ -1014,7 +972,7 @@ Button::unload()
     //       hit instance off the GC).
     _hitCharacters.clear();
 
-    bool hasUnloadEvent = character::unload();
+    bool hasUnloadEvent = DisplayObject::unload();
 
     return hasUnloadEvent || childsHaveUnload;
 }
@@ -1026,7 +984,7 @@ Button::destroy()
     for (DisplayObjects::iterator i = _stateCharacters.begin(),
             e=_stateCharacters.end(); i != e; ++i)
     {
-        character* ch = *i;
+        DisplayObject* ch = *i;
         if ( ! ch ) continue;
         if ( ch->isDestroyed() ) continue;
         ch->destroy();
@@ -1041,7 +999,7 @@ Button::destroy()
     //       hit instance off the GC).
     _hitCharacters.clear();
 
-    character::destroy();
+    DisplayObject::destroy();
 }
 
 bool
@@ -1049,13 +1007,13 @@ Button::get_member(string_table::key name_key, as_value* val,
     string_table::key nsname)
 {
     // FIXME: use addProperty interface for these !!
-    // TODO: or at least have a character:: protected method take
+    // TODO: or at least have a DisplayObject:: protected method take
     //       care of these ?
-    //       Duplicates code in character::get_path_element_character too..
+    //       Duplicates code in DisplayObject::getPathElementSeparator too..
     //
     if (name_key == NSV::PROP_uROOT) {
         // getAsRoot() will take care of _lockroot
-        val->set_as_object( const_cast<MovieClip*>( getAsRoot() )    );
+        val->set_as_object(getAsRoot());
         return true;
     }
 
@@ -1065,18 +1023,18 @@ Button::get_member(string_table::key name_key, as_value* val,
     //             to the SWF4 code but available to the SWF6 one.
     //
     // see MovieClip.as
-    if ( getSWFVersion() > 5 && name_key == NSV::PROP_uGLOBAL ) {
+    if (getMovieVersion() > 5 && name_key == NSV::PROP_uGLOBAL ) {
         // The "_global" ref was added in SWF6
-        val->set_as_object( _vm.getGlobal() );
+        val->set_as_object(getGlobal(*this));
         return true;
     }
 
-    const std::string& name = _vm.getStringTable().value(name_key);
+    const std::string& name = getStringTable(*this).value(name_key);
 
-    movie_root& mr = _vm.getRoot();
+    movie_root& mr = getRoot(*this);
     unsigned int levelno;
     if ( mr.isLevelTarget(name, levelno) ) {
-        movie_instance* mo = mr.getLevel(levelno).get();
+        Movie* mo = mr.getLevel(levelno).get();
         if ( mo ) {
             val->set_as_object(mo);
             return true;
@@ -1102,10 +1060,10 @@ Button::get_member(string_table::key name_key, as_value* val,
         if ( getChildByName(name) )
         {
             log_aserror(_("A button member (%s) clashes with "
-                    "the name of an existing character "
+                    "the name of an existing DisplayObject "
                     "in its display list.    "
                     "The member will hide the "
-                    "character"), name);
+                    "DisplayObject"), name);
         }
         );
 #endif
@@ -1114,7 +1072,7 @@ Button::get_member(string_table::key name_key, as_value* val,
     }
 
     // Try items on our display list.
-    character* ch = getChildByName(name);
+    DisplayObject* ch = getChildByName(name);
 
     if (ch) {
         // Found object.
@@ -1136,9 +1094,9 @@ Button::get_member(string_table::key name_key, as_value* val,
 }
 
 int
-Button::getSWFVersion() const
+Button::getMovieVersion() const
 {
-    return _def.getSWFVersion();
+    return _def->getSWFVersion();
 }
 
 static as_object*
@@ -1158,40 +1116,50 @@ getButtonInterface()
 static as_value
 button_ctor(const fn_call& /* fn */)
 {
-  boost::intrusive_ptr<as_object> clip = new as_object(getButtonInterface());
-  return as_value(clip.get());
+    return as_value();
 }
 
 void
-Button::init(as_object& global)
+button_class_init(as_object& global, const ObjectURI& uri)
 {
-  // This is going to be the global Button "class"/"function"
-  static boost::intrusive_ptr<builtin_function> cl=NULL;
+    // This is going to be the global Button "class"/"function"
+    Global_as* gl = getGlobal(global);
+    as_object* proto = getButtonInterface();
+    as_object* cl = gl->createClass(&button_ctor, proto);
 
-  if ( cl == NULL )
-  {
-    cl=new builtin_function(&button_ctor, getButtonInterface());
-    VM::get().addStatic(cl.get());
-  }
+    // Register _global.MovieClip
+    global.init_member(getName(uri), cl, as_object::DefaultFlags,
+            getNamespace(uri));
+}
 
-  // Register _global.MovieClip
-  global.init_member("Button", cl.get());
+void
+registerButtonNative(as_object& /*global*/)
+{
+    // TODO: button.getDepth
 }
 
 #ifdef USE_SWFTREE
-character::InfoTree::iterator 
+DisplayObject::InfoTree::iterator 
 Button::getMovieInfo(InfoTree& tr, InfoTree::iterator it)
 {
-    InfoTree::iterator selfIt = character::getMovieInfo(tr, it);
+    InfoTree::iterator selfIt = DisplayObject::getMovieInfo(tr, it);
     std::ostringstream os;
 
-    std::vector<character*> actChars;
+    DisplayObjects actChars;
     getActiveCharacters(actChars, true);
     std::sort(actChars.begin(), actChars.end(), charDepthLessThen);
 
-    os << actChars.size() << " active characters for state " << mouseStateName(m_mouse_state);
-    InfoTree::iterator localIter = tr.append_child(selfIt, StringPair(_("Button state"), os.str()));        
-    std::for_each(actChars.begin(), actChars.end(), boost::bind(&character::getMovieInfo, _1, tr, localIter)); 
+    os << actChars.size() << " active DisplayObjects for state " <<
+        mouseStateName(_mouseState);
+    InfoTree::iterator localIter = tr.append_child(selfIt,
+            StringPair(_("Button state"), os.str()));
+
+    os.str("");
+    os << std::boolalpha << isEnabled();
+    localIter = tr.append_child(selfIt, StringPair(_("Enabled"), os.str()));
+
+    std::for_each(actChars.begin(), actChars.end(),
+            boost::bind(&DisplayObject::getMovieInfo, _1, tr, localIter)); 
 
     return selfIt;
 
@@ -1203,10 +1171,10 @@ Button::mouseStateName(MouseState s)
 {
     switch (s)
     {
-        case UP: return "UP";
-        case DOWN: return "DOWN";
-        case OVER: return "OVER";
-        case HIT: return "HIT";
+        case MOUSESTATE_UP: return "UP";
+        case MOUSESTATE_DOWN: return "DOWN";
+        case MOUSESTATE_OVER: return "OVER";
+        case MOUSESTATE_HIT: return "HIT";
         default: return "UNKNOWN (error?)";
     }
 }
