@@ -25,73 +25,45 @@
 #include "VM.h"
 #include "namedStrings.h"
 #include "as_value.h"
+#include "asNamespace.h"
+#include "Global_as.h"
+
+#ifdef ENABLE_AVM2
+#include "asMethod.h"
+#include "abc_function.h"
+#endif
 
 namespace gnash {
-#define STV(x) VM::get().getStringTable().value(x).c_str()
-
-asMethod::asMethod():mBody()
-{
-//TODO
-}
-
-void
-asMethod::setOwner(asClass *pOwner)
-{
-	mPrototype->set_member(NSV::PROP_PROTOTYPE, pOwner->getPrototype());
-}
-
-void
-asMethod::setReturnType(asClass */*type*/)
-{
-	/* No-op */
-}
 
+#ifdef ENABLE_AVM2    
 bool
-asMethod::addValue(string_table::key name, asNamespace *ns, boost::uint32_t slotId,
-	asClass *type, as_value& val, bool isconst)
+asClass::addValue(string_table::key name, asNamespace *ns,
+        boost::uint32_t slotId, asClass *type, as_value& val, bool isconst,
+        bool isstatic)
 {
-	if (val.is_object())
-		val.to_object()->set_member(string_table::key(NSV::INTERNAL_TYPE), 
+    Global_as* g = VM::get().getGlobal();
+
+    if (val.is_object()) {
+		val.to_object(*g)->set_member(NSV::INTERNAL_TYPE, 
 			std::size_t(type->getName()));
+    }
 
 	string_table::key nsname = ns ? ns->getURI() : string_table::key(0);
 
-	int flags = as_prop_flags::dontDelete;
-
+	int flags = PropFlags::dontDelete;
 	if (isconst)
-		flags |= as_prop_flags::readOnly;
-
-	mPrototype->init_member(name, val, flags, nsname, slotId);
-	return true;
-}
-
-bool
-asClass::addValue(string_table::key name, asNamespace *ns, boost::uint32_t slotId,
-	asClass *type, as_value& val, bool isconst, bool isstatic)
-{
-	if (val.is_object())
-		val.to_object()->set_member(NSV::INTERNAL_TYPE, 
-			std::size_t(type->getName()));
-
-	string_table::key nsname = ns ? ns->getURI() : string_table::key(0);
-
-	int flags = as_prop_flags::dontDelete;
-	if (isconst)
-		flags |= as_prop_flags::readOnly;
+		flags |= PropFlags::readOnly;
 	if (isstatic)
-		flags |= as_prop_flags::staticProp;
+		flags |= PropFlags::staticProp;
 
-	mPrototype->init_member(name, val, flags, nsname, slotId);
+	if(slotId == 0){
+		_prototype->init_member(name, val, flags, nsname);
+	}
+	else{
+		_prototype->init_member(name, val, flags, nsname, slotId);
+	}
 	return true;
 }
-
-bool
-asMethod::addMemberClass(string_table::key name, asNamespace *ns,
-	boost::uint32_t slotId, asClass *type)
-{
-	return addSlot(name, ns, slotId, type);
-}
-
 bool
 asClass::addMemberClass(string_table::key name, asNamespace *ns,
 	boost::uint32_t slotId, asClass *type, bool isstatic)
@@ -99,27 +71,6 @@ asClass::addMemberClass(string_table::key name, asNamespace *ns,
 	return addSlot(name, ns, slotId, type, isstatic);
 }
 
-bool
-asMethod::addSlot(string_table::key name, asNamespace* ns, boost::uint32_t slotId,
-	asClass */*type*/)
-{
-	string_table::key nsname = ns ? ns->getURI() : string_table::key(0);
-	int flags = as_prop_flags::dontDelete;
-
-	mPrototype->init_member(name, as_value(), flags, nsname, slotId);
-	return true;
-}
-
-bool
-asMethod::addSlotFunction(string_table::key name, asNamespace *ns,
-	boost::uint32_t slotId, asMethod *method)
-{
-	asClass a;
-	a.setName(NSV::CLASS_FUNCTION);
-	as_value b(method->getPrototype());
-	return addValue(name, ns, slotId, &a, b, false);
-}
-
 bool
 asClass::addSlotFunction(string_table::key name, asNamespace *ns,
 	boost::uint32_t slotId, asMethod *method, bool isstatic)
@@ -131,43 +82,35 @@ asClass::addSlotFunction(string_table::key name, asNamespace *ns,
 }
 
 bool
-asClass::addSlot(string_table::key name, asNamespace* ns, boost::uint32_t slotId,
-	asClass */*type*/, bool isstatic)
+asClass::addSlot(string_table::key name, asNamespace* ns,
+        boost::uint32_t slotId, asClass* /*type*/, bool /*isstatic*/)
 {
-	string_table::key nsname = ns ? ns->getURI() : string_table::key(0);
-	int flags = as_prop_flags::dontDelete;
-	if (isstatic)
-		flags |= as_prop_flags::staticProp;
+	string_table::key nsname = ns ? ns->getURI() : 0;
 
-	mPrototype->init_member(name, as_value(), flags, nsname, slotId);
+	//TODO: Set flags.
+	if (slotId == 0) {
+		_prototype->init_member(name, as_value(), 0, nsname);
+	}
+	else {
+		_prototype->reserveSlot(ObjectURI(name, nsname), slotId);
+	}
 	return true;
 }
 
-bool
-asMethod::addMethod(string_table::key name, asNamespace* ns, asMethod* method)
+    bool
+asClass::addMethod(string_table::key name, asNamespace* /*ns*/,
+        asMethod* method, bool /*isstatic*/)
 {
-	string_table::key nsname = ns ? ns->getURI() : string_table::key(0);
-	as_value val(method->getPrototype());
+	as_value val = new abc_function(method, getVM(*_prototype).getMachine());
+	_prototype->init_member(name, val);
+//	int flags = PropFlags::readOnly | PropFlags::dontDelete
+//		| PropFlags::dontEnum;
+//	if (isstatic)
+//		flags |= PropFlags::staticProp;
 
-	mPrototype->init_member(name, val, as_prop_flags::readOnly |
-		as_prop_flags::dontDelete | as_prop_flags::dontEnum, nsname);
 	return true;
 }
 
-bool
-asClass::addMethod(string_table::key name, asNamespace* ns, asMethod* method,
-	bool isstatic)
-{
-	string_table::key nsname = ns ? ns->getURI() : string_table::key(0);
-	as_value val(method->getPrototype());
-	int flags = as_prop_flags::readOnly | as_prop_flags::dontDelete
-		| as_prop_flags::dontEnum;
-	if (isstatic)
-		flags |= as_prop_flags::staticProp;
-
-	mPrototype->init_member(name, val, flags, nsname);
-	return true;
-}
 
 bool
 asClass::addGetter(string_table::key name, asNamespace *ns, asMethod *method,
@@ -175,16 +118,16 @@ asClass::addGetter(string_table::key name, asNamespace *ns, asMethod *method,
 {
 	string_table::key nsname = ns ? ns->getURI() : string_table::key(0);
 
-	Property *getset = mPrototype->getOwnProperty(name, nsname);
+	Property *getset = _prototype->getOwnProperty(name, nsname);
 
 	if (getset)
 		getset->setGetter(method->getPrototype());
 	else
 	{
-		int flags = as_prop_flags::dontDelete | as_prop_flags::dontEnum;
+		int flags = PropFlags::dontDelete | PropFlags::dontEnum;
 		if (isstatic)
-			flags |= as_prop_flags::staticProp;
-		mPrototype->init_property(name, *method->getPrototype(), 
+			flags |= PropFlags::staticProp;
+		_prototype->init_property(name, *method->getPrototype(), 
 			*method->getPrototype(), flags, nsname);
 	}
 	return true;
@@ -196,65 +139,21 @@ asClass::addSetter(string_table::key name, asNamespace *ns, asMethod *method,
 {
 	string_table::key nsname = ns ? ns->getURI() : string_table::key(0);
 
-	Property *getset = mPrototype->getOwnProperty(name, nsname);
+	Property *getset = _prototype->getOwnProperty(name, nsname);
 
 	if (getset)
 		getset->setSetter(method->getPrototype());
 	else
 	{
-		int flags = as_prop_flags::dontDelete | as_prop_flags::dontEnum;
+		int flags = PropFlags::dontDelete | PropFlags::dontEnum;
 		if (isstatic)
-			flags |= as_prop_flags::staticProp;
-		mPrototype->init_property(name, *method->getPrototype(), 
+			flags |= PropFlags::staticProp;
+		_prototype->init_property(name, *method->getPrototype(), 
 			*method->getPrototype(), flags, nsname);
 	}
 	return true;
 }
 
-bool
-asMethod::addGetter(string_table::key name, asNamespace *ns, asMethod *method)
-{
-	string_table::key nsname = ns ? ns->getURI() : string_table::key(0);
-
-	Property *getset = mPrototype->getOwnProperty(name, nsname);
-
-	if (getset)
-		getset->setGetter(method->getPrototype());
-	else
-	{
-		int flags = as_prop_flags::dontDelete | as_prop_flags::dontEnum;
-		mPrototype->init_property(name, *method->getPrototype(), 
-			*method->getPrototype(), flags, nsname);
-	}
-	return true;
-}
-
-bool
-asMethod::addSetter(string_table::key name, asNamespace *ns, asMethod *method)
-{
-	string_table::key nsname = ns ? ns->getURI() : string_table::key(0);
-
-	Property *getset = mPrototype->getOwnProperty(name, nsname);
-
-	if (getset)
-		getset->setSetter(method->getPrototype());
-	else
-	{
-		int flags = as_prop_flags::dontDelete | as_prop_flags::dontEnum;
-		mPrototype->init_property(name, *method->getPrototype(), 
-			*method->getPrototype(), flags, nsname);
-	}
-	return true;
-}
-
-void
-asNamespace::stubPrototype(string_table::key name)
-{
-	asClass *pClass = VM::get().getClassHierarchy()->newClass();
-	pClass->setName(name);
-	addClass(name, pClass);
-}
-
 #if 0 // TODO
 void
 asClass::buildFromPrototype(as_object *o, string_table::key name,
@@ -335,5 +234,8 @@ asClass::addSlotFunction(string_table::key name, asNamespace *ns,
 		return addBinding(name, asBinding(ns, method, slotId, isstatic));
 	return addStaticBinding(name, asBinding(ns, method, slotId, isstatic));
 }
-#endif /* 0 */
+#endif 
+
+#endif
+
 } /* namespace gnash */
