@@ -22,15 +22,13 @@
 #include "as_object.h"
 #include "as_function.h" // for as_function
 #include "MovieClip.h" // for MOVIECLIP values
-#include "character.h" // for MOVIECLIP values
+#include "DisplayObject.h" // for MOVIECLIP values
 #include "as_environment.h" // for MOVIECLIP values
 #include "VM.h" // for MOVIECLIP values
 #include "movie_root.h" // for MOVIECLIP values
-#include "String_as.h" // for automatic as_value::STRING => String
-#include "Number_as.h" // for automatic as_value::NUMBER => Number
-#include "Boolean_as.h" // for automatic as_value::BOOLEAN => Boolean
 #include "action.h" // for call_method0
-#include "utility.h" // for typeName() and utility::isFinite
+#include "utility.h" // for typeName() 
+#include "GnashNumeric.h"
 #include "namedStrings.h"
 #include "element.h"
 #include "GnashException.h"
@@ -40,8 +38,11 @@
 #include "Date_as.h" // for Date type (readAMF0)
 #include "SimpleBuffer.h"
 #include "StringPredicates.h"
+#include "Global_as.h"
+#include "String_as.h"
 
-#include <cmath> // std::fmod
+#include <boost/shared_ptr.hpp>
+#include <cmath> 
 #include <boost/algorithm/string/case_conv.hpp>
 #include <boost/lexical_cast.hpp>
 #include <boost/format.hpp>
@@ -57,13 +58,13 @@
 //#define GNASH_DEBUG_CONVERSION_TO_PRIMITIVE 1
 
 // Define this macro to make soft references activity verbose
-//#define GNASH_DEBUG_SOFT_REFERENCES
+#define GNASH_DEBUG_SOFT_REFERENCES
 
 // Define this macro to make AMF parsing verbose
-//#define GNASH_DEBUG_AMF_DESERIALIZE
+#define GNASH_DEBUG_AMF_DESERIALIZE 1
 
 // Define this macto to make AMF writing verbose
-//#define GNASH_DEBUG_AMF_SERIALIZE
+// #define GNASH_DEBUG_AMF_SERIALIZE 1
 
 namespace gnash {
 
@@ -144,7 +145,7 @@ parsePositiveInt(const std::string& s, Base base, bool whole = true)
     char c;
 
     // If the cast fails, or if the whole string must be convertible and
-    // some characters are left, throw an exception.
+    // some DisplayObjects are left, throw an exception.
     if (!(is >> target) || (whole && is.get(c))) {
         throw boost::bad_lexical_cast();
     }
@@ -195,14 +196,22 @@ public:
                 str = val.to_string();
             }
             el.reset(new amf::Element(name, str));
-        }
-
-        else if (val.is_bool()) {
+        } else if (val.is_bool()) {
             bool flag = val.to_bool();
             el.reset(new amf::Element(name, flag));
-        }
-
-        else if (val.is_number()) { 
+        } else if (val.is_object()) {
+//            el.reset(new amf::Element(name, flag));
+        } else if (val.is_null()) {
+	    boost::shared_ptr<amf::Element> tmpel(new amf::Element);
+	    tmpel->setName(name);
+	    tmpel->makeNull();
+            el = tmpel;
+        } else if (val.is_undefined()) {
+	    boost::shared_ptr<amf::Element> tmpel(new amf::Element);
+	    tmpel->setName(name);
+	    tmpel->makeUndefined();
+            el = tmpel;
+        } else if (val.is_number()) { 
             double dub;
             if (val.is_undefined()) {
                 dub = 0.0;
@@ -287,7 +296,6 @@ public:
             _error=true;
         }
     }
-
 private:
 
     bool _allowStrict;
@@ -371,6 +379,11 @@ as_value::to_string() const
 		case AS_FUNCTION:
 		case OBJECT:
 		{
+            as_object* obj = m_type == AS_FUNCTION ? getFun().get() :
+                                                     getObj().get();
+            String_as* s;
+            if (isNativeType(obj, s)) return s->value();
+
 			try
 			{
 				as_value ret = to_primitive(STRING);
@@ -378,9 +391,7 @@ as_value::to_string() const
 				// specification, but seems required for compatibility with the
 				// reference player.
 #if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
-				log_debug(" %s.to_primitive(STRING) returned %s", 
-						*this,
-						ret);
+				log_debug(" %s.to_primitive(STRING) returned %s", *this, ret);
 #endif
 				if ( ret.is_string() ) return ret.to_string();
 			}
@@ -421,34 +432,29 @@ as_value::to_string_versioned(int version) const
 	return to_string();
 }
 
+/// This is only used in AVM2.
 primitive_types
 as_value::ptype() const
 {
-	VM& vm = VM::get();
-	int swfVersion = vm.getSWFVersion();
 
 	switch (m_type)
 	{
-	case STRING: return PTYPE_STRING;
-	case NUMBER: return PTYPE_NUMBER;
-	case AS_FUNCTION:
-	case UNDEFINED:
-	case NULLTYPE:
-	case MOVIECLIP:
-		return PTYPE_NUMBER;
-	case OBJECT:
-	{
-		as_object* obj = getObj().get();
-		// Date objects should return TYPE_STRING (but only from SWF6 up)
-		// See ECMA-262 8.6.2.6
-		if ( swfVersion > 5 && obj->isDateObject() ) return PTYPE_STRING;
-		return PTYPE_NUMBER;
-	}
-	case BOOLEAN:
-		return PTYPE_BOOLEAN;
-	default:
-		break; // Should be only exceptions here.
-	}
+        case STRING: return PTYPE_STRING;
+        case NUMBER: return PTYPE_NUMBER;
+        case AS_FUNCTION:
+        case UNDEFINED:
+        case NULLTYPE:
+        case MOVIECLIP:
+            return PTYPE_NUMBER;
+        case OBJECT:
+        {
+            return PTYPE_NUMBER;
+        }
+        case BOOLEAN:
+            return PTYPE_BOOLEAN;
+        default:
+            break; // Should be only exceptions here.
+        }
 	return PTYPE_NUMBER;
 }
 
@@ -461,10 +467,10 @@ as_value::to_primitive() const
 
 	AsType hint = NUMBER;
 
-	if ( m_type == OBJECT && swfVersion > 5 && getObj()->isDateObject() )
-	{
-		hint = STRING;
-	}
+	if (m_type == OBJECT && swfVersion > 5) {
+        Date_as* d;
+        if (isNativeType(getObj().get(), d)) hint = STRING;
+    }
 
 	return to_primitive(hint);
 }
@@ -477,12 +483,13 @@ as_value::convert_to_primitive()
 
 	AsType hint = NUMBER;
 
-	if ( m_type == OBJECT && swfVersion > 5 && getObj()->isDateObject() )
-	{
-		hint = STRING;
-	}
+	if (m_type == OBJECT && swfVersion > 5) {
+        Date_as* d;
+        if (isNativeType<Date_as>(getObj().get(), d)) hint = STRING;
+    }
 
-	return convert_to_primitive(hint);
+    *this = to_primitive(hint);
+    return *this;
 }
 
 // Conversion to primitive value.
@@ -511,7 +518,7 @@ as_value::to_primitive(AsType hint) const
 		if ( m_type == OBJECT ) obj = getObj().get();
 		else obj = getFun().get();
 
-		if ( (!obj->get_member(NSV::PROP_VALUE_OF, &method)) || (!method.is_object()) ) // ECMA says ! is_object()
+		if ((!obj->get_member(NSV::PROP_VALUE_OF, &method)) || (!method.is_function())) 
 		{
 #if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
 			log_debug(" valueOf not found");
@@ -543,14 +550,6 @@ as_value::to_primitive(AsType hint) const
 		// text representation for that object is used
 		// instead.
 		//
-		if ( ! obj->useCustomToString() )
-		{
-#if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
-			log_debug(" not using custom toString");
-#endif
-			return as_value(obj->get_text_value());
-		}
-
 		if ( (!obj->get_member(NSV::PROP_TO_STRING, &method)) ||
                 (!method.is_function()) ) // ECMA says ! is_object()
 		{
@@ -570,8 +569,8 @@ as_value::to_primitive(AsType hint) const
 
 	assert(obj);
 
-	as_environment env(obj->getVM());
-	as_value ret = call_method0(method, &env, obj);
+	as_environment env(getVM(*obj));
+	as_value ret = call_method0(method, env, obj);
 #if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
 	log_debug("to_primitive: method call returned %s", ret);
 #endif
@@ -585,112 +584,6 @@ as_value::to_primitive(AsType hint) const
 
 }
 
-// Conversion to primitive value.
-as_value&
-as_value::convert_to_primitive(AsType hint) 
-{
-	if ( m_type != OBJECT && m_type != AS_FUNCTION ) return *this; 
-	//if ( ! is_object() ) return *this; // include MOVIECLIP !!
-
-#if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
-	log_debug("to_primitive(%s)", hint==NUMBER ? "NUMBER" : "STRING");
-#endif 
-
-	// TODO: implement as_object::DefaultValue (ECMA-262 - 8.6.2.6)
-
-	as_value method;
-	as_object* obj = NULL;
-
-	if (hint == NUMBER)
-	{
-		if ( m_type == MOVIECLIP )
-		{
-			set_double(NaN);
-			return *this;
-		}
-		if ( m_type == OBJECT ) obj = getObj().get();
-		else obj = getFun().get();
-
-		if ( (!obj->get_member(NSV::PROP_VALUE_OF, &method)) ||
-                (!method.is_object()) ) // ECMA says ! is_object()
-		{
-#if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
-			log_debug(" valueOf not found");
-#endif
-			// Returning undefined here instead of throwing
-			// a TypeError passes tests in actionscript.all/Object.as
-			// and many swfdec tests, with no new failures (though
-			// perhaps we aren't testing enough).
-			set_undefined();
-			return *this;
-            
-		}
-	}
-	else
-	{
-		assert(hint==STRING);
-
-		if ( m_type == MOVIECLIP )
-		{
-			set_string(getCharacterProxy().getTarget());
-			return *this;
-		}
-
-		if ( m_type == OBJECT ) obj = getObj().get();
-		else obj = getFun().get();
-
-		// @@ Moock says, "the value that results from
-		// calling toString() on the object".
-		//
-		// When the toString() method doesn't exist, or
-		// doesn't return a valid number, the default
-		// text representation for that object is used
-		// instead.
-		//
-		if ( ! obj->useCustomToString() )
-		{
-#if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
-			log_debug(" not using custom toString");
-#endif
-			set_string(obj->get_text_value());
-			return *this;
-		}
-
-		if ( (!obj->get_member(NSV::PROP_TO_STRING, &method)) || 
-                (!method.is_function()) ) // ECMA says ! is_object()
-		{
-#if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
-			log_debug(" toString not found");
-#endif
-			if ( (!obj->get_member(NSV::PROP_VALUE_OF, &method)) || 
-                    (!method.is_function()) ) // ECMA says ! is_object()
-			{
-#if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
-				log_debug(" valueOf not found");
-#endif
-				throw ActionTypeError();
-			}
-		}
-	}
-
-	assert(obj);
-
-	as_environment env(obj->getVM());
-	as_value ret = call_method0(method, &env, obj);
-#if GNASH_DEBUG_CONVERSION_TO_PRIMITIVE
-	log_debug("to_primitive: method call returned %s", ret);
-#endif
-	if ( ret.m_type == OBJECT || ret.m_type == AS_FUNCTION ) // not a primitive 
-	{
-		throw ActionTypeError();
-	}
-
-	*this = ret;
-
-	return *this;
-}
-
-
 bool
 as_value::parseNonDecimalInt(const std::string& s, double& d, bool whole)
 {
@@ -749,7 +642,7 @@ as_value::to_number() const
             if (swfversion <= 4)
             {
                 // For SWF4, any valid number before non-numerical
-                // characters is returned, including exponent, positive
+                // DisplayObjects is returned, including exponent, positive
                 // and negative signs and whitespace before.
                 double d = 0;
                 std::istringstream is(s);
@@ -772,7 +665,7 @@ as_value::to_number() const
                 // Valid for SWF5 and above.
                 //
                 // boost::lexical_cast is remarkably inflexible and 
-                // fails for anything that has non-numerical characters.
+                // fails for anything that has non-numerical DisplayObjects.
                 // Fortunately, actionscript is equally inflexible.
                 std::string::size_type pos;
                 if ((pos = s.find_first_not_of(" \r\n\t")) 
@@ -846,24 +739,30 @@ as_value::to_number() const
     }
 }
 
-std::auto_ptr<amf::Element>
+boost::shared_ptr<amf::Element>
 as_value::to_element() const
 {
     VM& vm = VM::get();
     //int swfVersion = vm.getSWFVersion();
-    std::auto_ptr<amf::Element> el ( new amf::Element );
-    boost::intrusive_ptr<as_object> ptr = to_object();
+    boost::shared_ptr<amf::Element> el ( new amf::Element );
+    boost::intrusive_ptr<as_object> ptr = to_object(*vm.getGlobal());
 
     switch (m_type) {
+      case UNDEFINED:
+	  el->makeUndefined();
+	  break;
+      case NULLTYPE:
+	  el->makeNull();
+	  break;
+      case BOOLEAN:
+	  el->makeBoolean(getBool());
+	  break;
       case  STRING:
 	  el->makeString(getStr());
 	  break;
       case NUMBER:
 	  el->makeNumber(getNum());
 	  break;
-      case BOOLEAN:
-	  el->makeBoolean(getBool());
-	  break;
       case OBJECT:
       {
 	  el->makeObject();
@@ -894,7 +793,7 @@ as_value::to_int() const
 {
 	double d = to_number();
 
-	if ( ! utility::isFinite(d) ) return 0;
+	if (!isFinite(d)) return 0;
 
     return truncateToInt(d);
 }
@@ -1004,7 +903,7 @@ as_value::to_bool() const
 	
 // Return value as an object.
 boost::intrusive_ptr<as_object>
-as_value::to_object() const
+as_value::to_object(Global_as& global) const
 {
 	typedef boost::intrusive_ptr<as_object> ptr;
 
@@ -1017,18 +916,16 @@ as_value::to_object() const
 			return getFun().get();
 
 		case MOVIECLIP:
-			// FIXME: update when to_sprite will return
-			//        an intrusive_ptr directly
-			return ptr(to_character());
+			return ptr(toDisplayObject());
 
 		case STRING:
-			return init_string_instance(getStr());
+			return global.createString(getStr());
 
 		case NUMBER:
-			return init_number_instance(getNum());
+			return global.createNumber(getNum());
 
 		case BOOLEAN:
-			return init_boolean_instance(getBool());
+			return global.createBoolean(getBool());
 
 		default:
 			// Invalid to convert exceptions.
@@ -1041,13 +938,13 @@ as_value::to_sprite(bool allowUnloaded) const
 {
 	if ( m_type != MOVIECLIP ) return 0;
 
-	character *ch = getCharacter(allowUnloaded);
+	DisplayObject *ch = getCharacter(allowUnloaded);
 	if ( ! ch ) return 0;
 	return ch->to_movie();
 }
 
-character*
-as_value::to_character(bool allowUnloaded) const
+DisplayObject*
+as_value::toDisplayObject(bool allowUnloaded) const
 {
 	if ( m_type != MOVIECLIP ) return NULL;
 
@@ -1057,11 +954,11 @@ as_value::to_character(bool allowUnloaded) const
 void
 as_value::set_sprite(MovieClip& sprite)
 {
-	set_character(sprite);
+	setDisplayObject(sprite);
 }
 
 void
-as_value::set_character(character& sprite)
+as_value::setDisplayObject(DisplayObject& sprite)
 {
 	m_type = MOVIECLIP;
 	_value = CharacterProxy(&sprite);
@@ -1128,6 +1025,13 @@ as_value::set_null()
 	_value = boost::blank();
 }
 
+void
+as_value::set_unsupported()
+{
+	m_type = UNSUPPORTED;
+	_value = boost::blank();
+}
+
 void
 as_value::set_as_object(as_object* obj)
 {
@@ -1136,10 +1040,10 @@ as_value::set_as_object(as_object* obj)
 		set_null();
 		return;
 	}
-	character* sp = obj->to_character();
+	DisplayObject* sp = obj->toDisplayObject();
 	if ( sp )
 	{
-		set_character(*sp);
+		setDisplayObject(*sp);
 		return;
 	}
 	as_function* func = obj->to_function();
@@ -1234,7 +1138,7 @@ as_value::equals(const as_value& v) const
     if (m_type == NUMBER && v.m_type == STRING)
     {
         double n = v.to_number();
-        if ( ! utility::isFinite(n) ) return false;
+        if (!isFinite(n)) return false;
         return equalsSameType(n);
     }
 
@@ -1243,8 +1147,8 @@ as_value::equals(const as_value& v) const
     if (v.m_type == NUMBER && m_type == STRING)
     {
         double n = to_number();
-        if ( ! utility::isFinite(n) ) return false;
-            return v.equalsSameType(n); 
+        if (!isFinite(n)) return false;
+        return v.equalsSameType(n); 
     }
 
     // 18. If Type(x) is Boolean, return the result of the comparison ToNumber(x) == y.
@@ -1414,10 +1318,10 @@ as_value::typeOf() const
 
 		case as_value::MOVIECLIP:
 		{
-			character* ch = getCharacter();
+			DisplayObject* ch = getCharacter();
 			if ( ! ch ) return "movieclip"; // dangling
 			if ( ch->to_movie() ) return "movieclip"; // bound to movieclip
-			return "object"; // bound to some other character
+			return "object"; // bound to some other DisplayObject
 		}
 
 		case as_value::NULLTYPE:
@@ -1459,7 +1363,7 @@ as_value::equalsSameType(const as_value& v) const
 			return _value == v._value;
 
 		case MOVIECLIP:
-			return to_character() == v.to_character(); 
+			return toDisplayObject() == v.toDisplayObject(); 
 
 		case NUMBER:
 		{
@@ -1533,7 +1437,7 @@ as_value::toDebugString() const
 			const CharacterProxy& sp = getCharacterProxy();
 			if ( sp.isDangling() )
 			{
-				character* rebound = sp.get();
+				DisplayObject* rebound = sp.get();
 				if ( rebound )
 				{
 				    ret = boost::format("[rebound %s(%s):%p]") % 
@@ -1542,13 +1446,13 @@ as_value::toDebugString() const
 				}
 				else
 				{
-				    ret = boost::format("[dangling character:%s]") % 
+				    ret = boost::format("[dangling DisplayObject:%s]") % 
 					    sp.getTarget();
 				}
 			}
 			else
 			{
-				character* ch = sp.get();
+				DisplayObject* ch = sp.get();
 				ret = boost::format("[%s(%s):%p]") % typeName(*ch) %
 				                sp.getTarget() % static_cast<void*>(ch);
 			}
@@ -1757,7 +1661,7 @@ as_value::SpritePtr
 as_value::getSprite(bool allowUnloaded) const
 {
 	assert(m_type == MOVIECLIP);
-	character* ch = getCharacter(allowUnloaded);
+	DisplayObject* ch = getCharacter(allowUnloaded);
 	if ( ! ch ) return 0;
 	return ch->to_movie();
 }
@@ -1836,10 +1740,20 @@ as_value::as_value(const amf::Element& el)
 	:
 	m_type(UNDEFINED)
 {
+//     GNASH_REPORT_FUNCTION;    
+//     el.dump();
+    
     VM& vm = VM::get();
     string_table& st = vm.getStringTable();
-    
+
     switch (el.getType()) {
+      case amf::Element::NOTYPE:
+      {
+#ifdef GNASH_DEBUG_AMF_DESERIALIZE
+	  log_debug("as_value(Element&) : AMF type NO TYPE!");
+#endif
+	  break;
+      }
       case amf::Element::NULL_AMF0:
       {
 #ifdef GNASH_DEBUG_AMF_DESERIALIZE
@@ -1893,8 +1807,16 @@ as_value::as_value(const amf::Element& el)
 #ifdef GNASH_DEBUG_AMF_DESERIALIZE
             log_debug("as_value(Element&) : AMF type STRING");
 #endif
-            std::string str = el.to_string();
-            set_string(str);
+	    std::string str;
+	    // If there is data, convert it to a string for the as_value
+	    if (el.getDataSize() != 0) {
+		str = el.to_string();
+		// Element's store the property name as the name, not as data.
+	    } else if (el.getNameSize() != 0) {
+		str = el.getName();
+	    }
+	    
+	    set_string(str);
             break;
       }
 
@@ -1904,18 +1826,22 @@ as_value::as_value(const amf::Element& el)
 #ifdef GNASH_DEBUG_AMF_DESERIALIZE
           log_debug("as_value(Element&) : AMF type OBJECT");
 #endif
-          as_object* obj = new as_object(getObjectInterface());
+	  as_object* obj = new as_object(getObjectInterface());
           if (el.propertySize()) {
               for (size_t i=0; i < el.propertySize(); i++) {
-              const boost::shared_ptr<amf::Element> prop = el.getProperty(i);
-              if (prop == 0) {
-                  break;
-              } else {
-                  obj->set_member(st.find(prop->getName()), as_value(*prop));
-              }
+		  const boost::shared_ptr<amf::Element> prop = el.getProperty(i);
+		  if (prop == 0) {
+		      break;
+		  } else {
+		      if (prop->getNameSize() == 0) {
+			  log_debug("%s:(%d) Property has no name!", __PRETTY_FUNCTION__, __LINE__);
+		      } else {
+			  obj->set_member(st.find(prop->getName()), as_value(*prop));
+		      }
+		  }
               }
           }
-          set_as_object(obj);
+	  set_as_object(obj);
           break;
       }
 
@@ -1931,12 +1857,12 @@ as_value::as_value(const amf::Element& el)
           Array_as* obj = new Array_as;
           if (el.propertySize()) {
               for (size_t i=0; i < el.propertySize(); i++) {
-              const boost::shared_ptr<amf::Element> prop = el.getProperty(i);
-              if (prop == 0) {
-                  break;
-              } else {
-                  obj->set_member(st.find(prop->getName()), as_value(*prop));
-              }
+		  const boost::shared_ptr<amf::Element> prop = el.getProperty(i);
+		  if (prop == 0) {
+		      break;
+		  } else {
+		      obj->set_member(st.find(prop->getName()), as_value(*prop));
+		  }
               }
           }
           set_as_object(obj);
@@ -1958,7 +1884,11 @@ as_value::as_value(const amf::Element& el)
               if (prop == 0) {
                   break;
               } else {
-                  obj->set_member(st.find(prop->getName()), as_value(*prop));
+		  if (prop->getNameSize() == 0) {
+		      log_debug("%s:(%d) Property has no name!", __PRETTY_FUNCTION__, __LINE__);
+		  } else {
+		      obj->set_member(st.find(prop->getName()), as_value(*prop));
+		  }
               }
           }
           
@@ -1974,14 +1904,23 @@ as_value::as_value(const amf::Element& el)
 
       case amf::Element::DATE_AMF0:
       {
-        log_unimpl("DATE Element to as_value");
-        //if (swfVersion > 5) m_type = STRING;
-        break;
+#ifdef GNASH_DEBUG_AMF_DESERIALIZE
+	  log_debug("as_value(Element&) : AMF type DATE");
+#endif
+	  double num = el.to_number();
+	  set_double(num);
+	  break;
       }
-
+      //if (swfVersion > 5) m_type = STRING;
+      
       case amf::Element::UNSUPPORTED_AMF0:
-          log_unimpl("Unsupported data type is not supported yet");
-          break;
+      {
+#ifdef GNASH_DEBUG_AMF_DESERIALIZE
+	  log_debug("as_value(Element&) : AMF type UNSUPPORTED");
+#endif
+	  set_unsupported();
+	  break;
+      }
       case amf::Element::RECORD_SET_AMF0:
           log_unimpl("Record Set data type is not supported yet");
           break;
@@ -2034,14 +1973,10 @@ as_value::newAdd(const as_value& op2)
 		convert_to_string_versioned(version);
 		string_concat(v2.to_string_versioned(version));
 	}
-	else
-	{
+	else {
 		// use numeric semantic
 		double v2num = v2.to_number();
-		//log_debug(_("v2 num = %g"), v2num);
 		double v1num = to_number();
-		//log_debug(_("v1 num = %g"), v1num);
-
 		set_double(v2num + v1num); 
 
 	}
@@ -2415,8 +2350,15 @@ amf0_read_value(const boost::uint8_t *&b, const boost::uint8_t *end,
 #ifdef GNASH_DEBUG_AMF_DESERIALIZE
 			log_debug("amf0 read date: %e", dub);
 #endif
-            as_object* obj = new Date_as(dub);
-			ret.set_as_object(obj);
+
+            Global_as* gl = vm.getGlobal();
+            as_function* ctor = gl->getMember(NSV::CLASS_DATE).to_as_function();
+            if (ctor) {
+                fn_call::Args args;
+                args += dub;
+                ret.set_as_object(ctor->constructInstance(as_environment(vm),
+                            args));
+            }
 
 			if (b + 2 > end) {
 				log_error(_("AMF0 read: premature end of input reading "
@@ -2470,55 +2412,17 @@ as_value::writeAMF0(SimpleBuffer& buf,
 
         case OBJECT:
         {
-            as_object* obj = to_object().get();
+            as_object* obj = to_object(*vm.getGlobal()).get();
             assert(obj);
             OffsetTable::iterator it = offsetTable.find(obj);
-            if ( it == offsetTable.end() )
-            {
-                size_t idx = offsetTable.size()+1; // 1 for the first, etc...
+            if (it == offsetTable.end()) {
+                size_t idx = offsetTable.size() + 1; // 1 for the first, etc...
                 offsetTable[obj] = idx;
-
-                Array_as* ary = dynamic_cast<Array_as*>(obj);
-                if ( ary )
+                
+                Date_as* date;
+                if (isNativeType(obj, date))
                 {
-                    size_t len = ary->size();
-                    if ( allowStrict && ary->isStrict() )
-                    {
-#ifdef GNASH_DEBUG_AMF_SERIALIZE
-                        log_debug(_("writeAMF0: serializing array of %d "
-                                    "elements as STRICT_ARRAY (index %d)"),
-                                    len, idx);
-#endif
-                        buf.appendByte(amf::Element::STRICT_ARRAY_AMF0);
-                        buf.appendNetworkLong(len);
-
-                        as_value elem;
-                        for (size_t i=0; i<len; ++i)
-                        {
-                            elem = ary->at(i);
-                            if ( ! elem.writeAMF0(buf, offsetTable, vm, allowStrict) )
-                            {
-                                log_error("Problems serializing strict array member %d=%s", i, elem);
-                                return false;
-                            }
-                        }
-                        return true;
-                    }
-                    else 
-                    {
-#ifdef GNASH_DEBUG_AMF_SERIALIZE
-                        log_debug(_("writeAMF0: serializing array of %d "
-                                    "elements as ECMA_ARRAY (index %d) [allowStrict:%d, isStrict:%d]"),
-                                    len, idx, allowStrict, ary->isStrict());
-#endif
-                        buf.appendByte(amf::Element::ECMA_ARRAY_AMF0);
-                        buf.appendNetworkLong(len);
-                    }
-                }
-                else if ( obj->isDateObject() )
-                {
-                    const Date_as& date = dynamic_cast<const Date_as&>(*obj);
-                    double d = date.getTimeValue(); 
+                    double d = date->getTimeValue(); 
 #ifdef GNASH_DEBUG_AMF_SERIALIZE
                     log_debug(_("writeAMF0: serializing date object "
                                 "with index %d and value %g"), idx, d);
@@ -2535,8 +2439,44 @@ as_value::writeAMF0(SimpleBuffer& buf,
 
                     return true;
                 }
-                else
-                {
+
+                Array_as* ary = dynamic_cast<Array_as*>(obj);
+                if (ary) {
+                    const size_t len = ary->size();
+                    if (allowStrict && ary->isStrict()) {
+#ifdef GNASH_DEBUG_AMF_SERIALIZE
+                        log_debug(_("writeAMF0: serializing array of %d "
+                                    "elements as STRICT_ARRAY (index %d)"),
+                                    len, idx);
+#endif
+                        buf.appendByte(amf::Element::STRICT_ARRAY_AMF0);
+                        buf.appendNetworkLong(len);
+
+                        as_value elem;
+                        for (size_t i = 0; i < len; ++i) {
+                            elem = ary->at(i);
+                            if (!elem.writeAMF0(buf, offsetTable, vm,
+                                        allowStrict)) {
+                                log_error("Problems serializing strict array "
+                                        "member %d=%s", i, elem);
+                                return false;
+                            }
+                        }
+                        return true;
+                    }
+
+                    // A normal array.
+#ifdef GNASH_DEBUG_AMF_SERIALIZE
+                    log_debug(_("writeAMF0: serializing array of %d "
+                                "elements as ECMA_ARRAY (index %d) "
+                                "[allowStrict:%d, isStrict:%d]"),
+                                len, idx, allowStrict, ary->isStrict());
+#endif
+                    buf.appendByte(amf::Element::ECMA_ARRAY_AMF0);
+                    buf.appendNetworkLong(len);
+                }
+                else {
+                    // It's a simple object
 #ifdef GNASH_DEBUG_AMF_SERIALIZE
                     log_debug(_("writeAMF0: serializing object (or function) "
                                 "with index %d"), idx);
@@ -2546,8 +2486,7 @@ as_value::writeAMF0(SimpleBuffer& buf,
 
                 PropsBufSerializer props(buf, vm, offsetTable, allowStrict);
                 obj->visitNonHiddenPropertyValues(props);
-                if ( ! props.success() ) 
-                {
+                if (!props.success()) {
                     log_error("Could not serialize object");
                     return false;
                 }
@@ -2555,17 +2494,13 @@ as_value::writeAMF0(SimpleBuffer& buf,
                 buf.appendByte(amf::Element::OBJECT_END_AMF0);
                 return true;
             }
-            else // object already seen
-            {
-                size_t idx = it->second;
+            size_t idx = it->second;
 #ifdef GNASH_DEBUG_AMF_SERIALIZE
-                log_debug(_("writeAMF0: serializing object (or function) "
-                            "as reference to %d"), idx);
+            log_debug(_("writeAMF0: serializing object (or function) "
+                        "as reference to %d"), idx);
 #endif
-                buf.appendByte(amf::Element::REFERENCE_AMF0);
-                buf.appendNetworkShort(idx);
-                return true;
-            }
+            buf.appendByte(amf::Element::REFERENCE_AMF0);
+            buf.appendNetworkShort(idx);
             return true;
         }
 
