@@ -24,13 +24,14 @@
 namespace gnash {
 
 class as_object;
+class ObjectURI;
+class Global_as;
 
 /// Initialize the global Boolean class
-void boolean_class_init(as_object& global);
+void boolean_class_init(as_object& global, const ObjectURI& uri);
+
+void registerBooleanNative(as_object& global);
 
-/// Return a Boolean instance (in case the core lib needs it)
-boost::intrusive_ptr<as_object> init_boolean_instance(bool val);
-  
 } // end of gnash namespace
 
 #endif
