@@ -25,6 +25,7 @@
 #include "as_object.h" // for inheritance
 #include "log.h"
 #include "fn_call.h"
+#include "Global_as.h"
 #include "smart_ptr.h" // for boost intrusive_ptr
 #include "builtin_function.h" // need builtin_function
 #include "GnashException.h" // for ActionException
@@ -39,62 +40,28 @@ namespace {
     as_value error_toString(const fn_call& fn);
     as_value error_ctor(const fn_call& fn);
 
-    as_object* getErrorInterface();
     void attachErrorInterface(as_object& o);
 }
 
 
-class Error_as: public as_object
-{
-
-public:
-
-	Error_as()
-		:
-		as_object(getErrorInterface())
-	{}
-
-};
-
-
 // extern 
-void Error_class_init(as_object& where)
+void
+Error_class_init(as_object& where, const ObjectURI& uri)
 {
-	// This is going to be the Error "class"/"function"
-	// in the 'where' package
-	boost::intrusive_ptr<builtin_function> cl;
-	cl = new builtin_function(&error_ctor, getErrorInterface());
-
-	// Register _global.Error
-	where.init_member("Error", cl.get());
+    registerBuiltinClass(where, error_ctor, attachErrorInterface, 0, uri);   
 }
 
 
 namespace {
 
-as_object*
-getErrorInterface()
-{
-	static boost::intrusive_ptr<as_object> o;
-
-	if ( ! o )
-	{
-		o = new as_object(getObjectInterface());
-		VM::get().addStatic(o.get());
-
-		attachErrorInterface(*o);
-
-	}
-
-	return o.get();
-}
-
-
 void
 attachErrorInterface(as_object& o)
 {
+    Global_as* gl = getGlobal(o);
     int flags = 0;
-    o.init_member("toString", new builtin_function(error_toString), flags);
+
+    o.init_member("toString", gl->createFunction(error_toString), flags);
+    
     o.init_member("message", "Error", flags);
     o.init_member("name", "Error", flags);
 }
@@ -103,9 +70,9 @@ attachErrorInterface(as_object& o)
 as_value
 error_toString(const fn_call& fn)
 {
- 	boost::intrusive_ptr<Error_as> ptr = ensureType<Error_as>(fn.this_ptr);
+ 	boost::intrusive_ptr<as_object> ptr = ensureType<as_object>(fn.this_ptr);
 
-    string_table& st = ptr->getVM().getStringTable();
+    string_table& st = getStringTable(*ptr);
     as_value message;
     ptr->get_member(st.find("message"), &message);
 
@@ -119,15 +86,15 @@ error_ctor(const fn_call& fn)
 
     if (!fn.isInstantiation()) return as_value();
 
-	boost::intrusive_ptr<Error_as> err = new Error_as;
-	
-    string_table& st = err->getVM().getStringTable();
-    if (fn.nargs > 0)
-	{
+    as_object* err = fn.this_ptr;
+
+    string_table& st = getStringTable(fn);
+
+    if (fn.nargs) {
 		err->set_member(st.find("message"), fn.arg(0));
 	}
 
-	return as_value(err.get()); // will keep alive
+	return as_value();
 }
 
 }
