@@ -24,11 +24,14 @@
 #include "StreamProvider.h"
 #include "URL.h"
 #include "namedStrings.h"
+#include "movie_root.h"
 #include "VM.h"
 #include "builtin_function.h"
-#include "timers.h"
+#include "NativeFunction.h"
 #include "utf8.h"
 #include "fn_call.h"
+#include "GnashAlgorithm.h"
+#include "Global_as.h"
 
 #include <sstream>
 #include <map>
@@ -42,29 +45,19 @@ namespace {
     as_value loadableobject_sendAndLoad(const fn_call& fn);
 }
 
-LoadableObject::LoadableObject()
+LoadableObject::LoadableObject(as_object* owner)
     :
+    ActiveRelay(owner),
     _bytesLoaded(-1),
-    _bytesTotal(-1),
-    _loadCheckerTimer(0)
+    _bytesTotal(-1)
 {
 }    
 
 
 LoadableObject::~LoadableObject()
 {
-    for (LoadThreadList::iterator it = _loadThreads.begin(),
-            e = _loadThreads.end(); it != e; ++it)
-    {
-        // Joins the thread
-        delete *it;
-    }
-
-    if ( _loadCheckerTimer )
-    {
-        _vm.getRoot().clear_interval_timer(_loadCheckerTimer);
-    }
-
+    deleteAllChecked(_loadThreads);
+    getRoot(owner()).removeAdvanceCallback(this);
 }
 
 
@@ -72,33 +65,32 @@ void
 LoadableObject::send(const std::string& urlstr, const std::string& target,
         bool post)
 {
-    movie_root& m = _vm.getRoot();
+    movie_root& m = getRoot(owner());
 
     // Encode the object for HTTP. If post is true,
     // XML should not be encoded. LoadVars is always
     // encoded.
     // TODO: test properly.
-    std::ostringstream data;
-    toString(data, !post);
+    const std::string& str = as_value(&owner()).to_string();
 
     // Only GET and POST are possible here.
     MovieClip::VariablesMethod method = post ? MovieClip::METHOD_POST :
                                                MovieClip::METHOD_GET;
 
-    m.getURL(urlstr, target, data.str(), method);
+    m.getURL(urlstr, target, str, method);
 
 }
 
 
 void
-LoadableObject::sendAndLoad(const std::string& urlstr,
-                            as_object& target, bool post)
+LoadableObject::sendAndLoad(const std::string& urlstr, as_object& target,
+        bool post)
 {
 
     /// All objects get a loaded member, set to false.
     target.set_member(NSV::PROP_LOADED, false);
 
-    const RunInfo& ri = _vm.getRoot().runInfo();
+    const RunResources& ri = getRunResources(owner());
 	URL url(urlstr, ri.baseURL());
 
 	std::auto_ptr<IOChannel> str;
@@ -108,13 +100,13 @@ LoadableObject::sendAndLoad(const std::string& urlstr,
 
         NetworkAdapter::RequestHeaders headers;
 
-        if (get_member(NSV::PROP_uCUSTOM_HEADERS, &customHeaders))
+        if (owner().get_member(NSV::PROP_uCUSTOM_HEADERS, &customHeaders))
         {
 
             /// Read in our custom headers if they exist and are an
             /// array.
             Array_as* array = dynamic_cast<Array_as*>(
-                            customHeaders.to_object().get());
+                            customHeaders.to_object(*getGlobal(target)).get());
                             
             if (array)
             {
@@ -145,7 +137,7 @@ LoadableObject::sendAndLoad(const std::string& urlstr,
 
         as_value contentType;
 
-        if (get_member(NSV::PROP_CONTENT_TYPE, &contentType))
+        if (owner().get_member(NSV::PROP_CONTENT_TYPE, &contentType))
         {
             // This should not overwrite anything set in 
             // LoadVars.addRequestHeader();
@@ -156,35 +148,36 @@ LoadableObject::sendAndLoad(const std::string& urlstr,
         // Convert the object to a string to send. XML should
         // not be URL encoded for the POST method, LoadVars
         // is always URL encoded.
-        std::ostringstream data;
-        toString(data, false);
+        const std::string& strval = as_value(&owner()).to_string();
 
         /// It doesn't matter if there are no request headers.
-        str = ri.streamProvider().getStream(url, data.str(), headers);
+        str = ri.streamProvider().getStream(url, strval, headers);
     }
 	else
     {
         // Convert the object to a string to send. XML should
         // not be URL encoded for the GET method.
-        std::ostringstream data;
-        toString(data, true);
+        const std::string& dataString = as_value(&owner()).to_string();
 
-    	std::string getURL = urlstr + "?" + data.str();
-        log_debug("Using GET method for sendAndLoad: %s", getURL);
-        str = ri.streamProvider().getStream(getURL);
+        // Any data must be added to the existing querystring.
+        if (!dataString.empty()) {
+
+            std::string existingQS = url.querystring();
+            if (!existingQS.empty()) existingQS += "&";
+
+            url.set_querystring(existingQS + dataString);
+        }
+
+        log_debug("Using GET method for sendAndLoad: %s", url.str());
+        str = ri.streamProvider().getStream(url.str());
     }
 
-	if (!str.get()) 
-	{
-		log_error(_("Can't load from %s (security?)"), url.str());
-		return;
-		// TODO: check if this is correct
-		//as_value nullValue; nullValue.set_null();
-		//callMethod(_vm.getStringTable().find(PROPNAME("onData")), nullValue);
-	}
-
 	log_security(_("Loading from url: '%s'"), url.str());
-    target.queueLoad(str);
+	
+    LoadableObject* loadObject;
+    if (isNativeType(&target, loadObject)) {
+        loadObject->queueLoad(str);
+    }
 	
 }
 
@@ -193,23 +186,14 @@ LoadableObject::load(const std::string& urlstr)
 {
     // Set loaded property to false; will be updated (hopefully)
     // when loading is complete.
-	set_member(NSV::PROP_LOADED, false);
+	owner().set_member(NSV::PROP_LOADED, false);
 
-    const RunInfo& ri = _vm.getRoot().runInfo();
+    const RunResources& ri = getRunResources(owner());
 	URL url(urlstr, ri.baseURL());
 
     // Checks whether access is allowed.
     std::auto_ptr<IOChannel> str(ri.streamProvider().getStream(url));
 
-	if (!str.get()) 
-	{
-		log_error(_("Can't load variables from %s (security?)"), url.str());
-		return;
-		// TODO: check if this is correct
-		//as_value nullValue; nullValue.set_null();
-		//callMethod(_vm.getStringTable().find(PROPNAME("onData")), nullValue);
-	}
-
 	log_security(_("Loading from url: '%s'"), url.str());
     queueLoad(str);
 }
@@ -219,57 +203,42 @@ void
 LoadableObject::queueLoad(std::auto_ptr<IOChannel> str)
 {
 
-    bool startTimer = _loadThreads.empty();
+    if (_loadThreads.empty()) {
+        getRoot(owner()).addAdvanceCallback(this);
+    }
 
-    std::auto_ptr<LoadThread> lt ( new LoadThread(str) );
+    std::auto_ptr<LoadThread> lt (new LoadThread(str));
 
     // we push on the front to avoid invalidating
     // iterators when queueLoad is called as effect
     // of onData invocation.
     // Doing so also avoids processing queued load
     // request immediately
-    // 
     _loadThreads.push_front(lt.release());
 
-    if ( startTimer )
-    {
-        boost::intrusive_ptr<builtin_function> loadsChecker = 
-            new builtin_function(&checkLoads_wrapper);
-
-        std::auto_ptr<Timer> timer(new Timer);
-        timer->setInterval(*loadsChecker, 50, this);
-        _loadCheckerTimer = getVM().getRoot().add_interval_timer(timer, true);
-
-    }
-
     _bytesLoaded = 0;
     _bytesTotal = -1;
 
 }
 
-as_value
-LoadableObject::checkLoads_wrapper(const fn_call& fn)
-{
-	boost::intrusive_ptr<LoadableObject> ptr =
-        ensureType<LoadableObject>(fn.this_ptr);
-	ptr->checkLoads();
-	return as_value();
-}
-
-
 void
-LoadableObject::checkLoads()
+LoadableObject::update()
 {
 
-    if (_loadThreads.empty()) return; // nothing to do
+    if (_loadThreads.empty()) return;
 
     for (LoadThreadList::iterator it=_loadThreads.begin();
             it != _loadThreads.end(); )
     {
         LoadThread* lt = *it;
 
-        if ( lt->completed() )
-        {
+        /// An empty file is the same as a failure.
+        if (lt->failed() || (lt->completed() && !lt->size())) {
+            owner().callMethod(NSV::PROP_ON_DATA, as_value());
+            it = _loadThreads.erase(it);
+            delete lt; 
+        }
+        else if (lt->completed()) {
             size_t dataSize = _bytesTotal = _bytesLoaded = lt->getBytesTotal();
 
             boost::scoped_array<char> buf(new char[dataSize + 1]);
@@ -298,28 +267,38 @@ LoadableObject::checkLoads()
             it = _loadThreads.erase(it);
             delete lt; // supposedly joins the thread...
 
+            string_table& st = getStringTable(owner());
+            owner().set_member(st.find("_bytesLoaded"), _bytesLoaded);
+            owner().set_member(st.find("_bytesTotal"), _bytesTotal);
+            
             // might push_front on the list..
-            callMethod(NSV::PROP_ON_DATA, dataVal);
+            owner().callMethod(NSV::PROP_ON_DATA, dataVal);
 
         }
         else
         {
+            _bytesTotal = lt->getBytesTotal();
             _bytesLoaded = lt->getBytesLoaded();
+            
+            string_table& st = getStringTable(owner());
+            owner().set_member(st.find("_bytesLoaded"), _bytesLoaded);
+            // TODO: should this really be set on each iteration?
+            owner().set_member(st.find("_bytesTotal"), _bytesTotal);
             ++it;
         }
     }
 
-    if ( _loadThreads.empty() ) 
+    if (_loadThreads.empty()) 
     {
-        _vm.getRoot().clear_interval_timer(_loadCheckerTimer);
-        _loadCheckerTimer=0;
+        getRoot(owner()).removeAdvanceCallback(this);
     }
+
 }
 
 void
 LoadableObject::registerNative(as_object& o)
 {
-    VM& vm = o.getVM();
+    VM& vm = getVM(o);
 
     vm.registerNative(loadableobject_load, 301, 0);
     vm.registerNative(loadableobject_send, 301, 1);
@@ -329,6 +308,27 @@ LoadableObject::registerNative(as_object& o)
     vm.registerNative(loadableobject_decode, 301, 3);
 }
 
+as_value
+LoadableObject::loadableobject_getBytesLoaded(const fn_call& fn)
+{
+	boost::intrusive_ptr<as_object> ptr = ensureType<as_object>(fn.this_ptr);
+
+    as_value bytesLoaded;
+    string_table& st = getStringTable(fn);
+    ptr->get_member(st.find("_bytesLoaded"), &bytesLoaded);
+    return bytesLoaded;
+}
+    
+as_value
+LoadableObject::loadableobject_getBytesTotal(const fn_call& fn)
+{
+	boost::intrusive_ptr<as_object> ptr = ensureType<as_object>(fn.this_ptr);
+
+    as_value bytesTotal;
+    string_table& st = getStringTable(fn);
+    ptr->get_member(st.find("_bytesTotal"), &bytesTotal);
+    return bytesTotal;
+}
 
 /// Can take either a two strings as arguments or an array of strings,
 /// alternately header and value.
@@ -336,15 +336,12 @@ as_value
 LoadableObject::loadableobject_addRequestHeader(const fn_call& fn)
 {
     
-    boost::intrusive_ptr<LoadableObject> ptr = 
-        ensureType<LoadableObject>(fn.this_ptr);   
-
     as_value customHeaders;
     as_object* array;
 
-    if (ptr->get_member(NSV::PROP_uCUSTOM_HEADERS, &customHeaders))
+    if (fn.this_ptr->get_member(NSV::PROP_uCUSTOM_HEADERS, &customHeaders))
     {
-        array = customHeaders.to_object().get();
+        array = customHeaders.to_object(*getGlobal(fn)).get();
         if (!array)
         {
             IF_VERBOSE_ASCODING_ERRORS(
@@ -359,10 +356,10 @@ LoadableObject::loadableobject_addRequestHeader(const fn_call& fn)
         array = new Array_as;
         // This property is always initialized on the first call to
         // addRequestHeaders.
-        const int flags = as_prop_flags::dontEnum |
-                          as_prop_flags::dontDelete;
+        const int flags = PropFlags::dontEnum |
+                          PropFlags::dontDelete;
 
-        ptr->init_member(NSV::PROP_uCUSTOM_HEADERS, array, flags);
+        fn.this_ptr->init_member(NSV::PROP_uCUSTOM_HEADERS, array, flags);
     }
 
     if (fn.nargs == 0)
@@ -379,7 +376,8 @@ LoadableObject::loadableobject_addRequestHeader(const fn_call& fn)
     {
         // This must be an array. Keys / values are pushed in valid
         // pairs to the _customHeaders array.    
-        boost::intrusive_ptr<as_object> obj = fn.arg(0).to_object();
+        boost::intrusive_ptr<as_object> obj =
+            fn.arg(0).to_object(*getGlobal(fn));
         Array_as* headerArray = dynamic_cast<Array_as*>(obj.get());
 
         if (!headerArray)
@@ -459,9 +457,11 @@ loadableobject_decode(const fn_call& fn)
 
 	ValuesMap vals;
 
-	URL::parse_querystring(fn.arg(0).to_string(), vals);
+    const int version = getSWFVersion(fn);
 
-	string_table& st = ptr->getVM().getStringTable();
+	URL::parse_querystring(fn.arg(0).to_string_versioned(version), vals);
+
+	string_table& st = getStringTable(fn);
 	for  (ValuesMap::const_iterator it=vals.begin(), itEnd=vals.end();
 			it != itEnd; ++it)
 	{
@@ -479,8 +479,7 @@ loadableobject_decode(const fn_call& fn)
 as_value
 loadableobject_sendAndLoad(const fn_call& fn)
 {
-	boost::intrusive_ptr<LoadableObject> ptr =
-	                ensureType<LoadableObject>(fn.this_ptr);
+	LoadableObject* ptr = ensureNativeType<LoadableObject>(fn.this_ptr);
 
 	if ( fn.nargs < 2 )
 	{
@@ -508,12 +507,21 @@ loadableobject_sendAndLoad(const fn_call& fn)
 		return as_value(false);
 	}
 
+    // TODO: if this isn't an XML or LoadVars, it won't work, but we should
+    // check how far things get before it fails.
+	boost::intrusive_ptr<as_object> target =
+        fn.arg(1).to_object(*getGlobal(fn));
 
-	boost::intrusive_ptr<as_object> target = fn.arg(1).to_object();
-
-	// Post by default, override by ActionScript third argument
-	bool post = true;
-	if ( fn.nargs > 2 && fn.arg(2).to_string() == "GET" ) post = false;
+    // According to the Flash 8 Cookbook (Joey Lott, Jeffrey Bardzell), p 427,
+    // this method sends by GET unless overridden, and always by GET in the
+    // standalone player. We have no tests for this, but a Twitter widget
+    // gets Bad Request from the server if we send via POST.
+	bool post = false;
+	if (fn.nargs > 2) {
+        const std::string& method = fn.arg(2).to_string();
+        StringNoCaseEqual nc;
+        post = nc(method, "post");
+    }      
 
 	ptr->sendAndLoad(urlstr, *target, post);
 	return as_value(true);
@@ -523,8 +531,7 @@ loadableobject_sendAndLoad(const fn_call& fn)
 as_value
 loadableobject_load(const fn_call& fn)
 {
-	boost::intrusive_ptr<LoadableObject> obj = 
-        ensureType<LoadableObject>(fn.this_ptr);
+	LoadableObject* obj = ensureNativeType<LoadableObject>(fn.this_ptr);
 
 	if ( fn.nargs < 1 )
 	{
@@ -544,7 +551,12 @@ loadableobject_load(const fn_call& fn)
 	}
 
 	obj->load(urlstr);
-	return as_value(true);
+    
+    string_table& st = getStringTable(fn);
+    fn.this_ptr->set_member(st.find("_bytesLoaded"), 0.0);
+    fn.this_ptr->set_member(st.find("_bytesTotal"), as_value());
+
+    return as_value(true);
 
 }
 
@@ -552,8 +564,7 @@ loadableobject_load(const fn_call& fn)
 as_value
 loadableobject_send(const fn_call& fn)
 {
-    boost::intrusive_ptr<LoadableObject> ptr =
-        ensureType<LoadableObject>(fn.this_ptr);
+    LoadableObject* ptr = ensureNativeType<LoadableObject>(fn.this_ptr);
  
     std::ostringstream os;
     fn.dump_args(os);
