@@ -36,11 +36,11 @@ namespace gnash {
 /// It is a virtual base class because XML_as also inherits from XMLNode.
 //
 /// It may not be copied.
-class LoadableObject : public virtual as_object, private boost::noncopyable
+class LoadableObject : public ActiveRelay
 {
 public:
 
-    LoadableObject();
+    LoadableObject(as_object* owner);
 
     virtual ~LoadableObject();
 
@@ -81,7 +81,7 @@ public:
 		return _bytesTotal;
 	}  
 
-    /// Overrides as_object::queueLoad to begin loading from a stream
+    /// Begin loading from a stream
     //
     /// @param str      The stream to load from. It is destroyed when
     ///                 we're finished with it.
@@ -90,16 +90,17 @@ public:
     /// Shared AS methods for XML and LoadVars, which can be used
     /// interchangeably with each object in ActionScript.
     static as_value loadableobject_addRequestHeader(const fn_call& fn);
+    
+    /// These functions return the value of _bytesTotal and _bytesLoaded
+    static as_value loadableobject_getBytesLoaded(const fn_call& fn);
+    static as_value loadableobject_getBytesTotal(const fn_call& fn);
+
+    /// Scan the LoadThread queue (_loadThreads) to see if any of
+    /// them completed. If any did, invoke the onData event
+    virtual void update();
 
 protected:
 
-    /// Convert the Loadable Object to a string.
-    //
-    /// @param o        The ostream to write the string to.
-    /// @param encode   Whether URL encoding is necessary. How this
-    ///                 is done depends on the type of object.
-    virtual void toString(std::ostream& o, bool encode) const = 0;
-
     typedef std::list<LoadThread*> LoadThreadList;
 
     /// Queue of load requests
@@ -109,17 +110,6 @@ protected:
     
     long _bytesTotal;
 
-    /// The load checker interval timer used to make loads async
-    unsigned int _loadCheckerTimer;
-
-    /// Scan the LoadThread queue (_loadThreads) to see if any of
-    /// them completed. If any did, invoke the onData event
-    void checkLoads();
-
-private:
-
-    static as_value checkLoads_wrapper(const fn_call& fn);
-
 };
 
 
