@@ -22,20 +22,20 @@
 #endif
 
 #include "BitmapData_as.h"
+#include "MovieClip.h"
 #include "GnashImage.h"
 #include "Bitmap.h"
 #include "flash/geom/Rectangle_as.h" // for BitmapData.rectangle
 #include "as_object.h" // for inheritance
 #include "log.h"
 #include "fn_call.h"
+#include "Global_as.h"
 #include "smart_ptr.h" // for boost intrusive_ptr
 #include "builtin_function.h" // need builtin_function
 #include "GnashException.h" // for ActionException
 #include "Object.h" // for AS inheritance
 #include "VM.h" // for addStatics
 
-#include "BitmapMovieInstance.h"
-
 #include <vector>
 #include <sstream>
 #include <algorithm>
@@ -97,7 +97,8 @@ void
 BitmapData_as::markReachableResources() const
 {
     std::for_each(_attachedBitmaps.begin(), _attachedBitmaps.end(),
-            std::mem_fun(&character::setReachable));
+            std::mem_fun(&DisplayObject::setReachable));
+    markAsObjectReachable();
 }
 
 
@@ -223,28 +224,15 @@ BitmapData_as::dispose()
     updateAttachedBitmaps();
 }
 
-as_function*
-getFlashDisplayBitmapDataConstructor()
-{
-    static builtin_function* cl = NULL;
-    if ( ! cl )
-    {
-        cl=new builtin_function(&BitmapData_ctor, getBitmapDataInterface());
-        VM::get().addStatic(cl);
-        attachBitmapDataStaticProperties(*cl);
-    }
-    return cl;
-}
-
-
 // extern 
-void BitmapData_class_init(as_object& where)
+void
+bitmapdata_class_init(as_object& where, const ObjectURI& uri)
 {
-
-    string_table& st = where.getVM().getStringTable();
-	// Register _global.BitmapData
-	where.init_destructive_property(st.find("BitmapData"),
-	                get_flash_display_bitmap_data_constructor);
+    // TODO: this may not be correct, but it should be enumerable.
+    const int flags = 0;
+	where.init_destructive_property(getName(uri),
+                get_flash_display_bitmap_data_constructor, flags,
+                getNamespace(uri));
 }
 
 
@@ -373,7 +361,7 @@ BitmapData_fillRect(const fn_call& fn)
     }
 
     // This can be any object with the right properties.   
-    as_object* obj = arg.to_object().get();
+    as_object* obj = arg.to_object(*getGlobal(fn)).get();
     assert(obj);
     
     as_value x, y, w, h;
@@ -524,25 +512,58 @@ BitmapData_scroll(const fn_call& fn)
 as_value
 BitmapData_setPixel(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
-	UNUSED(ptr);
-	LOG_ONCE( log_unimpl (__FUNCTION__) );
+	boost::intrusive_ptr<BitmapData_as> ptr =
+        ensureType<BitmapData_as>(fn.this_ptr);
+
+    if (fn.nargs < 3) {
+        return as_value();
+    }
+
+    double x = fn.arg(0).to_number();
+    double y = fn.arg(1).to_number();
+    if (x < 0 || y < 0) return as_value();
+    if (x >= ptr->getWidth() || y >= ptr->getHeight()) {
+        return as_value();
+    }
+
+    // Ignore any transparency here.
+    boost::uint32_t color = fn.arg(2).to_int() & 0xffffff;
+
+    ptr->setPixel(x, y, color);
+
 	return as_value();
 }
 
 as_value
 BitmapData_setPixel32(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
-	UNUSED(ptr);
-	LOG_ONCE( log_unimpl (__FUNCTION__) );
+	boost::intrusive_ptr<BitmapData_as> ptr =
+        ensureType<BitmapData_as>(fn.this_ptr);
+
+    if (fn.nargs < 3) {
+        return as_value();
+    }
+
+    double x = fn.arg(0).to_number();
+    double y = fn.arg(1).to_number();
+    if (x < 0 || y < 0) return as_value();
+    if (x >= ptr->getWidth() || y >= ptr->getHeight()) {
+        return as_value();
+    }
+
+    // TODO: multiply.
+    boost::uint32_t color = fn.arg(2).to_int();
+
+    ptr->setPixel32(x, y, color);
+
 	return as_value();
 }
 
 as_value
 BitmapData_threshold(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	boost::intrusive_ptr<BitmapData_as> ptr =
+        ensureType<BitmapData_as>(fn.this_ptr);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
@@ -551,12 +572,14 @@ BitmapData_threshold(const fn_call& fn)
 as_value
 BitmapData_height(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	boost::intrusive_ptr<BitmapData_as> ptr =
+        ensureType<BitmapData_as>(fn.this_ptr);
 
     // Read-only
     if (fn.nargs) return as_value();
     
-    // Returns the immutable height of the bitmap or -1 if dispose() has been called.
+    // Returns the immutable height of the bitmap or -1 if dispose() has
+    // been called.
     if (ptr->getBitmapData().empty()) return -1;
 	return as_value(ptr->getHeight());
 }
@@ -564,26 +587,36 @@ BitmapData_height(const fn_call& fn)
 as_value
 BitmapData_rectangle(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	boost::intrusive_ptr<BitmapData_as> ptr = 
+        ensureType<BitmapData_as>(fn.this_ptr);
 
     // Returns the immutable rectangle of the bitmap or -1 if dispose()
     // has been called.
     if (ptr->getBitmapData().empty()) return -1;
 
-	boost::intrusive_ptr<as_object> obj = init_Rectangle_instance();
+    // If it's not found construction will fail.
+    as_value rectangle(fn.env().find_object("flash.geom.Rectangle"));
+    boost::intrusive_ptr<as_function> rectCtor = rectangle.to_as_function();
 
-	obj->set_member(NSV::PROP_X, 0.0);
-	obj->set_member(NSV::PROP_Y, 0.0);
-	obj->set_member(NSV::PROP_WIDTH, ptr->getWidth());
-	obj->set_member(NSV::PROP_HEIGHT, ptr->getHeight());
+    if (!rectCtor) {
+        log_error("Failed to construct flash.geom.Rectangle!");
+        return -1;
+    }
 
-	return as_value(obj.get()); // will keep alive
+    fn_call::Args args;
+    args += 0.0, 0.0, ptr->getWidth(), ptr->getHeight();
+
+    boost::intrusive_ptr<as_object> newRect =
+            rectCtor->constructInstance(fn.env(), args);
+
+    return as_value(newRect.get());
 }
 
 as_value
 BitmapData_transparent(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	boost::intrusive_ptr<BitmapData_as> ptr =
+        ensureType<BitmapData_as>(fn.this_ptr);
 
     // Read-only
     if (fn.nargs) return as_value();
@@ -596,7 +629,8 @@ BitmapData_transparent(const fn_call& fn)
 as_value
 BitmapData_width(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	boost::intrusive_ptr<BitmapData_as> ptr =
+        ensureType<BitmapData_as>(fn.this_ptr);
 
     // Read-only
     if (fn.nargs) return as_value();
@@ -611,7 +645,8 @@ BitmapData_width(const fn_call& fn)
 as_value
 BitmapData_loadBitmap(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	boost::intrusive_ptr<BitmapData_as> ptr =
+        ensureType<BitmapData_as>(fn.this_ptr);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
@@ -619,20 +654,26 @@ BitmapData_loadBitmap(const fn_call& fn)
 
 
 as_value
-get_flash_display_bitmap_data_constructor(const fn_call& /*fn*/)
+get_flash_display_bitmap_data_constructor(const fn_call& fn)
 {
     log_debug("Loading flash.display.BitmapData class");
-    return getFlashDisplayBitmapDataConstructor();
+    Global_as* gl = getGlobal(fn);
+    as_object* proto = getBitmapDataInterface();
+    as_object* cl = gl->createClass(&BitmapData_ctor, proto);
+    attachBitmapDataStaticProperties(*cl);
+    return cl;
 }
 
 as_value
 BitmapData_ctor(const fn_call& fn)
 {
 
-	if ( fn.nargs < 2)
-	{
-	    // TODO: should fail if not enough arguments are passed.
-        return as_value();
+	if (fn.nargs < 2) {
+        IF_VERBOSE_ASCODING_ERRORS(
+             log_aserror("BitmapData constructor requires at least two "
+                 "arguments. Will not construct a BitmapData");
+        );
+        throw ActionTypeError();
 	}
 
     size_t width, height;
@@ -655,8 +696,13 @@ BitmapData_ctor(const fn_call& fn)
     }
     
     // FIXME: Should fail to construct the object.
-    if (width > 2880 || height > 2880) return as_value();
-    if (width < 1 || height < 1) return as_value();
+    if (width > 2880 || height > 2880 || width < 1 || height < 1) {
+        IF_VERBOSE_ASCODING_ERRORS(
+             log_aserror("BitmapData width and height must be between "
+                 "1 and 2880. Will not construct a BitmapData");
+        );
+        throw ActionTypeError();
+    }
 
     boost::intrusive_ptr<BitmapData_as> obj =
                 new BitmapData_as(width, height, transparent, fillColor);
@@ -667,32 +713,38 @@ BitmapData_ctor(const fn_call& fn)
 void
 attachBitmapDataInterface(as_object& o)
 {
-    o.init_member("applyFilter", new builtin_function(BitmapData_applyFilter));
-    o.init_member("clone", new builtin_function(BitmapData_clone));
-    o.init_member("colorTransform", new builtin_function(BitmapData_colorTransform));
-    o.init_member("copyChannel", new builtin_function(BitmapData_copyChannel));
-    o.init_member("copyPixels", new builtin_function(BitmapData_copyPixels));
-    o.init_member("dispose", new builtin_function(BitmapData_dispose));
-    o.init_member("draw", new builtin_function(BitmapData_draw));
-    o.init_member("fillRect", new builtin_function(BitmapData_fillRect));
-    o.init_member("floodFill", new builtin_function(BitmapData_floodFill));
-    o.init_member("generateFilterRect", new builtin_function(BitmapData_generateFilterRect));
-    o.init_member("getColorBoundsRect", new builtin_function(BitmapData_getColorBoundsRect));
-    o.init_member("getPixel", new builtin_function(BitmapData_getPixel));
-    o.init_member("getPixel32", new builtin_function(BitmapData_getPixel32));
-    o.init_member("hitTest", new builtin_function(BitmapData_hitTest));
-    o.init_member("merge", new builtin_function(BitmapData_merge));
-    o.init_member("noise", new builtin_function(BitmapData_noise));
-    o.init_member("paletteMap", new builtin_function(BitmapData_paletteMap));
-    o.init_member("perlinNoise", new builtin_function(BitmapData_perlinNoise));
-    o.init_member("pixelDissolve", new builtin_function(BitmapData_pixelDissolve));
-    o.init_member("scroll", new builtin_function(BitmapData_scroll));
-    o.init_member("setPixel", new builtin_function(BitmapData_setPixel));
-    o.init_member("setPixel32", new builtin_function(BitmapData_setPixel32));
-    o.init_member("threshold", new builtin_function(BitmapData_threshold));
+    Global_as* gl = getGlobal(o);
+    o.init_member("applyFilter", gl->createFunction(BitmapData_applyFilter));
+    o.init_member("clone", gl->createFunction(BitmapData_clone));
+    o.init_member("colorTransform", gl->createFunction(
+                BitmapData_colorTransform));
+    o.init_member("copyChannel", gl->createFunction(BitmapData_copyChannel));
+    o.init_member("copyPixels", gl->createFunction(BitmapData_copyPixels));
+    o.init_member("dispose", gl->createFunction(BitmapData_dispose));
+    o.init_member("draw", gl->createFunction(BitmapData_draw));
+    o.init_member("fillRect", gl->createFunction(BitmapData_fillRect));
+    o.init_member("floodFill", gl->createFunction(BitmapData_floodFill));
+    o.init_member("generateFilterRect", gl->createFunction(
+                BitmapData_generateFilterRect));
+    o.init_member("getColorBoundsRect", gl->createFunction(
+                BitmapData_getColorBoundsRect));
+    o.init_member("getPixel", gl->createFunction(BitmapData_getPixel));
+    o.init_member("getPixel32", gl->createFunction(BitmapData_getPixel32));
+    o.init_member("hitTest", gl->createFunction(BitmapData_hitTest));
+    o.init_member("merge", gl->createFunction(BitmapData_merge));
+    o.init_member("noise", gl->createFunction(BitmapData_noise));
+    o.init_member("paletteMap", gl->createFunction(BitmapData_paletteMap));
+    o.init_member("perlinNoise", gl->createFunction(BitmapData_perlinNoise));
+    o.init_member("pixelDissolve", gl->createFunction(
+                BitmapData_pixelDissolve));
+    o.init_member("scroll", gl->createFunction(BitmapData_scroll));
+    o.init_member("setPixel", gl->createFunction(BitmapData_setPixel));
+    o.init_member("setPixel32", gl->createFunction(BitmapData_setPixel32));
+    o.init_member("threshold", gl->createFunction(BitmapData_threshold));
     o.init_property("height", BitmapData_height, BitmapData_height);
     o.init_property("rectangle", BitmapData_rectangle, BitmapData_rectangle);
-    o.init_property("transparent", BitmapData_transparent, BitmapData_transparent);
+    o.init_property("transparent", BitmapData_transparent,
+            BitmapData_transparent);
     o.init_property("width", BitmapData_width, BitmapData_width);
 
 }
@@ -700,8 +752,8 @@ attachBitmapDataInterface(as_object& o)
 void
 attachBitmapDataStaticProperties(as_object& o)
 {
-   
-    o.init_member("loadBitmap", new builtin_function(BitmapData_loadBitmap));
+    Global_as* gl = getGlobal(o);
+    o.init_member("loadBitmap", gl->createFunction(BitmapData_loadBitmap));
 }
 
 as_object*
@@ -709,15 +761,11 @@ getBitmapDataInterface()
 {
 	static boost::intrusive_ptr<as_object> o;
 
-	if ( ! o )
-	{
-		// TODO: check if this class should inherit from Object
-		//       or from a different class
+	if (!o) {
 		o = new as_object(getObjectInterface());
 		VM::get().addStatic(o.get());
 
 		attachBitmapDataInterface(*o);
-
 	}
 
 	return o.get();
