@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,9 +22,10 @@
 namespace gnash {
 
 class as_object;
+class ObjectURI;
 
 /// Initialize the global GradientGlowFilter class
-void GradientGlowFilter_class_init(as_object& global);
+void gradientglowfilter_class_init(as_object& where, const ObjectURI& uri);
 
 } // end of gnash namespace
 
