@@ -29,9 +29,10 @@
 namespace gnash {
 
 class as_object;
+class ObjectURI;
 
 /// Initialize the global Rectangle class
-void Rectangle_class_init(as_object& global);
+void rectangle_class_init(as_object& where, const ObjectURI& uri);
 
 /// Return a Rectangle instance (in case the core lib needs it)
 boost::intrusive_ptr<as_object> init_Rectangle_instance();
