@@ -29,9 +29,10 @@
 namespace gnash {
 
 class as_object;
+class ObjectURI;
 
 /// Initialize the global FileReferenceList class
-void FileReferenceList_class_init(as_object& global);
+void filereferencelist_class_init(as_object& where, const ObjectURI& uri);
 
 /// Return a FileReferenceList instance (in case the core lib needs it)
 //std::auto_ptr<as_object> init_FileReferenceList_instance();
