@@ -25,7 +25,7 @@
 #endif
 
 #include "DisplayListTag.h" // for inheritance
-#include "swf.h" // for TagType definition
+#include "SWF.h" // for TagType definition
 
 // Forward declarations
 namespace gnash {
@@ -34,7 +34,7 @@ namespace gnash {
 	class swf_event;
 	class movie_definition;
 	class DisplayList;
-    class RunInfo;
+    class RunResources;
 }
 
 namespace gnash {
@@ -42,7 +42,7 @@ namespace SWF {
 
 /// SWF Tag RemoveObject (5) or RemoveObject2 (28)
 //
-/// The RemoveObject tag removes the character instance at the specified depth.
+/// The RemoveObject tag removes the DisplayObject instance at the specified depth.
 ///
 /// TODO: make this and PlaceObject2Tag subclasses of DisplayListTag (subclass of ControlTag)
 ///
@@ -63,7 +63,7 @@ public:
 	void execute(MovieClip* m, DisplayList& dlist) const;
 
 	static void loader(SWFStream& in, TagType tag, movie_definition& m,
-            const RunInfo& r);
+            const RunResources& r);
 
 private:
 
