@@ -25,7 +25,7 @@
 #include "ControlTag.h" // for inheritance
 #include "sound_handler.h" // for sound_envelope (composition)
 #include "VM.h" // We only need this to get movie_root
-#include "swf.h" // for TagType definition
+#include "SWF.h" // for TagType definition
 #include "SoundInfoRecord.h"
 
 #include <vector> // for composition
@@ -37,7 +37,7 @@ namespace gnash {
 	class SWFStream;
 	class movie_definition;
 	class sound_sample;
-    class RunInfo;
+    class RunResources;
 }
 
 namespace gnash {
@@ -83,21 +83,21 @@ public:
 
 	/// Load a SWF::STARTSOUND tag.
 	static void loader(SWFStream& in, TagType tag, movie_definition& m,
-            const RunInfo& r);
+            const RunResources& r);
 
 };
 
 /// StartSound2Tag (SWF9)
 //
 /// Very similar to StartSoundTag, but uses a SoundClassName instead of
-/// character ID. This is not implemented.
+/// DisplayObject ID. This is not implemented.
 class StartSound2Tag
 {
 public:
 
 	/// Load a SWF::STARTSOUND2 tag.
 	static void loader(SWFStream& in, TagType tag, movie_definition& m,
-            const RunInfo& r);
+            const RunResources& r);
 };
 
 } // namespace gnash::SWF
