@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,13 +17,14 @@
 //
 #include "CallStack.h"
 #include "as_object.h"
-#include "as_function.h" // for as_environment::CallFrame::markReachableResources
+#include "as_function.h" 
+#include "Global_as.h" 
 
 namespace gnash {
 
 CallFrame::CallFrame(as_function* funcPtr)
 	:
-	locals(new as_object()),
+	locals(new as_object),
 	func(funcPtr)
 {
 }
