@@ -1,6 +1,6 @@
 // AudioDecoderNellymoser.cpp: Nellymoser decoding
 // 
-//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-// $Id: AudioDecoderNellymoser.cpp,v 1.8 2008/02/08 15:27:30 bjacques Exp $
 
 // This file incorporates work covered by the following copyright and permission
 // notice:
@@ -48,10 +47,12 @@
  */
 
 #include "AudioDecoderNellymoser.h"
-#include "utility.h"
-#include <stdlib.h>
-#include <time.h>
-#include <math.h>
+#include "SoundInfo.h"
+#include "log.h"
+
+#include <ctime>
+#include <cmath>
+#include <boost/random.hpp>
 
 namespace gnash {
 namespace media {
@@ -480,20 +481,19 @@ static int headroom(int *la, short *sa)
 
 static void get_sample_bits(float *buf, int *bits)
 {
-	int i, j;
+	int i = 0, j;
 	short sbuf[128];
 	int bitsum = 0, last_bitsum, small_bitsum, big_bitsum;
-	short shift, shift_saved;
-	int tmp;
+	short shift = -16, shift_saved;
+	int tmp = 0;
 	int big_off;
 	int off, diff;
 
-	tmp = 0;
-	for (i = 0; i < NELLY_FILL_LEN; i++) {
+	for (; i < NELLY_FILL_LEN; i++) {
 		if (buf[i] > tmp)
-			tmp = buf[i];
+			tmp = static_cast<int>(buf[i]);
 	}
-	shift = -16;
+
 	headroom(&tmp, &shift);
 
 	if (shift < 0)
@@ -625,7 +625,8 @@ static void get_sample_bits(float *buf, int *bits)
 	}
 }
 
-static unsigned char get_bits(unsigned char block[NELLY_BLOCK_LEN], int *off, int n)
+static unsigned
+char get_bits(const unsigned char block[NELLY_BLOCK_LEN], int *off, int n)
 {
 	char ret;
 	int boff = *off/8, bitpos = *off%8, mask = (1<<n)-1;
@@ -642,7 +643,23 @@ static unsigned char get_bits(unsigned char block[NELLY_BLOCK_LEN], int *off, in
 	return ret;
 }
 
-static void nelly_decode_block(nelly_handle* nh, unsigned char block[NELLY_BLOCK_LEN], float audio[256])
+static int
+gimme_random()
+{
+	using namespace boost;
+	
+    typedef boost::mt11213b RNG;
+
+    static RNG rnd;
+
+	uniform_int<> dist(0, std::numeric_limits<int>::max());
+	variate_generator<RNG, uniform_int<> > uni(rnd, dist);
+
+	return uni();
+}
+
+static void
+nelly_decode_block(nelly_handle* nh, const unsigned char block[NELLY_BLOCK_LEN], float audio[256])
 {
 	int i,j;
 	float buf[NELLY_BUF_LEN], pows[NELLY_BUF_LEN];
@@ -679,7 +696,9 @@ static void nelly_decode_block(nelly_handle* nh, unsigned char block[NELLY_BLOCK
 		for (j = 0; j < NELLY_FILL_LEN; j++) {
 			if (bits[j] <= 0) {
 				buf[j] = M_SQRT1_2*pows[j];
-				if (random() % 2)
+
+        
+				if (gimme_random() % 2)
 					buf[j] *= -1.0;
 			} else {
 				v = get_bits(block, &bit_offset, bits[j]);
@@ -711,15 +730,9 @@ static void nelly_util_floats2shorts(float audio[256], short shorts[256])
 
 static nelly_handle *nelly_get_handle()
 {
-	static int first = 1;
 	int i;
 	nelly_handle *nh;
 
-	if (first) {
-		srandom(time(NULL));
-		first = 0;
-	}
-
 	nh = new nelly_handle;
 
 	if (nh != NULL)
@@ -733,8 +746,9 @@ static void nelly_free_handle(nelly_handle *nh)
 {
 	delete nh;
 }
-	
-AudioDecoderNellymoser::AudioDecoderNellymoser ()
+
+
+AudioDecoderNellymoser::AudioDecoderNellymoser()
 	:
 	_sampleRate(0),
 	_stereo(false)
@@ -742,39 +756,95 @@ AudioDecoderNellymoser::AudioDecoderNellymoser ()
 	_nh = nelly_get_handle();
 }
 
+	
+AudioDecoderNellymoser::AudioDecoderNellymoser(const AudioInfo& info)
+	:
+	_sampleRate(0),
+	_stereo(false)
+{
+    setup(info);
+	_nh = nelly_get_handle();
+
+    assert(info.type == FLASH); // or we'd have thrown an exception
+    audioCodecType codec = (audioCodecType)info.codec;
+  	log_debug(_("AudioDecoderNellymoser: initialized FLASH codec %s (%d)"),
+		(int)codec, codec);
+}
+
+
+AudioDecoderNellymoser::AudioDecoderNellymoser(const SoundInfo& info)
+	:
+	_sampleRate(0),
+	_stereo(false)
+{
+    setup(info);
+	_nh = nelly_get_handle();
+
+    audioCodecType codec = info.getFormat();
+  	log_debug(_("AudioDecoderNellymoser: initialized FLASH codec %s (%d)"),
+		(int)codec, codec);
+}
+
+
 AudioDecoderNellymoser::~AudioDecoderNellymoser()
 {
 	nelly_free_handle(_nh);
 }
 
-bool AudioDecoderNellymoser::setup(SoundInfo* info)
+void
+AudioDecoderNellymoser::setup(const SoundInfo& info)
 {
-	if (info->getFormat() == AUDIO_CODEC_NELLYMOSER || info->getFormat() == AUDIO_CODEC_NELLYMOSER_8HZ_MONO) {
-		_sampleRate = info->getSampleRate();
-		_stereo = info->isStereo();
-		return true;
-	} else {
-		return false;
+	audioCodecType codec = info.getFormat();
+    switch (codec)
+    {
+        case AUDIO_CODEC_NELLYMOSER:
+        case AUDIO_CODEC_NELLYMOSER_8HZ_MONO:
+            _sampleRate = info.getSampleRate();
+            _stereo = info.isStereo();
+            break;
+
+        default:
+            boost::format err = boost::format(
+                _("AudioDecoderNellymoser: attempt to use with flash codec %d (%s)"))
+                % (int)codec % codec;
+            throw MediaException(err.str());
 	}
 }
 
-bool AudioDecoderNellymoser::setup(AudioInfo* info)
+void AudioDecoderNellymoser::setup(const AudioInfo& info)
 {
-	if (info->type == FLASH && (info->codec == AUDIO_CODEC_NELLYMOSER || info->codec == AUDIO_CODEC_NELLYMOSER_8HZ_MONO)) {
-		_sampleRate = info->sampleRate;
-		_stereo = info->stereo;
-		return true;
-	} else {
-		return false;
+	if (info.type != FLASH)
+    {
+        boost::format err = boost::format(
+            _("AudioDecoderNellymoser: unable to intepret custom audio codec id %s"))
+            % info.codec;
+        throw MediaException(err.str());
+    }
+
+	audioCodecType codec = static_cast<audioCodecType>(info.codec);
+    switch (codec)
+    {
+        case AUDIO_CODEC_NELLYMOSER:
+        case AUDIO_CODEC_NELLYMOSER_8HZ_MONO:
+            _sampleRate = info.sampleRate;
+            _stereo = info.stereo;
+            break;
+
+        default:
+            boost::format err = boost::format(
+                _("AudioDecoderNellymoser: attempt to use with flash codec %d (%s)"))
+                % (int)codec % codec;
+            throw MediaException(err.str());
 	}
 }
 
-float* AudioDecoderNellymoser::decode(boost::uint8_t* in_buf, boost::uint32_t inputSize, boost::uint32_t* outputSize)
+float*
+AudioDecoderNellymoser::decode(const boost::uint8_t* in_buf, boost::uint32_t inputSize, boost::uint32_t* outputSize)
 {
-        size_t out_buf_size = (inputSize / NELLY_BLOCK_LEN) * 256;
+    size_t out_buf_size = (inputSize / NELLY_BLOCK_LEN) * 256;
 	float* out_buf = new float[out_buf_size];
 	
-	boost::uint8_t* input_ptr = in_buf;
+	const boost::uint8_t* input_ptr = in_buf;
 	float* output_ptr = out_buf;
 
 	while (inputSize > 0) {
@@ -789,7 +859,10 @@ float* AudioDecoderNellymoser::decode(boost::uint8_t* in_buf, boost::uint32_t in
 	return out_buf;	
 }
 
-boost::uint8_t* AudioDecoderNellymoser::decode(boost::uint8_t* input, boost::uint32_t inputSize, boost::uint32_t& outputSize, boost::uint32_t& decodedBytes, bool /*parse*/)
+boost::uint8_t*
+AudioDecoderNellymoser::decode(const boost::uint8_t* input,
+        boost::uint32_t inputSize, boost::uint32_t& outputSize,
+        boost::uint32_t& decodedBytes, bool /*parse*/)
 {
 
 	float float_buf[256];
