@@ -23,22 +23,27 @@
 #include "URL.h"
 #include "noseek_fd_adapter.h"
 #include "movie_definition.h"
-#include "movie_instance.h"
+#include "Movie.h"
 #include "movie_root.h"
 #include "MovieClip.h"
-#include "as_environment.h"
-#include "gnash.h" // for create_movie and create_library_movie and for gnash::key namespace
-#include "VM.h" // for initialization
+#include "MovieFactory.h"
 #include "sound_handler.h" // for creating the "test" sound handlers
 #include "NullSoundHandler.h"
-#include "RGBA.h" // for rgba class
-#include "FuzzyPixel.h"
-#include "render.h"
-#include "render_handler.h"
-#include "render_handler_agg.h"
-#include "SystemClock.h"
+#include "RGBA.h" // for rgba class (pixel checking)
+#include "FuzzyPixel.h" // for pixel checking
+#include "Renderer.h"
+#include "ManualClock.h" // for use by advance
+#include "StreamProvider.h" // for passing to RunResources
+#include "swf/TagLoadersTable.h"
+#include "swf/DefaultTagLoaders.h"
 #ifdef RENDERER_CAIRO
-#include "render_handler_cairo.h"
+# include "Renderer_cairo.h"
+#endif
+#ifdef RENDERER_OPENGL
+# include "Renderer_ogl.h"
+#endif
+#ifdef RENDERER_AGG
+# include "Renderer_agg.h"
 #endif
 
 #include "MediaHandler.h"
@@ -53,6 +58,7 @@
 #include <memory> // for auto_ptr
 #include <cmath> // for ceil and (possibly) exp2
 #include <iostream>
+#include <boost/shared_ptr.hpp>
 
 #define SHOW_INVALIDATED_BOUNDS_ON_ADVANCE 1
 
@@ -80,10 +86,15 @@ MovieTester::MovieTester(const std::string& url)
 	// Initialize the sound handler(s)
 	initTestingSoundHandlers();
 
-    _runInfo.reset(new RunInfo(url));
-    _runInfo->setSoundHandler(_sound_handler);
+    _runResources.reset(new RunResources(url));
+    _runResources->setSoundHandler(_sound_handler);
+    
+    boost::shared_ptr<SWF::TagLoadersTable> loaders(new SWF::TagLoadersTable());
+    addDefaultLoaders(*loaders);
 
-    _runInfo->setStreamProvider(boost::shared_ptr<StreamProvider>(
+    _runResources->setTagLoaders(loaders);
+
+    _runResources->setStreamProvider(boost::shared_ptr<StreamProvider>(
                 new StreamProvider));
 
 	if ( url == "-" )
@@ -91,7 +102,7 @@ MovieTester::MovieTester(const std::string& url)
 		std::auto_ptr<IOChannel> in (
 				noseek_fd_adapter::make_stream(fileno(stdin))
 				);
-		_movie_def = gnash::create_movie(in, url, *_runInfo, false);
+		_movie_def = MovieFactory::makeMovie(in, url, *_runResources, false);
 	}
 	else
 	{
@@ -111,7 +122,7 @@ MovieTester::MovieTester(const std::string& url)
 #endif
 		}
 		// _url should be always set at this point...
-		_movie_def = gnash::create_library_movie(urlObj, *_runInfo,
+		_movie_def = MovieFactory::makeMovie(urlObj, *_runResources,
                 NULL, false);
 	}
 
@@ -120,7 +131,7 @@ MovieTester::MovieTester(const std::string& url)
 		throw GnashException("Could not load movie from "+url);
 	}
 
-	_movie_root = new movie_root(*_movie_def, _clock, *_runInfo);
+	_movie_root = new movie_root(*_movie_def, _clock, *_runResources);
 
 	// Initialize viewport size with the one advertised in the header
 	_width = unsigned(_movie_def->get_width_pixels());
@@ -139,7 +150,7 @@ MovieTester::MovieTester(const std::string& url)
 	dbglogfile.setVerbosity(1);
 
 	
-	std::auto_ptr<movie_instance> mi ( _movie_def->create_movie_instance() );
+	std::auto_ptr<Movie> mi ( _movie_def->createMovie() );
 
 	// Set _movie before calling ::render
 	_movie = mi.get();
@@ -152,20 +163,24 @@ MovieTester::MovieTester(const std::string& url)
 }
 
 void
-MovieTester::render(render_handler& h, InvalidatedRanges& invalidated_regions) 
+MovieTester::render(boost::shared_ptr<Renderer> h,
+        InvalidatedRanges& invalidated_regions) 
 {
 	assert(_movie);
 
-	set_render_handler(&h);
+    // This is a bit dangerous, as there isn't really support for swapping
+    // renderers during runtime; though the only problem is likely to be
+    // that BitmapInfos are missing.
+	_runResources->setRenderer(h);
 
-	h.set_invalidated_regions(invalidated_regions);
+	h->set_invalidated_regions(invalidated_regions);
 
 	// We call display here to simulate effect of a real run.
 	//
 	// What we're particularly interested about is 
 	// proper computation of invalidated bounds, which
 	// needs clear_invalidated() to be called.
-	// display() will call clear_invalidated() on characters
+	// display() will call clear_invalidated() on DisplayObjects
 	// actually modified so we're fine with that.
 	//
 	// Directly calling _movie->clear_invalidated() here
@@ -205,10 +220,10 @@ MovieTester::render()
 		_forceRedraw = false; // reset to no forced redraw
 	}
 
-	for (TRenderers::const_iterator it=_testingRenderers.begin(), itE=_testingRenderers.end();
-				it != itE; ++it)
+	for (TestingRenderers::const_iterator it=_testingRenderers.begin(),
+            itE=_testingRenderers.end(); it != itE; ++it)
 	{
-		TestingRenderer& rend = *(*it);
+		const TestingRenderer& rend = *it;
 		render(rend.getRenderer(), ranges);
 	}
 	
@@ -219,7 +234,7 @@ MovieTester::render()
 		// What we're particularly interested about is 
 		// proper computation of invalidated bounds, which
 		// needs clear_invalidated() to be called.
-		// display() will call clear_invalidated() on characters
+		// display() will call clear_invalidated() on DisplayObjects
 		// actually modified so we're fine with that.
 		//
 		// Directly calling _movie->clear_invalidated() here
@@ -238,28 +253,33 @@ MovieTester::advanceClock(unsigned long ms)
 {
 	_clock.advance(ms);
 
-    // We need to fetch as many samples
-    // as needed for a theoretical 44100hz loop.
-    // That is 44100 samples each second.
-    // 44100/1000 = x/ms
-    //  x = (44100*ms) / 1000
-    unsigned int nSamples = (441*ms) / 10;
-    if ( ms%10 )
+    if ( _sound_handler )
     {
-        log_error("MovieTester::advanceClock: %d ms lost in sound advancement",
-            ms%10);
-    }
+        // We need to fetch as many samples
+        // as needed for a theoretical 44100hz loop.
+        // That is 44100 samples each second.
+        // 44100/1000 = x/ms
+        //  x = (44100*ms) / 1000
+        unsigned int nSamples = (441*ms) / 10;
+        if ( ms%10 )
+        {
+            log_error("MovieTester::advanceClock: %d ms lost in sound advancement",
+                ms%10);
+        }
 
-    unsigned int toFetch = nSamples*2;
+        // We double because sound_handler interface takes
+        // "mono" samples... (eh.. would be wise to change)
+        unsigned int toFetch = nSamples*2;
 
-    log_debug("advanceClock(%d) needs to fetch %d samples", ms, toFetch);
+        log_debug("advanceClock(%d) needs to fetch %d samples", ms, toFetch);
 
-    boost::int16_t samples[1024];
-    while (toFetch)
-    {
-        unsigned int n = std::min(toFetch, 1024u);
-        _sound_handler->fetchSamples((boost::int16_t*)&samples, n);
-        toFetch -= n;
+        boost::int16_t samples[1024];
+        while (toFetch)
+        {
+            unsigned int n = std::min(toFetch, 1024u);
+            _sound_handler->fetchSamples((boost::int16_t*)&samples, n);
+            toFetch -= n;
+        }
     }
     
 }
@@ -299,31 +319,31 @@ MovieTester::resizeStage(int x, int y)
 		if (yscale < xscale) xscale = yscale;
 
         // Scale for all renderers.
-        for (TRenderers::const_iterator it=_testingRenderers.begin(), itE=_testingRenderers.end();
-			        it != itE; ++it)
+        for (TestingRenderers::iterator it=_testingRenderers.begin(),
+                itE=_testingRenderers.end(); it != itE; ++it)
         {
-            TestingRenderer& rend = *(*it);
-            render_handler& h = rend.getRenderer();
-            h.set_scale(xscale, yscale);
+            TestingRenderer& rend = *it;
+            Renderer* h = rend.getRenderer().get();
+            h->set_scale(xscale, yscale);
         }
 	}
 
 }
 
-const character*
+const DisplayObject*
 MovieTester::findDisplayItemByName(const MovieClip& mc,
 		const std::string& name) 
 {
 	const DisplayList& dlist = mc.getDisplayList();
-	return dlist.get_character_by_name(name);
+	return dlist.getDisplayObjectByName(name);
 }
 
-const character*
+const DisplayObject*
 MovieTester::findDisplayItemByDepth(const MovieClip& mc,
 		int depth)
 {
 	const DisplayList& dlist = mc.getDisplayList();
-	return dlist.get_character_at_depth(depth);
+	return dlist.getDisplayObjectAtDepth(depth);
 }
 
 void
@@ -351,10 +371,10 @@ MovieTester::checkPixel(int x, int y, unsigned radius, const rgba& color,
 
 	//std::cout <<"chekPixel(" << color << ") called" << std::endl;
 
-	for (TRenderers::const_iterator it=_testingRenderers.begin(), itE=_testingRenderers.end();
-				it != itE; ++it)
+	for (TestingRenderers::const_iterator it=_testingRenderers.begin(),
+            itE=_testingRenderers.end(); it != itE; ++it)
 	{
-		const TestingRenderer& rend = *(*it);
+		const TestingRenderer& rend = *it;
 
 		std::stringstream ss;
 		ss << rend.getName() <<" ";
@@ -362,7 +382,7 @@ MovieTester::checkPixel(int x, int y, unsigned radius, const rgba& color,
 
 		rgba obt_col;
 
-		render_handler& handler = rend.getRenderer();
+		const Renderer& handler = *rend.getRenderer();
 
 	        if ( ! handler.getAveragePixel(obt_col, x, y, radius) )
 		{
@@ -486,7 +506,7 @@ MovieTester::soundsStopped()
 void
 MovieTester::initTestingRenderers()
 {
-	std::auto_ptr<render_handler> handler;
+    boost::shared_ptr<Renderer> handler;
 
 	// TODO: add support for testing multiple renderers
 	// This is tricky as requires changes in the core lib
@@ -504,7 +524,7 @@ MovieTester::initTestingRenderers()
 		const char* pixelFormat = aggPixelFormats[i];
 		std::string name = "AGG_" + std::string(pixelFormat);
 
-		handler.reset( create_render_handler_agg(pixelFormat) );
+		handler.reset( create_Renderer_agg(pixelFormat) );
 		if ( handler.get() )
 		{
 			//log_debug("Renderer %s initialized", name.c_str());
@@ -527,13 +547,14 @@ MovieTester::initTestingRenderers()
 
 #ifdef RENDERER_OPENGL
 	// Initialize opengl renderer
-	handler.reset(create_render_handler_ogl(false));
+	handler.reset(create_Renderer_ogl(false));
 	addTestingRenderer(handler, "OpenGL");
 #endif
 }
 
 void
-MovieTester::addTestingRenderer(std::auto_ptr<render_handler> h, const std::string& name)
+MovieTester::addTestingRenderer(boost::shared_ptr<Renderer> h,
+        const std::string& name)
 {
 	if ( ! h->initTestBuffer(_width, _height) )
 	{
@@ -550,16 +571,16 @@ MovieTester::addTestingRenderer(std::auto_ptr<render_handler> h, const std::stri
 			<< " because gnash core lib is unable to support testing of "
 			<< "multiple renderers from a single process "
 			<< "and we're already testing render handler "
-			<< _testingRenderers.front()->getName()
+			<< _testingRenderers.front().getName()
 			<< std::endl;
 		return;
 	}
 
-	_testingRenderers.push_back(TestingRendererPtr(new TestingRenderer(h, name)));
+	_testingRenderers.push_back(TestingRenderer(h, name));
 
 	// this will be needed till we allow run-time swapping of renderers,
 	// see above UNTESTED message...
-	set_render_handler(&(_testingRenderers.back()->getRenderer()));
+	_runResources->setRenderer(_testingRenderers.back().getRenderer());
 }
 
 bool
@@ -573,8 +594,15 @@ MovieTester::canTestVideo() const
 void
 MovieTester::initTestingSoundHandlers()
 {
-
-    _sound_handler.reset( new sound::NullSoundHandler() );
+    // Currently, SoundHandler can't be constructed
+    // w/out a registered MediaHandler .
+    // Should be fixed though...
+    //
+    if ( gnash::media::MediaHandler::get() ) {
+        log_error("MediaHandler::get returns: %s",
+            gnash::media::MediaHandler::get());
+        _sound_handler.reset( new sound::NullSoundHandler() );
+    }
 }
 
 void
@@ -588,7 +616,7 @@ MovieTester::initTestingMediaHandlers()
 #elif defined(USE_GST)
         handler.reset( new gnash::media::gst::MediaHandlerGst() );
 #else
-	std::cerr << "Neigher SOUND_SDL nor SOUND_GST defined" << std::endl;
+	std::cerr << "Neigther SOUND_SDL nor SOUND_GST defined" << std::endl;
 	return;
 #endif
 
@@ -599,7 +627,7 @@ void
 MovieTester::restart() 
 {
 	_movie_root->clear(); // restart();
-	_movie = _movie_def->create_movie_instance();
+	_movie = _movie_def->createMovie();
 	_movie_root->setRootMovie(_movie);
 
 	// Set _movie before calling ::render
