@@ -28,10 +28,10 @@
 #include "Range2d.h"
 #include "GnashKey.h"
 #include "sound_handler.h" // for creating the "test" sound handlers
-#include "render_handler.h" // for dtor visibility by auto_ptr
-#include "movie_instance.h" 
+#include "Renderer.h" // for dtor visibility by auto_ptr
+#include "Movie.h" 
 #include "ManualClock.h" // for composition
-#include "RunInfo.h" // For initialization.
+#include "RunResources.h" // For initialization.
 
 #include <memory> // for auto_ptr
 #include <string> 
@@ -57,7 +57,7 @@ namespace gnash {
 	class movie_definition;
 	class movie_root;
 	class MovieClip;
-	class character;
+	class DisplayObject;
 	class FuzzyPixel;
 	class VirtualClock;
 	class rgba;
@@ -73,7 +73,8 @@ class TestingRenderer
 
 public:
 
-	TestingRenderer(std::auto_ptr<render_handler> renderer, const std::string& name)
+	TestingRenderer(boost::shared_ptr<Renderer> renderer,
+            const std::string& name)
 		:
 		_name(name),
 		_renderer(renderer)
@@ -82,16 +83,14 @@ public:
 	const std::string& getName() const { return _name; }
 
 	/// Return the underlying render handler
-	render_handler& getRenderer() const { return *_renderer; }
+    boost::shared_ptr<Renderer> getRenderer() const { return _renderer; }
 
 private:
 
 	std::string _name;
-	std::auto_ptr<render_handler> _renderer;
+    boost::shared_ptr<Renderer> _renderer;
 };
 
-typedef boost::shared_ptr<TestingRenderer> TestingRendererPtr;
-
 /// An utility class for testing movie playback
 //
 /// This is a just born implementation and doesn't
@@ -138,20 +137,20 @@ public:
 	///
 	geometry::SnappingRanges2d<int> getInvalidatedRanges() const;
 
-	/// Find a character in the display list of a sprite by name.
+	/// Find a DisplayObject in the display list of a sprite by name.
 	//
-	/// Return NULL if there's no character with that name in
+	/// Return NULL if there's no DisplayObject with that name in
 	/// the sprite's display list.
 	///
-	const character* findDisplayItemByName(const MovieClip& mc,
+	const DisplayObject* findDisplayItemByName(const MovieClip& mc,
 			const std::string& name);
 
-	/// Find a character in the display list of a sprite by depth.
+	/// Find a DisplayObject in the display list of a sprite by depth.
 	//
-	/// Return NULL if there's no character at that depth in
+	/// Return NULL if there's no DisplayObject at that depth in
 	/// the sprite's display list.
 	///
-	const character* findDisplayItemByDepth(const MovieClip& mc,
+	const DisplayObject* findDisplayItemByDepth(const MovieClip& mc,
 			int depth);
 
 	/// Get the topmost sprite instance of this movie
@@ -225,7 +224,7 @@ public:
 	void releaseKey(key::code k);
 
 	/// Return true if the currently active 
-	/// character is over a character that
+	/// DisplayObject is over a DisplayObject that
 	/// handles mouse events
 	bool isMouseOverMouseEntity();
 
@@ -251,7 +250,7 @@ public:
 	//
 	/// Pixel checking will be supported as long as a testing-capable render handler
 	/// was compiled in. Testing-capable means capable of off-screen rendering, which
-	/// is implementing the render_handler::initTestBuffer method.
+	/// is implementing the Renderer::initTestBuffer method.
 	///
 	bool canTestRendering() const { return ! _testingRenderers.empty(); }
 
@@ -308,20 +307,23 @@ private:
 	/// @param invalidated
 	///	The invalidated ranges as computed by the core lib.
 	///
-	void render(render_handler& renderer, InvalidatedRanges& invalidated);
+	void render(boost::shared_ptr<Renderer> renderer,
+            InvalidatedRanges& invalidated);
 
-	/// Add a testing renderer to the list, initializing it with current viewport size
-	void addTestingRenderer(std::auto_ptr<render_handler> h, const std::string& name);
+	/// Add a testing renderer to the list, initializing it with current
+    //viewport size
+	void addTestingRenderer(boost::shared_ptr<Renderer> h,
+            const std::string& name);
 
 	gnash::movie_root* _movie_root;
 
 	gnash::movie_definition* _movie_def;
 
-	gnash::movie_instance* _movie;
+	gnash::Movie* _movie;
 
     boost::shared_ptr<sound::sound_handler> _sound_handler;
 
-    std::auto_ptr<RunInfo> _runInfo;
+    std::auto_ptr<RunResources> _runResources;
 	/// Current pointer position - X ordinate
 	int _x;
 
@@ -340,13 +342,9 @@ private:
 	/// information out.
 	InvalidatedRanges _invalidatedBounds;
 
-	/// I'd use ptr_list here, but trying not to spread
-	/// boost 1.33 requirement for the moment.
-	/// Still, I'd like to simplify things...
-	/// is shared_ptr fine ?
-	typedef std::vector< TestingRendererPtr > TRenderers;
+    typedef std::vector<TestingRenderer> TestingRenderers;
 
-	std::vector< TestingRendererPtr > _testingRenderers;
+	TestingRenderers _testingRenderers;
 
 	// When true, pass world invalidated ranges
 	// to the renderer(s) at ::render time.
