@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,8 +21,8 @@
 #define INPUT_FILENAME "DefineTextTest.swf"
 
 #include "MovieTester.h"
-#include "sprite_instance.h"
-#include "character.h"
+#include "MovieClip.h"
+#include "DisplayObject.h"
 #include "DisplayList.h"
 #include "log.h"
 #include "VM.h"
@@ -44,24 +44,24 @@ main(int /*argc*/, char** /*argv*/)
 	gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 	dbglogfile.setVerbosity(1);
 
-	sprite_instance* root = tester.getRootMovie();
+	MovieClip* root = tester.getRootMovie();
 	assert(root);
 
 	check_equals(root->get_frame_count(), 3);
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 	check_equals(root->get_current_frame(), 0);
 
 	rgba white(255,255,255,255);
 	rgba red(255,0,0,255);
 	rgba green(0,255,0,255);
 
-	geometry::Point2d cXmm(220,327); // character X center
-	geometry::Point2d cXum(220,440); // character X underline/middle
+	geometry::Point2d cXmm(220,327); // DisplayObject X center
+	geometry::Point2d cXum(220,440); // DisplayObject X underline/middle
 
-	geometry::Point2d cOmr(135,326); // character O middle/right
-	geometry::Point2d cOml(21,330); // character O middle/left
-	geometry::Point2d cOmm(77,327); // character O center
-	geometry::Point2d cOum(78,440); // character O underline/middle
+	geometry::Point2d cOmr(135,326); // DisplayObject O middle/right
+	geometry::Point2d cOml(21,330); // DisplayObject O middle/left
+	geometry::Point2d cOmm(77,327); // DisplayObject O center
+	geometry::Point2d cOum(78,440); // DisplayObject O underline/middle
 
 
 	tester.advance(); // first frame only contains dejagnu
