@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,11 +21,11 @@
 #define INPUT_FILENAME "DrawingApiTest.swf"
 
 #include "MovieTester.h"
-#include "sprite_instance.h"
-#include "character.h"
-#include "dlist.h"
+#include "MovieClip.h"
+#include "DisplayObject.h"
+#include "DisplayList.h"
 #include "log.h"
-#include "gnash.h" // for gnash::key::code
+#include "GnashKey.h" // for gnash::key::code
 
 #include "check.h"
 #include <string>
@@ -77,13 +77,18 @@ main(int /*argc*/, char** /*argv*/)
 	gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 	dbglogfile.setVerbosity(1);
 
-	sprite_instance* root = tester.getRootMovie();
+	MovieClip* root = tester.getRootMovie();
 	assert(root);
 
-	check_equals(root->get_frame_count(), 1);
+	check_equals(root->get_frame_count(), 2);
 	check_equals(root->get_current_frame(), 0);
-
-	//tester.advance();
+	
+	// advance to the second frame, first frame is an empty frame.
+	//
+	// Don't use the Dejagnuclip for this test, the EditText box in it
+	// would violates many checks(invalidatedBound...) here.
+	//
+	tester.advance();
 
 	rgba white(255, 255, 255, 255);
 	rgba blue(0, 0, 255, 255);
@@ -119,7 +124,7 @@ main(int /*argc*/, char** /*argv*/)
 
 	// Inside cyan clockwise fill
 	tester.movePointerTo(190, 112);
-	// this fails since Udo's rewrite of shape_character_def::point_test
+	// this fails since Udo's rewrite of Shape_def::point_test
 	// won't turn it into an 'expected' change as reverting the point_test 
 	// would fix it.
 	check(tester.isMouseOverMouseEntity());
@@ -364,7 +369,14 @@ main(int /*argc*/, char** /*argv*/)
 	// Check hitdetector bounds and reactions on mouse movement 
 	//--------------------------------------------------------------
 
-	typedef struct IntPoint { int x; int y; IntPoint(int nx, int ny) : x(nx), y(ny) {} };
+	struct IntPoint {
+        int x;
+        int y;
+        IntPoint(int nx, int ny)
+            :
+            x(nx), y(ny)
+        {}
+    };
 
 	IntPoint c1s(6, 346); // top-right of first yellow circle (in when small)
 	IntPoint c1b(16, 329); // top-right of first yellow circle (in when big, out when small)
@@ -468,7 +480,7 @@ main(int /*argc*/, char** /*argv*/)
 	tester.movePointerTo(50, 50); // out of any drawing
 	tester.advance();
 
-        // Inside bottom-left blue fill
+    // Inside bottom-left blue fill
 	check_pixel(60, 215, 2, white, 2); 
 	tester.movePointerTo(60, 215); tester.advance(); // move the mask over it...
 	check_pixel(60, 215, 2, blue, 2); 
@@ -723,7 +735,7 @@ main(int /*argc*/, char** /*argv*/)
 	// Upper-On_09_stroke
 	// AGG fails rendering a white stroke on the red background.
 	// Cairo succeeds.
-	xcheck_pixel(xo + (20*scale), yo + (15*scale), 2, red, 2);
+	check_pixel(xo + (20*scale), yo + (15*scale), 2, red, 2);
 	// Upper-Right
 	check_pixel(xo + (35*scale), yo + (15*scale), 2, red, 2);
 
@@ -744,9 +756,117 @@ main(int /*argc*/, char** /*argv*/)
 	// On the 0-9 stroke, out of fill 
 	// AGG fails rendering a white stroke on the red background.
 	// Cairo succeeds.
-	xcheck_pixel(xo + (20*scale), yo + (25*scale), 3, white, 2);
+	check_pixel(xo + (20*scale), yo + (25*scale), 3, white, 2);
 
-	//----------------------------------------------------------
+    // Test picture 3
+	tester.pressKey(gnash::key::_3); tester.advance();
+
+    const int w = 100, h = 100;
+
+    int x = 20, y = 20;
+
+    // Shape 1
+
+    // Bottom left corner (green line).
+    check_pixel(x, y + h, 2, green, 2);
+    // Bottom left fill (red)
+    check_pixel(x + 20, y + 60, 2, red, 2);
+    // Top right fill (red)
+    check_pixel(x + 80, y + 20, 2, red, 2);
+    // Dead centre fill (red)
+    xcheck_pixel(x + w / 2, y + h / 2, 2, red, 2);
+    // Top right corner (blue line)
+    check_pixel(x + w, y, 2, blue, 2);
+
+    // Shape 2
+    
+    x += 200;
+
+    // Bottom left corner (green line).
+    check_pixel(x, y + h, 2, green, 2);
+    // Bottom left fill (red)
+    check_pixel(x + 20, y + 60, 2, red, 2);
+    // Top right fill (red)
+    xcheck_pixel(x + 80, y + 20, 2, red, 2);
+    // Dead centre fill (red)
+    xcheck_pixel(x + w / 2, y + h / 2, 2, red, 2);
+    // Top right corner (blue line, is correct to be over black line ending)
+    check_pixel(x + w, y, 2, blue, 2);
+    // Top centre (black line)
+    check_pixel(x + w / 2, y, 2, black, 2);
+
+    // Shape 3
+    
+    x += 200;
+
+    // Bottom left corner (black line).
+    check_pixel(x, y + h, 2, black, 2);
+    // Bottom left fill (none)
+    check_pixel(x + 20, y + 60, 2, white, 2);
+    // Top right fill (none)
+    check_pixel(x + 80, y + 20, 2, white, 2);
+    // Dead centre fill (none)
+    check_pixel(x + w / 2, y + h / 2, 2, white, 2);
+    // Top right corner (nothing)
+    check_pixel(x + w, y, 2, white, 2);
+
+    // Shape 4
+    
+    x = 20;
+    y += 150;
+
+    // Should look the same as Shape 3
+
+    // Bottom left corner (black line).
+    check_pixel(x, y + h, 2, black, 2);
+    // Bottom left fill (none)
+    check_pixel(x + 20, y + 60, 2, white, 2);
+    // Top right fill (none)
+    check_pixel(x + 80, y + 20, 2, white, 2);
+    // Dead centre fill (none)
+    check_pixel(x + w / 2, y + h / 2, 2, white, 2);
+    // Top right corner (nothing)
+    check_pixel(x + w, y, 2, white, 2);
+
+    // Shape 5
+
+    x += 200;
+
+    // Bottom left corner (black line).
+    check_pixel(x, y + h, 2, black, 2);
+    // Bottom left fill (black)
+    check_pixel(x + 20, y + 60, 2, black, 2);
+    // Top right fill (none)
+    check_pixel(x + 80, y + 20, 2, white, 2);
+    // Dead centre (black line)
+    check_pixel(x + w / 2, y + h / 2, 2, black, 2);
+    // Top right corner (nothing)
+    check_pixel(x + w, y, 2, white, 2);
+    
+    // Shape 6
+
+    x += 200;
+
+    // NB: the rendering of this shape is not consistent
+    // across different versions of the pp. It doesn't seem
+    // like a sane case, so there is probably no need
+    // to worry about compatibility.
+
+    // Bottom left corner (yellow line).
+    check_pixel(x, y + h, 2, yellow, 2);
+    // Bottom left fill (green)
+    check_pixel(x + 20, y + 60, 2, green, 2);
+
+    // The following tests are possibly meaningless:
+    
+    // Top right fill (blue)
+    check_pixel(x + 80, y + 20, 2, blue, 2);
+    // Dead centre (yellow line)
+    check_pixel(x + w / 2, y + h / 2, 2, yellow, 2);
+    // Top right corner (yellow line)
+    check_pixel(x + w, y, 2, yellow, 2);
+
+    //----------------------------------------------------------
 	// TODO: check startDrag/stopDrag on the hit detector
 	// (hit 'd' key to toggle)
 	//----------------------------------------------------------
