@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,8 +21,8 @@
 #define INPUT_FILENAME "RollOverOutTest.swf"
 
 #include "MovieTester.h"
-#include "sprite_instance.h"
-#include "character.h"
+#include "MovieClip.h"
+#include "DisplayObject.h"
 #include "DisplayList.h"
 #include "log.h"
 
@@ -42,23 +42,23 @@ main(int /*argc*/, char** /*argv*/)
 	gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 	dbglogfile.setVerbosity(1);
 
-	sprite_instance* root = tester.getRootMovie();
+	MovieClip* root = tester.getRootMovie();
 	assert(root);
 
 	check_equals(root->get_frame_count(), 4);
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 	check_equals(root->get_current_frame(), 0);
 
 	tester.advance();  // advance to the second frame.
 
-	const character* mc1 = tester.findDisplayItemByName(*root, "square1");
+	const DisplayObject* mc1 = tester.findDisplayItemByName(*root, "square1");
 	check(mc1);
-	const character* mc2 = tester.findDisplayItemByName(*root, "square2");
+	const DisplayObject* mc2 = tester.findDisplayItemByName(*root, "square2");
 	check(mc2);
 
-	check_equals(mc1->get_visible(), true);
-	check_equals(mc2->get_visible(), false);
-	check_equals(root->get_play_state(), sprite_instance::STOP);
+	check_equals(mc1->visible(), true);
+	check_equals(mc2->visible(), false);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_STOP);
 	check_equals(root->get_current_frame(), 1);
 
 	// we're in stop mode, so advance should not advance anything
@@ -73,7 +73,7 @@ main(int /*argc*/, char** /*argv*/)
 	tester.movePointerTo(60, 60);
 	tester.advance();
 	check_equals(root->get_current_frame(), 2);
-	check_equals(root->get_play_state(), sprite_instance::STOP);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_STOP);
 	tester.advance();
 	check_equals(root->get_current_frame(), 2);
 
