@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,8 +21,8 @@
 #define INPUT_FILENAME "replace_shapes1test.swf"
 
 #include "MovieTester.h"
-#include "sprite_instance.h"
-#include "character.h"
+#include "MovieClip.h"
+#include "DisplayObject.h"
 #include "DisplayList.h"
 #include "log.h"
 
@@ -56,22 +56,22 @@ main(int /*argc*/, char** /*argv*/)
 	Range2d<int> greenRange(130,330,190,390);
 
 	Ranges invalidated;
-	sprite_instance* root = tester.getRootMovie();
+	MovieClip* root = tester.getRootMovie();
 	assert(root);
 
 	// FRAME 1 (start)
 
 	check_equals(root->get_frame_count(), 4);
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 	check_equals(root->get_current_frame(), 0);
 	check_equals(root->getDisplayList().size(), 1);  // dejagnu clip
 	invalidated = tester.getInvalidatedRanges();
 	check( invalidated.contains(76, 4) ); // the "-xtrace enabled-" label...
 
-	tester.advance(); // FRAME 2, place character
+	tester.advance(); // FRAME 2, place DisplayObject
 	invalidated = tester.getInvalidatedRanges();
 	
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 	check_equals(root->get_current_frame(), 1);
 	check_equals(root->getDisplayList().size(), 2); // dejagnu + red char
 
@@ -90,10 +90,10 @@ main(int /*argc*/, char** /*argv*/)
 	check_pixel(130, 296, 2, white, 2); // Top
 	check_pixel(130, 364, 2, white, 2); // Bottom
 
-	tester.advance(); // FRAME 3, replace character
+	tester.advance(); // FRAME 3, replace DisplayObject
 	invalidated = tester.getInvalidatedRanges();
 	
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_PLAY);
 	check_equals(root->get_current_frame(), 2);
 	check_equals(root->getDisplayList().size(), 2); // dejagnu + green char 
 
@@ -119,7 +119,7 @@ main(int /*argc*/, char** /*argv*/)
 	tester.advance(); // FRAME 4, jump to frame 2 and stop
 	invalidated = tester.getInvalidatedRanges();
 
-	check_equals(root->get_play_state(), sprite_instance::STOP);
+	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_STOP);
 	check_equals(root->get_current_frame(), 1);
 	check_equals(root->getDisplayList().size(), 2); // dejagnu + red char 
 
