@@ -28,16 +28,19 @@
 # include "gst/MediaHandlerGst.h"
 #endif
 
+#include <ios>
 #include <iostream>
 #include <cstdio>
 #include <cstdlib>
-#include <sys/time.h>
 #include <ctime>
 
 #ifdef ENABLE_NLS
 #include <locale>
 #endif
 
+#include "MovieFactory.h"
+#include "swf/TagLoadersTable.h"
+#include "swf/DefaultTagLoaders.h"
 #include "gettext.h"
 #include "ClockTime.h"
 #include "gnash.h"
@@ -69,6 +72,17 @@ extern "C"{
 #endif
 }
 
+#ifdef BOOST_NO_EXCEPTIONS
+
+namespace boost
+{
+	void throw_exception(std::exception const & e)
+	{
+		std::abort();
+	}
+}
+#endif
+
 // How many seconds to wait for a frame advancement 
 // before kicking the movie (forcing it to next frame)
 static const double waitforadvance = 5;
@@ -109,7 +123,7 @@ struct movie_data
 };
 
 static boost::intrusive_ptr<gnash::movie_definition> play_movie(
-        const std::string& filename, const RunInfo& runInfo);
+        const std::string& filename, const RunResources& runResources);
 
 static bool s_do_output = false;
 static bool s_stop_on_errors = true;
@@ -211,6 +225,8 @@ FsCommandExecutor execFsCommand;
 int
 main(int argc, char *argv[])
 {
+    std::ios::sync_with_stdio(false);
+
     /// Initialize gnash core library
     gnashInit();
 
@@ -260,7 +276,7 @@ main(int argc, char *argv[])
         dbglogfile.setVerbosity();
     }
 
-    while ((c = getopt (argc, argv, ":hwvapr:gf:d:")) != -1) {
+    while ((c = getopt (argc, argv, ":hwvapr:gf:d:n")) != -1) {
 	switch (c) {
 	  case 'h':
 	      usage (argv[0]);
@@ -281,6 +297,9 @@ main(int argc, char *argv[])
 #else
               log_error (_("The debugger has been disabled at configuration time"));
 #endif
+	  case 'n':
+	      dbglogfile.setNetwork(true); 
+	      break;
 	  case 'a':
 #if VERBOSE_ACTION
 	      dbglogfile.setActionDump(true); 
@@ -341,36 +360,41 @@ main(int argc, char *argv[])
     }
 
 
-#ifdef USE_FFMPEG
-    std::auto_ptr<media::MediaHandler> handler(
-            new gnash::media::ffmpeg::MediaHandlerFfmpeg() );
-#elif defined(USE_GST)
-    std::auto_ptr<media::MediaHandler> handler(
-            new gnash::media::gst::MediaHandlerGst() );
-#else
-    std::cerr << "Neither SOUND_SDL nor SOUND_GST defined" << std::endl;
-    exit(1);
-#endif
-    gnash::media::MediaHandler::set(handler);
+    std::auto_ptr<gnash::media::MediaHandler> mediaHandler;
+    boost::shared_ptr<sound::sound_handler> soundHandler;
 
-    boost::shared_ptr<sound::sound_handler> soundHandler(
-            new sound::NullSoundHandler());
+#ifdef USE_FFMPEG
+    mediaHandler.reset( new gnash::media::ffmpeg::MediaHandlerFfmpeg() );
+#elif defined(USE_GST)
+    mediaHandler.reset( new gnash::media::gst::MediaHandlerGst() );
+#endif
+
+    if ( mediaHandler.get() )
+    {
+        gnash::media::MediaHandler::set(mediaHandler);
+        soundHandler.reset( new sound::NullSoundHandler() );
+    }
 
     boost::shared_ptr<StreamProvider> sp(new StreamProvider);
 
     std::vector<movie_data>	data;
+        
+
+    boost::shared_ptr<SWF::TagLoadersTable> loaders(new SWF::TagLoadersTable());
+    addDefaultLoaders(*loaders);
 
     // Play through all the movies.
     for (std::vector<std::string>::const_iterator i = infiles.begin(), 
             e = infiles.end(); i != e; ++i)
     {
 
-        RunInfo runInfo(*i);
-        runInfo.setSoundHandler(soundHandler);
-        runInfo.setStreamProvider(sp);
+        RunResources runResources(*i);
+        runResources.setSoundHandler(soundHandler);
+        runResources.setStreamProvider(sp);
+        runResources.setTagLoaders(loaders);
 
 	    boost::intrusive_ptr<gnash::movie_definition> m =
-            play_movie(*i, runInfo);
+            play_movie(*i, runResources);
 	    if (!m) {
 	        if (s_stop_on_errors) {
 		    // Fail.
@@ -399,7 +423,7 @@ main(int argc, char *argv[])
 //
 // Return the movie definition.
 boost::intrusive_ptr<gnash::movie_definition>
-play_movie(const std::string& filename, const RunInfo& runInfo)
+play_movie(const std::string& filename, const RunResources& runResources)
 {
     boost::intrusive_ptr<gnash::movie_definition> md;
 
@@ -411,7 +435,7 @@ play_movie(const std::string& filename, const RunInfo& runInfo)
       {
          std::auto_ptr<IOChannel> in (
                  noseek_fd_adapter::make_stream(fileno(stdin)) );
-         md = gnash::create_movie(in, filename, runInfo, false);
+         md = MovieFactory::makeMovie(in, filename, runResources, false);
       }
       else
       {
@@ -428,7 +452,7 @@ play_movie(const std::string& filename, const RunInfo& runInfo)
              log_debug(_("%s appended to local sandboxes"), path.c_str());
 #endif
          }
-         md = gnash::create_library_movie(url, runInfo, NULL, false);
+         md = MovieFactory::makeMovie(url, runResources, NULL, false);
       }
     }
     catch (GnashException& ge)
@@ -452,7 +476,7 @@ play_movie(const std::string& filename, const RunInfo& runInfo)
 
     // Use a clock advanced at every iteration to match exact FPS speed.
     ManualClock cl;
-    gnash::movie_root m(*md, cl, runInfo);
+    gnash::movie_root m(*md, cl, runResources);
     
     // Register processor to receive ActionScript events (Mouse, Stage
     // System etc).
@@ -461,7 +485,7 @@ play_movie(const std::string& filename, const RunInfo& runInfo)
 
     md->completeLoad();
 
-    std::auto_ptr<movie_instance> mi ( md->create_movie_instance() );
+    std::auto_ptr<Movie> mi ( md->createMovie() );
 
     m.setRootMovie( mi.release() );
     if ( quitrequested )  // setRootMovie would execute actions in first frame
@@ -542,7 +566,7 @@ play_movie(const std::string& filename, const RunInfo& runInfo)
                 fprintf(stderr, "Kicking movie after %g seconds in STOP mode, kick ct = %d\n", waitforadvance, kick_count);
                 fflush(stderr);
                 m.goto_frame(last_frame + 1);
-                m.set_play_state(gnash::MovieClip::PLAY);
+                m.set_play_state(gnash::MovieClip::PLAYSTATE_PLAY);
                 kick_count++;
 
                 if (kick_count > 10) {
